########################################################################
# Talih, M. (2012).
# A reference-invariant health disparity index based on Renyi divergence.
# Annals of Applied Statistics. Accepted for publication.
########################################################################

########################################################################
# Code is backward compatible with that used in:
#
# Borrell, L.N., M. Talih (2012). 
# Examining periodontal disease disparities among U.S. adults 20 years of age and older: 
# NHANES III (1988-1994) and NHANES 1999-2004. 
# Public Health Reports, 127, pp.497-506.
#
# Borrell, L.N., M. Talih (2011). 
# A Symmetrized Theil index measure of health disparities: 
# An example using dental caries in U.S. children and adolescents. 
# Statistics in Medicine, 30 (3), pp. 277-90. 
# First published online 5 November 2010. 
# DOI: 10.1002/sim.4114.
#
#########################################################################

###############################
# Last modified 11 December 2012
###############################

#######################################################################################################
# You can modify the output formatting by editing the two main wrapper functions:
# theil.decomposition.bin
# theil.decomposition.out
#######################################################################################################

theil.decomposition.bin =     # Print method for binary outcome variable,
                              # with exactly one categorical variable to be specified in 'grp.var'
  function(type="T",          # String indicating index to work with: T=Theil, L=MLD, S=STI, 
                              # REW=Renyi-equally-weighted, RPW = Renyi-population-weighted, SREW or SRPW = Symmetrized Renyi.
	   alpha=0.5,	      # Inequality aversion parameter for Renyi family of indices
           grp.var="",        # Variable label from grouping variable to use.
           w.var="",          # Variable label for weighting variable to use.
           bin.var="",        # Variable label for binary individual-level outcome.
           response.grp=NULL, # Either a binary variable name bin.var or a logical condition must be specified (eg. >0 vs <=0).
           data=NULL,         # Either a survey design object or a data frame must be specified.
           w.boot.data=NULL,  # For the scaled bootstrap method, a pre-computed set of alternative weights.
           w.BRR.data=NULL,   # For the BRR method, a pre-computed set of alternative weights.
           fay.rho=0,         # Fay's parameter for BRR method.
           na.rm=TRUE,        # What to do with missing values in x or w variables.
           digits=4,          # Significant digits to keep in output.
           svobj=NULL,        # Either a survey design object or a data frame must be specified.
                              # If both are supplied, data frame is over-ridden by survey design object.
           out.file="",       # Path to output file for tabulated indices and their 95% CIs.
           print.prev=TRUE,   # whether to print contingency table, test of association, and prevalence estimates
           store.runs=FALSE,  # whether to store BRR and bootstrap runs in workspace for later use.
           ...)               # Additional parameters, as needed, passed to 'svytbl' summary method.

{
  ### Preliminary set-up:

  if (length(grp.var) != 1) {
    stop("Experimental print method: Exactly one categorical variable name must be specified in 'grp.var'!")
  }
  if (identical(bin.var, "") & is.null(response.grp)) {
    stop("Either a binary variable name or a logical condition must be specified.")
  }
  if (is.null(svobj)) {
    if (is.null(data)) {
      stop("Either a survey design object or a data frame must be specified.")
    } else {
      if (!is.element(w.var, names(data))) {
        stop(paste("Variable", w.var, "not found"))
      } else {
        w = data[,w.var]
      }
    }
  } else {
    if (!is.null(data)) {
      warning("Supplied data frame over-ridden by survey design object.")
    }
    data = svobj$variables
    w = 1/svobj$prob
  }
  if (!is.element(grp.var, names(data))) {
    stop(paste("Variable", grp.var, "not found"))
  }
  if (!is.element(bin.var, names(data))) {
    if (is.null(response.grp)) { 
      stop(paste("Variable", bin.var, "not found"))
    } else {
      warning("Supplied binary variable ", bin.var, " not found but ignored")
    }
  } else {
    if (!is.null(response.grp)) {
      warning("Supplied 'response.grp' overridden by binary variable ", bin.var, " == 1")
    }
    response.grp = as.numeric(data[, bin.var]) == 1
  }
      
  ## Switch for special limiting cases alpha tends to 0 or 1:

  if (isTRUE(all.equal(alpha, 1))) {
     if (type == "RPW") {
     	type = "L"	# MLD is limiting case when alpha tends to 1
     } else if (type == "SRPW") {
     	type = "S" 	# STI is limiting case when alpha tends to 1
     } else if (type == "REW") {
     	type = "RL" 	# "equally-weighted" MLD is limiting case when alpha tends to 1
     } else if (type == "SREW") {
     	type = "RS" 	# "equally-weighted" STI is limiting case when alpha tends to 1
     }
  } else if (isTRUE(all.equal(alpha, 0))) {
     if (type == "RPW") {
     	type = "T"	# TI is limiting case when alpha tends to 0
     } else if (type == "SRPW") {
     	type = "S" 	# STI is limiting case when alpha tends to 0
     } else if (type == "REW") {
     	type = "RT" 	# "equally-weighted" TI is limiting case when alpha tends to 0
     } else if (type == "SREW") {
     	type = "RS" 	# "equally-weighted" STI is limiting case when alpha tends to 0
     }
  }

  ## Index label for report printout:

  if (type == "T") {
    name = "Theil"
  } else if (type == "L") {
    name = "MLD"
  } else if (type == "S") {
    name = "STI"
  } else if (type == "RT") {
    name = "EW-Theil"
  } else if (type == "RL") {
    name = "EW-MLD"
  } else if (type == "RS") {
    name = "EW-STI"
  } else if (type == "RPW") {
    name = paste("PW-RI, alpha=", alpha)
  } else if (type == "SRPW") {
    name = paste("PW-SRI, alpha=", alpha)
  } else if (type == "REW") {
    name = paste("EW-RI, alpha=", alpha)
  } else if (type == "SREW") {
    name = paste("EW-SRI, alpha=", alpha)
  }

  ## Any NA in grouping variable?
  anyNA = any(is.na(data[,grp.var]))

  ## group levels
  grp.levels = levels(data[,grp.var])
     
  ## Initialize design-based SEs and CIs for index:

  BJ.se = NA; BRR.se = NA; boot.se = NA; BJ.ci = c(NA, NA); BRR.ci = c(NA, NA); boot.ci = c(NA, NA)
  if (anyNA) {
    BJ.seNA = NA; BRR.seNA = NA; boot.seNA = NA; BJ.ciNA = c(NA, NA); RR.ciNA = c(NA, NA); boot.ciNA = c(NA, NA)
  }

  ## Counts:

  h.n = sapply(grp.levels, function(g) sum(w[(response.grp | !response.grp) & data[,grp.var] == g], na.rm=na.rm))
  h.s = sapply(grp.levels, function(g) sum(w[response.grp & data[,grp.var] == g], na.rm=na.rm))
  h.m = ifelse(h.n > 0, h.s/h.n, 0)
  cond.n = (h.n > 0)
  h.n = h.n[cond.n] # trim empty groups
  h.s = h.s[cond.n]
  h.m = h.m[cond.n]
  all.n = sum(w[response.grp | !response.grp], na.rm=na.rm)
  all.s = sum(w[response.grp], na.rm=na.rm)

  ## Compute between-group index for non-NA categories only

  if (type == "T") {
    ind = sum(ifelse(h.s > 0, h.s*log(h.m), 0))/sum(h.s) - log(sum(h.s)) + log(sum(h.n))
  } else if (type == "L") {
    ind = -sum(h.n*log(h.m))/sum(h.n) + log(sum(h.s)) - log(sum(h.n))
  } else if (type == "S") {
    ind = (sum(ifelse(h.s > 0, h.s*log(h.m), 0))/sum(h.s) - sum(h.n*log(h.m))/sum(h.n))/2
  } else if (type == "RT") {
    ind = sum(ifelse(h.s > 0, h.m*log(h.m), 0))/sum(h.m) - log(sum(h.m)) + log(length(h.s))
  } else if (type == "RL") {
    ind = -sum(log(h.m))/length(h.s) + log(sum(h.m)) - log(length(h.s))
  } else if (type == "RS") {
    ind = (sum(ifelse(h.s > 0, h.m*log(h.m), 0))/sum(h.m) - sum(log(h.m))/length(h.s))/2
  } else if (type == "RPW") {
    ind = (alpha*log(sum(h.n)) + (1-alpha)*log(sum(h.s)) - log(sum(h.n*h.m^(1-alpha))))/(alpha*(1-alpha))
  } else if (type == "SRPW") {
    ind = (log(sum(h.n)) + log(sum(h.s)) - log(sum(h.n*h.m^(1-alpha))) - log(sum(h.n*h.m^alpha)))/(2*alpha*(1-alpha))
  } else if (type == "REW") {
    ind = (alpha*log(length(h.s)) + (1-alpha)*log(sum(h.m)) - log(sum(h.m^(1-alpha))))/(alpha*(1-alpha))
  } else if (type == "SREW") {
    ind = (log(length(h.s)) + log(sum(h.m)) - log(sum(h.m^(1-alpha))) - log(sum(h.m^alpha)))/(2*alpha*(1-alpha))
  }

  if (anyNA) { # redo for NA vs non-NA
    h.nNA = c(sum(h.n), all.n - sum(h.n))
    anyNA = (h.nNA[2] > 0)
  }

  if (anyNA) { # proceed only if NA-group non-empty
    h.sNA = c(sum(h.s), all.s - sum(h.s))
    h.mNA = h.sNA/h.nNA

    if (type == "T") {
      indNA = sum(ifelse(h.sNA > 0, h.sNA*log(h.mNA), 0))/sum(h.sNA) - log(sum(h.sNA)) + log(sum(h.nNA))
    } else if (type == "L") {
      indNA = -sum(h.nNA*log(h.mNA))/sum(h.nNA) + log(sum(h.sNA)) - log(sum(h.nNA))
    } else if (type == "S") {
      indNA = (sum(ifelse(h.sNA > 0, h.sNA*log(h.mNA), 0))/sum(h.sNA) - sum(h.nNA*log(h.mNA))/sum(h.nNA))/2
    } else if (type == "RT") {
      indNA = sum(ifelse(h.sNA > 0, h.mNA*log(h.mNA), 0))/sum(h.mNA) - log(sum(h.mNA)) + log(length(h.sNA))
    } else if (type == "RL") {
      indNA = -sum(log(h.mNA))/length(h.sNA) + log(sum(h.mNA)) - log(length(h.sNA))
    } else if (type == "RS") {
      indNA = (sum(ifelse(h.sNA > 0, h.mNA*log(h.mNA), 0))/sum(h.mNA) - sum(log(h.mNA))/length(h.sNA))/2
    } else if (type == "RPW") {
      indNA = (alpha*log(sum(h.nNA)) + (1-alpha)*log(sum(h.sNA)) - log(sum(h.nNA*h.mNA^(1-alpha))))/(alpha*(1-alpha))
    } else if (type == "SRPW") {
      indNA = (log(sum(h.nNA)) + log(sum(h.sNA)) - log(sum(h.nNA*h.mNA^(1-alpha))) - log(sum(h.nNA*h.mNA^alpha)))/(2*alpha*(1-alpha))
    } else if (type == "REW") {
      indNA = (alpha*log(length(h.sNA)) + (1-alpha)*log(sum(h.mNA)) - log(sum(h.mNA^(1-alpha))))/(alpha*(1-alpha))
    } else if (type == "SREW") {
      indNA = (log(length(h.sNA)) + log(sum(h.mNA)) - log(sum(h.mNA^(1-alpha))) - log(sum(h.mNA^alpha)))/(2*alpha*(1-alpha))
    }
  }

  if (print.prev) {

    ## Conditional vs unconditional relative frequencies (for later use in printout):
  
    tbl = rbind(h.s/sum(h.s), h.n/sum(h.n))
    row.names(tbl) = c("conditional", "unconditional")
    if (anyNA) {
      tbl2 = cbind(c(sum(h.s)/all.s, sum(h.n)/all.n), c(1 - sum(h.s)/all.s, 1 - sum(h.n)/all.n))
      row.names(tbl2) = c("conditional", "unconditional")
      dimnames(tbl2)[[2]] = c("<non-NA>", "<NA>")
    }

    ## Printout of overall prevalence for report:
  
    cat("\n - Overall prevalence of", bin.var, "= 'Yes': ", round(all.s/all.n, digits), "\n\n")
  
  }

  ## clean-up
  rm(h.n, h.s, h.m, cond.n, all.n, all.s)
  if (anyNA) {
    rm(h.nNA, h.sNA, h.mNA)
  }

  ## Taylor linearization method for SE:

  if (!is.null(svobj)) {
    BJ.se = between.BJ.se(type=type, alpha=alpha, xlab=bin.var, grp.var=grp.var, svobj=svobj, na.rm=na.rm)
    BJ.ci = ind + c(-1,1)*BJ.se*1.959964
    if (anyNA) {
      BJ.seNA = between.BJ.seNA(type=type, alpha=alpha, xlab=bin.var, grp.var=grp.var, svobj=svobj, na.rm=na.rm)
      BJ.ciNA = indNA + c(-1,1)*BJ.seNA*1.959964
    }
  }

  ## Scaled bootstrap method for SE:
  
  if (!is.null(w.boot.data)) {
    B = ncol(w.boot.data)
    boot = vector("numeric", B)
    if (anyNA) {
      bootNA = vector("numeric", B)
    }
    for (i in 1:B) {
      h.n = sapply(grp.levels, function(g) sum(w.boot.data[(response.grp | !response.grp) & data[,grp.var] == g, i], na.rm=na.rm))
      h.s = sapply(grp.levels, function(g) sum(w.boot.data[response.grp & data[,grp.var] == g, i], na.rm=na.rm))
      h.m = ifelse(h.n > 0, h.s/h.n, 0)
      cond.n = (h.n > 0)
      h.n = h.n[cond.n] # trim empty groups
      h.s = h.s[cond.n]
      h.m = h.m[cond.n]
      all.n = sum(w.boot.data[response.grp | !response.grp], na.rm=na.rm)
      all.s = sum(w.boot.data[response.grp], na.rm=na.rm)

      if (type == "T") {
        boot[i] = sum(ifelse(h.s > 0, h.s*log(h.m), 0))/sum(h.s) - log(sum(h.s)) + log(sum(h.n))
      } else if (type == "L") {
        boot[i] = -sum(h.n*log(h.m))/sum(h.n) + log(sum(h.s)) - log(sum(h.n))
      } else if (type == "S") {
        boot[i] = (sum(ifelse(h.s > 0, h.s*log(h.m), 0))/sum(h.s) - sum(h.n*log(h.m))/sum(h.n))/2
      } else if (type == "RT") {
        boot[i] = sum(ifelse(h.s > 0, h.m*log(h.m), 0))/sum(h.m) - log(sum(h.m)) + log(length(h.s))
      } else if (type == "RL") {
        boot[i] = -sum(log(h.m))/length(h.s) + log(sum(h.m)) - log(length(h.s))
      } else if (type == "RS") {
        boot[i] = (sum(ifelse(h.s > 0, h.m*log(h.m), 0))/sum(h.m) - sum(log(h.m))/length(h.s))/2
      } else if (type == "RPW") {
      	boot[i] = (alpha*log(sum(h.n)) + (1-alpha)*log(sum(h.s)) - log(sum(h.n*h.m^(1-alpha))))/(alpha*(1-alpha))
      } else if (type == "SRPW") {
        boot[i] = (log(sum(h.n)) + log(sum(h.s)) - log(sum(h.n*h.m^(1-alpha))) - log(sum(h.n*h.m^alpha)))/(2*alpha*(1-alpha))
      } else if (type == "REW") {
        boot[i] = (alpha*log(length(h.s)) + (1-alpha)*log(sum(h.m)) - log(sum(h.m^(1-alpha))))/(alpha*(1-alpha))
      } else if (type == "SREW") {
        boot[i] = (log(length(h.s)) + log(sum(h.m)) - log(sum(h.m^(1-alpha))) - log(sum(h.m^alpha)))/(2*alpha*(1-alpha))
      }

      if (anyNA) { # redo for NA vs non-NA
        h.nNA = c(sum(h.n), all.n - sum(h.n))
        h.sNA = c(sum(h.s), all.s - sum(h.s))
        h.mNA = h.sNA/h.nNA

        if (type == "T") {
          bootNA[i] = sum(ifelse(h.sNA > 0, h.sNA*log(h.mNA), 0))/sum(h.sNA) - log(sum(h.sNA)) + log(sum(h.nNA))
        } else if (type == "L") {
          bootNA[i] = -sum(h.nNA*log(h.mNA))/sum(h.nNA) + log(sum(h.sNA)) - log(sum(h.nNA))
        } else if (type == "S") {
          bootNA[i] = (sum(ifelse(h.sNA > 0, h.sNA*log(h.mNA), 0))/sum(h.sNA) - sum(h.nNA*log(h.mNA))/sum(h.nNA))/2
        } else if (type == "RT") {
          bootNA[i] = sum(ifelse(h.sNA > 0, h.mNA*log(h.mNA), 0))/sum(h.mNA) - log(sum(h.mNA)) + log(length(h.sNA))
        } else if (type == "RL") {
          bootNA[i] = -sum(log(h.mNA))/length(h.sNA) + log(sum(h.mNA)) - log(length(h.sNA))
        } else if (type == "RS") {
          bootNA[i] = (sum(ifelse(h.sNA > 0, h.mNA*log(h.mNA), 0))/sum(h.mNA) - sum(log(h.mNA))/length(h.sNA))/2
        } else if (type == "RPW") {
       	  bootNA[i] = (alpha*log(sum(h.nNA)) + (1-alpha)*log(sum(h.sNA)) - log(sum(h.nNA*h.mNA^(1-alpha))))/(alpha*(1-alpha))
        } else if (type == "SRPW") {
          bootNA[i] = (log(sum(h.nNA)) + log(sum(h.sNA)) - log(sum(h.nNA*h.mNA^(1-alpha))) - log(sum(h.nNA*h.mNA^alpha)))/(2*alpha*(1-alpha))
        } else if (type == "REW") {
          bootNA[i] = (alpha*log(length(h.sNA)) + (1-alpha)*log(sum(h.mNA)) - log(sum(h.mNA^(1-alpha))))/(alpha*(1-alpha))
        } else if (type == "SREW") {
          bootNA[i] = (log(length(h.sNA)) + log(sum(h.mNA)) - log(sum(h.mNA^(1-alpha))) - log(sum(h.mNA^alpha)))/(2*alpha*(1-alpha))
        }
      }
    }

    boot.se = sqrt(var(boot, na.rm=na.rm))
    boot.ci = quantile(boot, c(0.025, 0.975), na.rm=na.rm)
    if (anyNA) {
      boot.seNA = sqrt(var(bootNA, na.rm=na.rm))
      boot.ciNA = quantile(bootNA, c(0.025, 0.975), na.rm=na.rm)
    }
  }

  ## clean-up:

  rm(h.n, h.s, h.m, cond.n, all.n, all.s)
  if (anyNA) {
    rm(bootNA, h.nNA, h.sNA, h.mNA)
  }

  ## Balanced Repeated Replication method for SE:
  
  if (!is.null(w.BRR.data)) {
    H = ncol(w.BRR.data)
    BRR = vector("numeric", H)
    if (anyNA) {
      BRRNA = vector("numeric", H)
    }
    
    for (i in 1:H) {
      h.n = sapply(grp.levels, function(g) sum(w.BRR.data[(response.grp | !response.grp) & data[,grp.var] == g, i], na.rm=na.rm))
      h.s = sapply(grp.levels, function(g) sum(w.BRR.data[response.grp & data[,grp.var] == g, i], na.rm=na.rm))
      h.m = ifelse(h.n > 0, h.s/h.n, 0)
      cond.n = (h.n > 0)
      h.n = h.n[cond.n] # trim empty groups
      h.s = h.s[cond.n]
      h.m = h.m[cond.n]
      all.n = sum(w.BRR.data[response.grp | !response.grp], na.rm=na.rm)
      all.s = sum(w.BRR.data[response.grp], na.rm=na.rm)

      if (type == "T") {
        BRR[i] = sum(ifelse(h.s > 0, h.s*log(h.m), 0))/sum(h.s) - log(sum(h.s)) + log(sum(h.n))
      } else if (type == "L") {
        BRR[i] = -sum(h.n*log(h.m))/sum(h.n) + log(sum(h.s)) - log(sum(h.n))
      } else if (type == "S") {
        BRR[i] = (sum(ifelse(h.s > 0, h.s*log(h.m), 0))/sum(h.s) - sum(h.n*log(h.m))/sum(h.n))/2
      } else if (type == "RT") {
        BRR[i] = sum(ifelse(h.s > 0, h.m*log(h.m), 0))/sum(h.m) - log(sum(h.m)) + log(length(h.s))
      } else if (type == "RL") {
        BRR[i] = -sum(log(h.m))/length(h.s) + log(sum(h.m)) - log(length(h.s))
      } else if (type == "RS") {
        BRR[i] = (sum(ifelse(h.s > 0, h.m*log(h.m), 0))/sum(h.m) - sum(log(h.m))/length(h.s))/2
      } else if (type == "RPW") {
      	BRR[i] = (alpha*log(sum(h.n)) + (1-alpha)*log(sum(h.s)) - log(sum(h.n*h.m^(1-alpha))))/(alpha*(1-alpha))
      } else if (type == "SRPW") {
        BRR[i] = (log(sum(h.n)) + log(sum(h.s)) - log(sum(h.n*h.m^(1-alpha))) - log(sum(h.n*h.m^alpha)))/(2*alpha*(1-alpha))
      } else if (type == "REW") {
        BRR[i] = (alpha*log(length(h.s)) + (1-alpha)*log(sum(h.m)) - log(sum(h.m^(1-alpha))))/(alpha*(1-alpha))
      } else if (type == "SREW") {
        BRR[i] = (log(length(h.s)) + log(sum(h.m)) - log(sum(h.m^(1-alpha))) - log(sum(h.m^alpha)))/(2*alpha*(1-alpha))
      }

      if (anyNA) { # redo for NA vs non-NA
        h.nNA = c(sum(h.n), all.n - sum(h.n))
        h.sNA = c(sum(h.s), all.s - sum(h.s))
        h.mNA = h.sNA/h.nNA

        if (type == "T") {
          BRRNA[i] = sum(ifelse(h.sNA > 0, h.sNA*log(h.mNA), 0))/sum(h.sNA) - log(sum(h.sNA)) + log(sum(h.nNA))
        } else if (type == "L") {
          BRRNA[i] = -sum(h.nNA*log(h.mNA))/sum(h.nNA) + log(sum(h.sNA)) - log(sum(h.nNA))
        } else if (type == "S") {
          BRRNA[i] = (sum(ifelse(h.sNA > 0, h.sNA*log(h.mNA), 0))/sum(h.sNA) - sum(h.nNA*log(h.mNA))/sum(h.nNA))/2
        } else if (type == "RT") {
          BRRNA[i] = sum(ifelse(h.sNA > 0, h.mNA*log(h.mNA), 0))/sum(h.mNA) - log(sum(h.mNA)) + log(length(h.sNA))
        } else if (type == "RL") {
          BRRNA[i] = -sum(log(h.mNA))/length(h.sNA) + log(sum(h.mNA)) - log(length(h.sNA))
        } else if (type == "RS") {
          BRRNA[i] = (sum(ifelse(h.sNA > 0, h.mNA*log(h.mNA), 0))/sum(h.mNA) - sum(log(h.mNA))/length(h.sNA))/2
        } else if (type == "RPW") {
          BRRNA[i] = (alpha*log(sum(h.nNA)) + (1-alpha)*log(sum(h.sNA)) - log(sum(h.nNA*h.mNA^(1-alpha))))/(alpha*(1-alpha))
        } else if (type == "SRPW") {
          BRRNA[i] = (log(sum(h.nNA)) + log(sum(h.sNA)) - log(sum(h.nNA*h.mNA^(1-alpha))) - log(sum(h.nNA*h.mNA^alpha)))/(2*alpha*(1-alpha))
        } else if (type == "REW") {
          BRRNA[i] = (alpha*log(length(h.sNA)) + (1-alpha)*log(sum(h.mNA)) - log(sum(h.mNA^(1-alpha))))/(alpha*(1-alpha))
        } else if (type == "SREW") {
          BRRNA[i] = (log(length(h.sNA)) + log(sum(h.mNA)) - log(sum(h.mNA^(1-alpha))) - log(sum(h.mNA^alpha)))/(2*alpha*(1-alpha))
        }
      }
    }

    BRR.se = sqrt(mean((BRR - ind)^2, na.rm=na.rm))/(1-fay.rho)
    BRR.ci = ind + c(-1,1)*BRR.se*1.959964
    if (anyNA) {
      BRR.seNA = sqrt(mean((BRRNA - indNA)^2, na.rm=na.rm))/(1-fay.rho)
      BRR.ciNA = indNA + c(-1,1)*BRR.seNA*1.959964
    }
  }
  
  ## clean-up:

  rm(h.n, h.s, h.m, cond.n, all.n, all.s)
  if (anyNA) {
    rm(BRRNA, h.nNA, h.sNA, h.mNA)
  }
  
  ## Contingency table print-out for report:

  if (print.prev) {

    ## Design-based contingency table test from 'svytable':
    
    if (!is.null(svobj)) { 
      fmla = as.formula(paste("~ ", paste(c(bin.var, grp.var), collapse= "+")))
      svtbl = svytable(fmla, svobj)
      fmla2 = as.formula(paste("~ ", paste(c(bin.var), collapse= "+")))
      svtbl.sum = svytable(fmla2, svobj)

      cat("\n - (Weighted) Contingency table for", bin.var, ":\n\n")
      svtbl.pr = rbind(svtbl.sum, t(svtbl))
      row.names(svtbl.pr)[1] = "ALL"
      if (anyNA) {
        svtbl.pr = rbind(svtbl.pr, svtbl.sum - apply(svtbl, 1, sum))
        row.names(svtbl.pr)[nrow(svtbl.pr)] = "<NA>"
      }
      print(svtbl.pr)
      if (anyNA) {
         cat("\n Note: NA category added for reference only---it is not used in statistical testing\n")
      }
      if (all(svtbl.sum > 0) & sum(apply(svtbl, 2, function(x) any(x > 0))) > 1) {
        print(summary(svtbl, ...)$stat)
      } else {
        cat("\n\t Design-based Wald test of association unavailable\n")
      }

      # clean-up
      rm(fmla, fmla2, svtbl, svtbl.sum, svtbl.pr)
    }
  
    ## Printout of relatives shares for report:
    
    cat("\n - Relative shares per group, conditional on", bin.var, "= 'Yes':\n\n")
    ALL = c(1,1) 
  
    if (anyNA) {
      print(round(rbind(ALL, t(tbl2)), digits))
      cat("\n")
    }

    print(round(rbind(ALL, t(tbl)), digits))

    rm(ALL) # clean-up!
  }

  if (anyNA) {

    ## print index and standard errors

    cat("\n ... NA vs. non-NA categories.\n")
    cat("\n - Between-group", name, "index: ", round(indNA, digits))
    cat("\n - Taylor Linearization SE: ", round(BJ.seNA, digits))
    cat("\n - Taylor Linearization P-value: ", pnorm(indNA/BJ.seNA, lower.tail=FALSE))
    cat("\n - Taylor Linearization 95% CI: ", "(", paste(round(BJ.ciNA, digits), collapse=", "), ")")

    if (!is.null(w.BRR.data)) {
      cat("\n - Balanced Repeated Replication (BRR) SE: ", round(BRR.seNA, digits))
      cat("\n - Balanced Repeated Replication (BRR) P-value: ", pnorm(indNA/BRR.seNA, lower.tail = FALSE))
      cat("\n - Balanced Repeated Replication (BRR) 95% CI: ", "(", paste(round(BRR.ciNA, digits), collapse=", "), ")")
    }  
    if (!is.null(w.boot.data)) {
      cat("\n - Rescaled bootstrap SE with", B, "replications: ", round(boot.seNA, digits))
      cat("\n - Rescaled bootstrap P-value with", B, "replications: ", pnorm(indNA/boot.seNA, lower.tail=FALSE))
      cat("\n - Rescaled bootstrap 95% CI with", B, "replications: ", "(", paste(round(boot.ciNA, digits), collapse=", "), ")")
    }
    cat("\n")
  }

  ## index and standard errors

  if (anyNA) {
    cat("\n ... Non-NA categories only.\n")
  }

  cat("\n - Between-group", name, "index: ", round(ind, digits))
  cat("\n - Taylor Linearization SE: ", round(BJ.se, digits))
  cat("\n - Taylor Linearization P-value: ", pnorm(ind/BJ.se, lower.tail=FALSE))
  cat("\n - Taylor Linearization 95% CI: ", "(", paste(round(BJ.ci, digits), collapse=", "), ")")

  if (!is.null(w.BRR.data)) {
    cat("\n - Balanced Repeated Replication (BRR) SE: ", round(BRR.se, digits))
    cat("\n - Balanced Repeated Replication (BRR) P-value: ", pnorm(ind/BRR.se, lower.tail = FALSE))
    cat("\n - Balanced Repeated Replication (BRR) 95% CI: ", "(", paste(round(BRR.ci, digits), collapse=", "), ")")
  }  
  if (!is.null(w.boot.data)) {
    cat("\n - Rescaled bootstrap SE with", B, "replications: ", round(boot.se, digits))
    cat("\n - Rescaled bootstrap P-value with", B, "replications: ", pnorm(ind/boot.se, lower.tail=FALSE))
    cat("\n - Rescaled bootstrap 95% CI with", B, "replications: ", "(", paste(round(boot.ci, digits), collapse=", "), ")")
  }
  cat("\n")

  ## In addition to the main report file, index (for non-NA categories) and 95% CI are outputed to out.file in a tabular format

  if (out.file != "") {
    cat(type, "\t", alpha, "\t", ind, "\t", BJ.se, "\t", BJ.ci[1], "\t", BJ.ci[2], "\t", BRR.se, "\t", BRR.ci[1], "\t", BRR.ci[2], "\t", boot.se, "\t", boot.ci[1], "\t", boot.ci[2], "\n", file=out.file, append=TRUE)
  }

  if (!store.runs) {
    return ()
  } else {
    return(list(type=type, alpha=alpha, ind=ind, BRR=BRR, boot=boot))
  }

} # theil.decomposition.bin

###########################################################################################################

theil.decomposition.out =     # This is a wrapper and print function for the main recursive function 'theil.decomposition'.
  function(type="T",          # Character indicating index to work with: T=Theil, L=MLD, S=STI, etc.
	   alpha=0.5,	      # Inequality aversion parameter for Renyi family of indices
           data=NULL,         # Either a survey design object or a data frame must be specified
           w.boot.data=NULL,  # For the bootstrap method, a pre-computed set of alternative weights.
           w.BRR.data=NULL,   # For the BRR method, a pre-computed set of alternative weights.
           fay.rho=0,         # Fay's parameter for BRR method.
           subset=NULL,       # Logical condition for subsetting the data frame.
           response="",       # Label for response variable.
           w.var="",          # Label for weighting variable.
           grouping=NULL,     # This is still an experimental print method:
                              # At most one variable name must be specified in 'grouping'.
           na.rm=TRUE,        # What to do with missing values in the weights and/or response variable
           bw="nrd",          # Parameter to pass to 'compute.MC.se', which implements the Martinez-Camblor method.
           kernel="gaussian", # Parameter to pass to compute.MC.se, which implements the Martinez-Camblor method.
           from=0,            # Parameter to pass to compute.MC.se, which implements the Martinez-Camblor method.
           digits=4,          # Number of decimals to keep in output
           svobj=NULL,        # Either a survey design object or a data frame must be specified.
                              # If both are supplied, data frame is over-ridden by survey design object.
           print.means=TRUE,  # Whether to report the means in addition to the indices and SEs.
           print.diff=TRUE,   # whether to print out pairwise differences in means.
           n.suppress=30,     # whether to suppress groups of size < 30 from printout.
           compute.MC=TRUE,   # Whether to compute Martinez-Camblor type SE and CI.
           compute.BJ=TRUE,   # Whether to compute Biewen-Jenkins type based SE and CI (ie. Taylor series linearization).
           out.file="")       # Path to output file for tabulated indices and their 95% CIs.
{ 

  ## Preambule
  
  if (length(grouping) > 1) {
    stop("Experimental print method: At most one variable name must be specified in 'grouping'!")
  }
  if (!is.null(svobj)) {
    if (nrow(svobj$variables) != sum(subset)) {
      stop("mismatched 'subset' condition and svobj data")
    }
  }
  if (!is.null(data)) {
    if (!is.null(svobj)) {
      warning("Supplied data frame over-ridden by survey design object.")
    }
    if (nrow(data) != length(subset)) {
      stop("mismatched 'subset' condition and data frame")
    }
  }
  if (!is.null(w.boot.data)) {
    if (nrow(w.boot.data) != length(subset)) {
      stop("mismatched 'subset' condition and data frame")
    }
  }
  if (!is.null(w.BRR.data)) {
    if (nrow(w.BRR.data) != length(subset)) {
      stop("mismatched 'subset' condition and data frame")
    }
  }

  ## Switch for special limiting cases alpha tends to 0 or 1:

  if (isTRUE(all.equal(alpha, 1))) {
     if (type == "RPW") {
     	type = "L"	# MLD is limiting case when alpha tends to 1
     } else if (type == "SRPW") {
     	type = "S" 	# STI is limiting case when alpha tends to 1
     } else if (type == "REW") {
     	type = "RL" 	# "equally-weighted" MLD is limiting case when alpha tends to 1
     } else if (type == "SREW") {
     	type = "RS" 	# "equally-weighted" STI is limiting case when alpha tends to 1
     }
  } else if (isTRUE(all.equal(alpha, 0))) {
     if (type == "RPW") {
     	type = "T"	# TI is limiting case when alpha tends to 0
     } else if (type == "SRPW") {
     	type = "S" 	# STI is limiting case when alpha tends to 0
     } else if (type == "REW") {
     	type = "RT" 	# "equally-weighted" TI is limiting case when alpha tends to 0
     } else if (type == "SREW") {
     	type = "RS" 	# "equally-weighted" STI is limiting case when alpha tends to 0
     }
  }

  ## Call the main recursive function and store result in td:
  
  td = theil.decomposition(type=type, alpha=alpha, data=data[subset,], response=response, w.var=w.var, svobj=svobj, grouping=grouping[1], na.rm=na.rm, compute.MC=compute.MC, compute.BJ=compute.BJ, w.alt=NULL, bw=bw, kernel=kernel, from=from)

  ## Scaled bootstrap method:
  
  if (!is.null(w.boot.data)) {
    B = ncol(w.boot.data)
    td.boot = matrix(0, B, 3)
    for (i in 1:B) {
      i.td = theil.decomposition(type=type, alpha=alpha, data=data[subset,], response=response, w.var="", svobj=svobj, grouping=grouping[1], na.rm=na.rm, compute.MC=FALSE, compute.BJ=FALSE, w.alt=w.boot.data[subset, i])
      td.boot[i, 1] = i.td$overall.index$index
      td.boot[i, 2] = i.td$wtd.wthn.grp.index$index
      td.boot[i, 3] = i.td$between.grp.index$index
    }
  }

  ## BRR method:
  
  if (!is.null(w.BRR.data)) {
    H = ncol(w.BRR.data)
    td.BRR = matrix(0, H, 3)
    for (i in 1:H) {
      i.td = theil.decomposition(type=type, alpha=alpha, data=data[subset,], response=response, w.var="", svobj=svobj, grouping=grouping[1], na.rm=na.rm, compute.MC=FALSE, compute.BJ=FALSE, w.alt=w.BRR.data[subset, i])
      td.BRR[i, 1] = i.td$overall.index$index
      td.BRR[i, 2] = i.td$wtd.wthn.grp.index$index
      td.BRR[i, 3] = i.td$between.grp.index$index
    }
  }
  
  ## Print-out means and design-based standard errors for non-NA groups.

  if (print.means & !is.null(svobj)) {
    if (is.null(grouping)) {
      cat("\n Overall mean & (design-based) std error.\n")
      fml = as.formula(paste("~", response))
      b = as.data.frame(svymean(fml, design=svobj, na.rm=na.rm))
      row.names(b) = c("ALL")
      print(round(b, digits))
    } else {
      fml1 = as.formula(paste("~ ", response))
      fml2 = as.formula(paste("~ ", paste(grouping[1], collapse="+")))
      a = as.data.frame(svyby(fml1, fml2, design=svobj, svymean, na.rm=na.rm))[,-1]
      level.names = row.names(a)[row.names(a) != "NA"] # condition needed?
      b = as.data.frame(svymean(fml1, design=svobj, na.rm=na.rm))
      if (!is.null(td$relG)) { # print out mean for <NA> group
        a.na = as.data.frame(svymean(fml1, design=subset(svobj, is.na(svobj$variables[,grouping[1]])), na.rm=na.rm))
      } else {
        a.na = NULL
      }
      fml2 = as.formula(paste(paste(response, "~ "), paste(grouping[1], collapse="+")))

      d = summary(svyglm(fml2, design=svobj))$coef
      dd = NULL; ddd = NULL; dddd = NULL; ddddd = NULL
      if (length(grouping) == 1) {
        if (length(level.names) == 3) {
          fml3 = as.formula(paste(paste(response, "~ ", paste("factor(", grouping, ", levels=levels(", grouping, ")", "[c(2,1,3)])", sep=""))))
          dd = summary(svyglm(fml3, design=svobj))$coef
        } else { 
	  if (length(level.names) == 4) {
            fml3 = as.formula(paste(paste(response, "~ ", paste("factor(", grouping, ", levels=levels(", grouping, ")", "[c(2,1,3,4)])", sep=""))))
            dd = summary(svyglm(fml3, design=svobj))$coef
            fml4 = as.formula(paste(paste(response, "~ ", paste("factor(", grouping, ", levels=levels(", grouping, ")", "[c(3,1,2,4)])", sep=""))))
            ddd = summary(svyglm(fml4, design=svobj))$coef
          } else {
            if (length(level.names) == 5) {
              fml3 = as.formula(paste(paste(response, "~ ", paste("factor(", grouping, ", levels=levels(", grouping, ")", "[c(2,1,3,4,5)])", sep=""))))
              dd = summary(svyglm(fml3, design=svobj))$coef
              fml4 = as.formula(paste(paste(response, "~ ", paste("factor(", grouping, ", levels=levels(", grouping, ")", "[c(3,1,2,4,5)])", sep=""))))
              ddd = summary(svyglm(fml4, design=svobj))$coef
              fml5 = as.formula(paste(paste(response, "~ ", paste("factor(", grouping, ", levels=levels(", grouping, ")", "[c(4,1,2,3,5)])", sep=""))))
              dddd = summary(svyglm(fml5, design=svobj))$coef        
            } else {
              if (length(level.names) == 6) {
                fml3 = as.formula(paste(paste(response, "~ ", paste("factor(", grouping, ", levels=levels(", grouping, ")", "[c(2,1,3,4,5,6)])", sep=""))))
                dd = summary(svyglm(fml3, design=svobj))$coef
                fml4 = as.formula(paste(paste(response, "~ ", paste("factor(", grouping, ", levels=levels(", grouping, ")", "[c(3,1,2,4,5,6)])", sep=""))))
                ddd = summary(svyglm(fml4, design=svobj))$coef
                fml5 = as.formula(paste(paste(response, "~ ", paste("factor(", grouping, ", levels=levels(", grouping, ")", "[c(4,1,2,3,5,6)])", sep=""))))
                dddd = summary(svyglm(fml5, design=svobj))$coef
                fml6 = as.formula(paste(paste(response, "~ ", paste("factor(", grouping, ", levels=levels(", grouping, ")", "[c(5,1,2,3,4,6)])", sep=""))))
                ddddd = summary(svyglm(fml6, design=svobj))$coef        
              }
            }
          }
        }        
      }
      
      ## Call formatting function:
      n.n = t(td$TTs[c("n", "N"),])
      format.print.svymeans(a, b, n.n, a.na, d, dd, ddd, dddd, ddddd, level.names, digits, print.diff, n.suppress)
    }
  }
  
  ## Print out relative shares:
  
  if (print.means & !is.null(grouping)) {
    cat("\n - Relative shares per group \n (relative to population aggregate share)\n compared to relative sizes of the groups in the population:\n\n")
    if (!is.null(td$relG)) {
      cat("\n ... including <NA> category\n")
    } 
    rel.tbl2 = rbind(c(1,1), t(td$rel))
    row.names(rel.tbl2)[1] = "ALL"
    print(round(rel.tbl2[rel.tbl2[,"rel.n"] > 0,], digits))
   
    if (!is.null(td$relG)) {
      cat("\n ... not including <NA> category\n")
      rel.tbl1 = rbind(c(1,1), t(td$relG))
      row.names(rel.tbl1)[1] = "ALL"
      print(round(rel.tbl1[rel.tbl1[,"rel.nG"] > 0,], digits))
    }
  }
  
  ## Print out Theil indices and SE's/CI's
  
  if (type == "T") {
    name = "Theil"
  } else if (type == "L") {
    name = "MLD"
  } else if (type == "S") {
    name = "STI"
  } else if (type == "RT") {
    name = "EW-Theil"
  } else if (type == "RL") {
    name = "EW-MLD"
  } else if (type == "RS") {
    name = "EW-STI"
  } else if (type == "RPW") {
    name = paste("PW-RI alpha=", alpha)
  } else if (type == "SRPW") {
    name = paste("PW-SRI alpha=", alpha)
  } else if (type == "REW") {
    name = paste("EW-RI alpha=", alpha)
  } else if (type == "SREW") {
    name = paste("EW-SRI alpha=", alpha)
  }
    
  index.tbl = t(td$TTs[c("n", "N", "index", "norm.index", "mc.se", "bj.se"),])
  mc.pval = rep(NA, nrow(index.tbl))
  bj.pval = rep(NA, nrow(index.tbl))
  boot.se1 = NA; boot.pval1 = NA; boot.LCL1 = NA; boot.UCL1 = NA
  BRR.se1 = NA; BRR.pval1 = NA; BRR.LCL1 = NA; BRR.UCL1 = NA

  if (compute.MC) {
    mc.pval = pnorm(index.tbl[,"index"]/index.tbl[,"mc.se"], lower.tail=FALSE)
  }
  if (compute.BJ) {
    bj.pval = pnorm(index.tbl[,"index"]/index.tbl[,"bj.se"], lower.tail=FALSE)
  }
  if (nrow(index.tbl) == 1) {
    if (!is.null(w.boot.data)) {
      boot.se1 = sqrt(mean((td.boot[,1]  - td$overall.index$index)^2, na.rm=T))
      boot.pval1 = pnorm(td$overall.index$index/boot.se1, lower.tail=FALSE)
      boot.LCL1 = quantile(td.boot[,1], c(0.025), na.rm=T)
      boot.UCL1 = quantile(td.boot[,1], c(0.975), na.rm=T)
    }
    if (!is.null(w.BRR.data)) {
      BRR.se1 = sqrt(mean((td.BRR[,1] - td$overall.index$index)^2, na.rm=T))/(1-fay.rho)
      BRR.pval1 = pnorm(td$overall.index$index/BRR.se1, lower.tail=FALSE)
      BRR.LCL1 = td$overall.index$index - BRR.se1*1.95996
      BRR.UCL1 = td$overall.index$index + BRR.se1*1.95996
    }
    index.tbl = cbind(index.tbl, BRR.se1, boot.se1, mc.pval, bj.pval, BRR.pval1, boot.pval1)
    dimnames(index.tbl)[[2]][7:12] = c("BRR.se", "bs.se", "mc.pval", "bj.pval", "BRR.pval", "bs.pval")
    prnt.tbl = index.tbl
    prnt.tbl[,3:8] = round(index.tbl[,3:8], digits)
  } else {
    index.tbl = cbind(index.tbl, mc.pval, bj.pval)
    dimnames(index.tbl)[[2]][7:8] = c("mc.pval", "bj.pval")
    prnt.tbl = index.tbl
    prnt.tbl[,3:6] = round(index.tbl[,3:6], digits) 
  }

  if (!is.element(type, c("RT", "RL", "RS", "REW", "SREW"))) { # Totals table not meaningful: Print aggregate index instead.

    if (!is.null(grouping)) {
      cat("\n -", name, "index by group:\n\n")
    } else {
      cat("\n -", name, "index and standard errors:\n\n")    
    }  

    if (any(prnt.tbl[,"n"] < n.suppress)) {
      cat("\t Indices for groups with sample size n < ", n.suppress, "are suppressed from this printout.\n\n")
      prnt.tbl[prnt.tbl[,"n"] < n.suppress, ] = NA
    }
    print(prnt.tbl[,-c(1:2)])

    if (print.diff) {
      if (compute.MC | compute.BJ) {
        if (!is.null(grouping)) {
          if (is.null(td$relG)) {
            cat("\n - Differences in within-group indexes:\n\n")
          } else {
            cat("\n - Differences in within-group indexes for non-NA categories:\n\n")
          } 
          if (compute.BJ) {
            print(format.print.index.tbl(index.tbl[, c("n", "index", "bj.se")], digits, n.suppress))
          } else {
            print(format.print.index.tbl(index.tbl[, c("n", "index", "mc.se")], digits, n.suppress))
          }
        }
      }
    }
  }

  if (!is.null(grouping)) {

    if (is.element(type, c("RT", "RL", "RS", "REW", "SREW"))) { # Print aggregate index.

      cat("\n -", name, "index and decomposition:\n\n")   

      if (is.null(td$rel)) {
        cat("\n - Aggregate", name, "index: ", round(td$overall.index$index, digits))
      } else {
        cat("\n - Aggregate", name, "index (for non-NA categories): ", round(td$overall.index$index, digits))
      }
      if (!is.null(svobj) & compute.BJ) {
        cat("\n - Taylor Linearization SE: ", round(td$overall.index$bj.se, digits))
        cat("\n - Taylor Linearization P-value: ", pnorm(td$overall.index$index/td$overall.index$bj.se, lower.tail=FALSE))
        cat("\n - Taylor Linearization 95% CI: ", "(", paste(round(td$overall.index$bj.CI, digits), collapse=", "), ")")
      }
      if (!is.null(w.BRR.data)) {
        BRR.se1 = sqrt(mean((td.BRR[,1] - td$overall.index$index)^2, na.rm=T))/(1-fay.rho)
        BRR.LCL1 = td$overall.index$index - BRR.se1*1.95996
        BRR.UCL1 = td$overall.index$index + BRR.se1*1.95996
        cat("\n - Balanced Repeated Replication (BRR) SE: ", round(BRR.se1, digits))
        cat("\n - Balanced Repeated Replication (BRR) P-value: ", pnorm(td$overall.index$index/BRR.se1, lower.tail=FALSE))
        cat("\n - Balanced Repeated Replication (BRR) 95% CI: ", "(", paste(round(c(BRR.LCL1, BRR.UCL1), digits), collapse=", "), ")")
      }
      if (!is.null(w.boot.data)) {
        boot.se1 = sqrt(mean((td.boot[,1]  - td$overall.index$index)^2, na.rm=T))
        boot.LCL1 = quantile(td.boot[,1], c(0.025), na.rm=T)
        boot.UCL1 = quantile(td.boot[,1], c(0.975), na.rm=T)
        cat("\n - Rescaled bootstrap SE with", B, "replications: ", round(boot.se1, digits))
        cat("\n - Rescaled bootstrap P-value: ", pnorm(td$overall.index$index/boot.se1, lower.tail=FALSE))        
        cat("\n - Rescaled bootstrap 95% CI with", B, "replications: ", "(", paste(round(c(boot.LCL1, boot.UCL1), digits), collapse=", "), ")")
      }
    
      cat("\n")
    }

    if (is.null(td$rel)) {
      cat("\n - Weighted within-group", name, "index: ", round(td$wtd.wthn.grp.index$index, digits))
    } else {
      cat("\n - Weighted within-group", name, "index (for non-NA categories): ", round(td$wtd.wthn.grp.index$index, digits))
    }
    if (compute.MC) {
      cat("\n - Martinez-Camblor SE: ", round(td$wtd.wthn.grp.index$mc.se, digits))
      cat("\n - Martinez-Camblor P-value: ", pnorm(td$wtd.wthn.grp.index$index/td$wtd.wthn.grp.index$mc.se, lower.tail=FALSE))
      cat("\n - Martinez-Camblor 95% CI: ", "(", paste(round(td$wtd.wthn.grp.index$mc.CI, digits), collapse=", "), ")")
    }
    if (!is.null(svobj) & compute.BJ) {
      cat("\n - Taylor Linearization SE: ", round(td$wtd.wthn.grp.index$bj.se, digits))
      cat("\n - Taylor Linearization P-value: ", pnorm(td$wtd.wthn.grp.index$index/td$wtd.wthn.grp.index$bj.se, lower.tail=FALSE))
      cat("\n - Taylor Linearization 95% CI: ", "(", paste(round(td$wtd.wthn.grp.index$bj.CI, digits), collapse=", "), ")")
    }

    BRR.se2 = NA; BRR.LCL2 = NA; BRR.UCL2 = NA

    if (!is.null(w.BRR.data)) {
      BRR.se2 = sqrt(mean((td.BRR[,2] - td$wtd.wthn.grp.index$index)^2, na.rm=T))/(1-fay.rho)
      BRR.LCL2 = td$wtd.wthn.grp.index$index - BRR.se2*1.959964
      BRR.UCL2 = td$wtd.wthn.grp.index$index + BRR.se2*1.959964
      cat("\n - Balanced Repeated Replication (BRR) SE: ", round(BRR.se2, digits))
      cat("\n - Balanced Repeated Replication (BRR) P-value: ", pnorm(td$wtd.wthn.grp.index$index/BRR.se2, lower.tail=FALSE))
      cat("\n - Balanced Repeated Replication (BRR) 95% CI: ", "(", paste(round(c(BRR.LCL2, BRR.UCL2), digits), collapse=", "), ")")
    }

    boot.se2 = NA; boot.LCL2 = NA; boot.UCL2 = NA

    if (!is.null(w.boot.data)) {
      boot.se2 = sqrt(mean((td.boot[,2]  - td$wtd.wthn.grp.index$index)^2, na.rm=T))
      boot.LCL2 = quantile(td.boot[,2], c(0.025), na.rm=T)
      boot.UCL2 = quantile(td.boot[,2], c(0.975), na.rm=T)
      cat("\n - Rescaled bootstrap SE with", B, "replications: ", round(boot.se2, digits))
      cat("\n - Rescaled bootstrap P-value: ", pnorm(td$wtd.wthn.grp.index$index/boot.se2, lower.tail=FALSE))
      cat("\n - Rescaled bootstrap 95% CI with", B, "replications: ", "(", paste(round(c(boot.LCL2, boot.UCL2), digits), collapse=", "), ")")
    }
    
    cat("\n")

    if (is.null(td$rel)) {
      cat("\n - Between-group", name, "index: ", round(td$between.grp.index$index, digits))
    } else {
      cat("\n - Between-group", name, "index (for non-NA categories): ", round(td$between.grp.index$index, digits))
    }
    cat("\n - Proportion in marginal", name, "index: ", round(td$prop.ineq.from.between, digits))

    if (compute.MC) {
      cat("\n - Martinez-Camblor SE (marginal minus weighted-within): ", round(td$between.grp.index$mc.se, digits))
      cat("\n - Approximate Martinez-Camblor CI (marginal minus weighted-within): ", "(", paste(round(td$between.grp.index$mc.CI, digits), collapse=", "), ")")
    }
    if (!is.null(svobj) & compute.BJ) {
      cat("\n - Taylor Linearization SE: ", round(td$between.grp.index$bj.se, digits))
      cat("\n - Taylor Linearization P-value: ", pnorm(td$between.grp.index$index/td$between.grp.index$bj.se, lower.tail=FALSE))
      cat("\n - Taylor Linearization 95% CI: ", "(", paste(round(td$between.grp.index$bj.CI, digits), collapse=", "), ")")
    }

    BRR.se3 = NA; BRR.LCL3 = NA; BRR.UCL3 = NA

    if (!is.null(w.BRR.data)) {
        BRR.se3 = sqrt(mean((td.BRR[,3] - td$between.grp.index$index)^2, na.rm=T))/(1-fay.rho)
        BRR.LCL3 = td$between.grp.index$index - BRR.se3*1.959964
        BRR.UCL3 = td$between.grp.index$index + BRR.se3*1.959964
        cat("\n - Balanced Repeated Replication (BRR) SE: ", round(BRR.se3, digits))
        cat("\n - Balanced Repeated Replication (BRR) P-value: ", pnorm(td$between.grp.index$index/BRR.se3, lower.tail=FALSE))
        cat("\n - Balanced Repeated Replication (BRR) 95% CI: ", "(", paste(round(c(BRR.LCL3, BRR.UCL3), digits), collapse=", "), ")")
    }

    boot.se3 = NA; boot.LCL3 = NA; boot.UCL3 = NA

    if (!is.null(w.boot.data)) {
      boot.se3 = sqrt(mean((td.boot[,3]  - td$between.grp.index$index)^2, na.rm=T))
      boot.LCL3 = quantile(td.boot[,3], c(0.025), na.rm=T)
      boot.UCL3 = quantile(td.boot[,3], c(0.975), na.rm=T)
      cat("\n - Rescaled bootstrap SE with", B, "replications: ", round(boot.se3, digits))
      cat("\n - Rescaled bootstrap P-value: ", pnorm(td$between.grp.index$index/boot.se3, lower.tail=FALSE))           
      cat("\n - Rescaled bootstrap 95% CI with", B, "replications: ", "(", paste(round(c(boot.LCL3, boot.UCL3), digits), collapse=", "), ")")
    }
  }

  cat("\n")

  # In addition to the main report file, index and 95% CI are outputed to out.file in a tabular format.
  if (out.file != "") {
    if (is.null(grouping)) {
      cat(type, "\t", alpha, "\t", "total", "\t",
          td$overall.index$index, "\t", 1, "\t", 
          td$overall.index$bj.se, "\t", td$overall.index$bj.CI[1], "\t", td$overall.index$bj.CI[2], "\t",
      	  BRR.se1, "\t",  BRR.LCL1, "\t", BRR.UCL1, "\t", boot.se1, "\t", boot.LCL1, "\t", boot.UCL1, "\n", 
          file=out.file, append=TRUE)
    } else {  
      cat(type, "\t", alpha, "\t", "aggregate", "\t",
          td$overall.index$index, "\t", 1, "\t", 
          td$overall.index$bj.se, "\t", td$overall.index$bj.CI[1], "\t", td$overall.index$bj.CI[2], "\t",
      	  BRR.se1, "\t",  BRR.LCL1, "\t", BRR.UCL1, "\t", boot.se1, "\t", boot.LCL1, "\t", boot.UCL1, "\n", 
          file=out.file, append=TRUE)

      cat("", "\t", "", "\t", file=out.file, append=TRUE)

      cat(type, "\t", alpha, "\t", "within", "\t",
          td$wtd.wthn.grp.index$index, "\t", 1-td$prop.ineq.from.between, "\t", 
          td$wtd.wthn.grp.index$bj.se, "\t", td$wtd.wthn.grp.index$bj.CI[1], "\t", td$wtd.wthn.grp.index$bj.CI[2], "\t",
      	  BRR.se2, "\t",  BRR.LCL2, "\t", BRR.UCL2, "\t", boot.se2, "\t", boot.LCL2, "\t", boot.UCL2, "\n", 
          file=out.file, append=TRUE)

      cat("", "\t", "", "\t", file=out.file, append=TRUE)

      cat(type, "\t", alpha, "\t", "between", "\t",
          td$between.grp.index$index, "\t", td$prop.ineq.from.between, "\t", 
          td$between.grp.index$bj.se, "\t", td$between.grp.index$bj.CI[1], "\t", td$between.grp.index$bj.CI[2], "\t",
      	  BRR.se3, "\t",  BRR.LCL3, "\t", BRR.UCL3, "\t", boot.se3, "\t", boot.LCL3, "\t", boot.UCL3, "\n", 
          file=out.file, append=TRUE)
    }
  }
  
  return()

} # theil.decomposition.out

#######################################################################################################

within.BJ.se =            # Taylor Linearization technique in Biewen and Jenkins (2006) and Borrell and Talih (2011) for within-group component.
  function(type = "T",    # character indicating index to work with: T=Theil, L=MLD, S=STI, etc.
	   alpha=0.5,	  # Inequality aversion parameter for Renyi family of indices 
           xlab,          # response variable label, to be looked up in the dataframe, from which index is to be computed 
           grp.var,       # label of grouping variable, to be looked up in dataframe, from which index is decomposed
           svobj,         # survey design object from 'svyobject', cf. Lumley's 'survey' package
           speed.up=NULL, # option to speed up calculation of sufficient statistics U0, U1, Ua, Uaa, T0, T1: those could be passed as a vector
           na.rm=TRUE)    # what do to with missing values in the response variable and sampling weights
{
    
  ## Require survey design object:
  if (is.null(svobj)) {
    stop("A survey design object must be supplied")
  }
  if (!is.element(grp.var, names(svobj$variables))) {
    stop("Variable", grp.var, "not found in survey design object")
  }
  if (!is.element(xlab, names(svobj$variables))) {
    stop("Variable", xlab, "not found in survey design object")
  }

  ## Switch for special limiting cases alpha tends to 0 or 1:
  if (isTRUE(all.equal(alpha, 1))) {
     if (type == "RPW") {
     	type = "L"	# MLD is limiting case when alpha tends to 1
     } else if (type == "SRPW") {
     	type = "S" 	# STI is limiting case when alpha tends to 1
     } else if (type == "REW") {
     	type = "RL" 	# "equally-weighted" MLD is limiting case when alpha tends to 1
     } else if (type == "SREW") {
     	type = "RS" 	# "equally-weighted" STI is limiting case when alpha tends to 1
     }
  } else if (isTRUE(all.equal(alpha, 0))) {
     if (type == "RPW") {
     	type = "T"	# TI is limiting case when alpha tends to 0
     } else if (type == "SRPW") {
     	type = "S" 	# STI is limiting case when alpha tends to 0
     } else if (type == "REW") {
     	type = "RT" 	# "equally-weighted" TI is limiting case when alpha tends to 0
     } else if (type == "SREW") {
     	type = "RS" 	# "equally-weighted" STI is limiting case when alpha tends to 0
     }
  }

  ## drop observations with missing group membership info:
  svobj = subset(svobj, !is.na(svobj$variables[,grp.var]))

  grp = svobj$variables[,grp.var]
  x = svobj$variables[,xlab]
  w = 1/svobj$prob

  grp.names = levels(grp)
  if (length(grp.names) < 2) {
    warning("Expecting a factor with at least two levels. Omitting calculation of within.BJ.se.")
    return(NA)
  }
  
  if (is.null(speed.up)) {
    U0 = sapply(grp.names, function(g) sum(w[grp == g], na.rm=na.rm))
    U1 = sapply(grp.names, function(g) sum(w[grp == g]*x[grp == g], na.rm=na.rm))
    Ua = sapply(grp.names, function(g) sum(w[grp == g]*x[grp == g]^alpha, na.rm=na.rm))
    Uaa = sapply(grp.names, function(g) sum(w[grp == g]*x[grp == g]^(1-alpha), na.rm=na.rm))
    T0 = sapply(grp.names, function(g) sum(w[grp == g]*log(x[grp == g]), na.rm=na.rm))
    T1 = sapply(grp.names, function(g) sum(w[grp == g]*ifelse(x[grp == g] > 0, x[grp == g]*log(x[grp == g]), 0), na.rm=na.rm))
  } else { ## if passed internally
    U0 = speed.up$U0
    U1 = speed.up$U1
    Ua = speed.up$Ua
    Uaa = speed.up$Uaa
    T0 = speed.up$T0
    T1 = speed.up$T1
  }

  DD0 = vector("numeric", length(grp.names))
  names(DD0) = grp.names
  DD1 = vector("numeric", length(grp.names))
  names(DD1) = grp.names
  DDa = vector("numeric", length(grp.names))
  names(DDa) = grp.names
  DDaa = vector("numeric", length(grp.names))
  names(DDaa) = grp.names
  TT0 = vector("numeric", length(grp.names))
  names(TT0) = grp.names
  TT1 = vector("numeric", length(grp.names))
  names(TT1) = grp.names

  Um = ifelse(U0 > 0, U1/U0, 0)

  cond.0 = (U0 > 0)

  U0 = U0[cond.0] # trim empty groups
  U1 = U1[cond.0]
  Ua = Ua[cond.0]
  Uaa = Uaa[cond.0]
  T0 = T0[cond.0]
  T1 = T1[cond.0]
  Um = Um[cond.0]

  u0 = sum(U0)
  u1 = sum(U1)
  ua = sum(Ua)
  uaa = sum(Uaa)
  um = sum(Um)
  mm = length(U0)
  
  bj.vr = NA
  s = vector("numeric", length(x))
  
  if (type == "T") {
    ## derivatives of within-group Theil index:
    DD0[cond.0] = (1/u1)*Um
    DD1[cond.0] = -(1/u1)*(log(Um) + 1) - (1/u1^2)*sum(T1 - ifelse(U1 > 0, U1*log(Um), 0))
    TT1[cond.0] = 1/u1

    ## variance elements
    for (g in grp.names) {
      s[grp == g] = DD0[g] + DD1[g]*x[grp == g] + TT1[g]*ifelse(x[grp == g] > 0, x[grp == g]*log(x[grp == g]), 0)
    }
  } else if (type == "L") {
    ## derivatives of within-group MLD index:
    DD0[cond.0] = (1/u0)*(log(Um) - 1) + (1/u0^2)*sum(T0 - U0*log(Um))
    DD1[cond.0] = (1/u0)*(1/Um)
    TT0[cond.0] = -1/u0

    ## variance elements
    for (g in grp.names) {
      s[grp == g] = DD0[g] + DD1[g]*x[grp == g] + TT0[g]*log(x[grp == g])
    }
  } else if (type == "S") {
    ## derivatives of the symmetrized within-group Theil index:
    DD0[cond.0] = ( (1/u1)*Um + (1/u0)*(log(Um) - 1) + (1/u0^2)*sum(T0 - U0*log(Um)) )/2
    DD1[cond.0] = ( (1/u0)*(1/Um) - (1/u1)*(log(Um) + 1) - (1/u1^2)*sum(T1 - ifelse(U1 > 0, U1*log(Um), 0)) )/2
    TT0[cond.0] = (-1/u0)/2
    TT1[cond.0] = (1/u1)/2

    ## variance elements
    for (g in grp.names) {
      s[grp == g] = DD0[g] + DD1[g]*x[grp == g] + TT0[g]*log(x[grp == g]) + TT1[g]*ifelse(x[grp == g] > 0, x[grp == g]*log(x[grp == g]), 0)
    }
  } else if (type == "RT") {
    ## derivatives of equally weighted within-group Theil index:
    DD0[cond.0] = -(1/um)*(1/U0)*(T1/U0 - ifelse(U1 > 0, Um*(log(Um) + 1), 0)) + (1/um^2)*(1/U0)*Um*sum(T1/U0 - ifelse(U1 > 0, Um*log(Um), 0)) 
    DD1[cond.0] = -(1/um)*(1/U0)*(log(Um) + 1) - (1/um^2)*(1/U0)*sum(T1/U0 - ifelse(U1 > 0, Um*log(Um), 0))
    TT1[cond.0] = (1/um)*(1/U0)

    ## variance elements
    for (g in grp.names) {
      s[grp == g] = DD0[g] + DD1[g]*x[grp == g] + TT1[g]*ifelse(x[grp == g] > 0, x[grp == g]*log(x[grp == g]), 0)
    }
  } else if (type == "RL") {
    ## derivatives of equally weighted within-group MLD index:
    DD0[cond.0] = -(1/mm)*(1/U0 - T0/U0^2) 
    DD1[cond.0] = (1/mm)*(1/U1)
    TT0[cond.0] = -(1/mm)*(1/U0)

    ## variance elements
    for (g in grp.names) {
      s[grp == g] = DD0[g] + DD1[g]*x[grp == g] + TT0[g]*log(x[grp == g])
    }
  } else if (type == "RS") {
    ## derivatives of equally weighted within-group STI index:
    DD0[cond.0] = (-(1/mm)*(1/U0 - T0/U0^2) - (1/um)*(1/U0)*(T1/U0 - ifelse(U1 > 0, Um*(log(Um) + 1), 0)) + (1/um^2)*(1/U0)*Um*sum(T1/U0 - ifelse(U1 > 0, Um*log(Um), 0)))/2
    DD1[cond.0] = ((1/mm)*(1/U1) - (1/um)*(1/U0)*(log(Um) + 1) - (1/um^2)*(1/U0)*sum(T1/U0 - ifelse(U1 > 0, Um*log(Um), 0)))/2
    TT0[cond.0] = -((1/mm)*(1/U0))/2
    TT1[cond.0] = ((1/um)*(1/U0))/2

    ## variance elements
    for (g in grp.names) {
      s[grp == g] = DD0[g] + DD1[g]*x[grp == g] + TT0[g]*log(x[grp == g]) + TT1[g]*ifelse(x[grp == g] > 0, x[grp == g]*log(x[grp == g]), 0)
    }
  } else if (type == "RPW") {
    DD0[cond.0] = (Um^(1-alpha)/sum(U0*Um^(1-alpha)))/(1-alpha)
    DD1[cond.0] = (Um^(-alpha)/sum(U0*Um^(1-alpha)))/alpha
    DDaa[cond.0] = (-1/uaa)/(alpha*(1-alpha))

    for (g in grp.names) {
      s[grp == g] = DD0[g] + DD1[g]*x[grp == g] + DDaa[g]*x[grp == g]^(1-alpha)
    }
  } else if (type == "SRPW") {
    DD0[cond.0] = ((Um^(1-alpha)/sum(U0*Um^(1-alpha)))/(1-alpha) + (Um^alpha/sum(U0*Um^alpha))/alpha)/2
    DD1[cond.0] = ((Um^(-alpha)/sum(U0*Um^(1-alpha)))/alpha + (Um^(alpha-1)/sum(U0*Um^alpha))/(1-alpha))/2
    DDaa[cond.0] = (-1/uaa)/(2*alpha*(1-alpha)) 
    DDa[cond.0] = (-1/ua)/(2*alpha*(1-alpha))  

    for (g in grp.names) {
      s[grp == g] = DD0[g] + DD1[g]*x[grp == g] + DDaa[g]*x[grp == g]^(1-alpha) + DDa[g]*x[grp == g]^alpha
    }
  } else if (type == "REW") {
    DD0[cond.0] = (-(1-alpha)*Um^(1-alpha)/(U0*sum(Um^(1-alpha))) + (Uaa/U0)/(U0*sum(Uaa/U0)))/(alpha*(1-alpha))
    DD1[cond.0] = (Um^(-alpha)/(U0*sum(Um^(1-alpha))))/alpha
    DDaa[cond.0] = -(1/(U0*sum(Uaa/U0)))/(alpha*(1-alpha))

    for (g in levels(grp)) {
      s[grp == g] = DD0[g] + DD1[g]*x[grp == g] + DDaa[g]*x[grp == g]^(1-alpha)
    }
  } else if (type == "SREW") {
    DD0[cond.0] = ( -(1-alpha)*Um^(1-alpha)/(U0*sum(Um^(1-alpha))) + (Uaa/U0)/(U0*sum(Uaa/U0)) - alpha*Um^alpha/(U0*sum(Um^alpha)) + (Ua/U0)/(U0*sum(Ua/U0)) ) /(2*alpha*(1-alpha))
    DD1[cond.0] = (Um^(-alpha)/(U0*sum(Um^(1-alpha))))/(2*alpha) + (Um^(alpha-1)/(U0*sum(Um^alpha)))/(2*(1-alpha))
    DDaa[cond.0] = -(1/(U0*sum(Uaa/U0)))/(2*alpha*(1-alpha))
    DDa[cond.0] = -(1/(U0*sum(Ua/U0)))/(2*alpha*(1-alpha))

    for (g in grp.names) {
      s[grp == g] = DD0[g] + DD1[g]*x[grp == g] + DDaa[g]*x[grp == g]^(1-alpha) + DDa[g]*x[grp == g]^alpha
    }
  }
  
  if (!all(is.na(s))) { # should cover is.nan as well
     svobj = update(svobj, s = s)
     bj.vr = attr(svytotal(~s, svobj, na.rm=na.rm), "var") 
  }
  return(sqrt(bj.vr))
  
} # within.BJ.se

#######################################################################################################

between.BJ.se =           # Taylor Linearization technique in Biewen and Jenkins (2006) and Borrell and Talih (2011) for between-group component.
  function(type = "T",    # character indicating index to work with: T=Theil, L=MLD, S=STI, etc.
	   alpha=0.5,	  # Inequality aversion parameter for Renyi family of indices 
           xlab,          # response variable label, to be looked up in the dataframe, from which index is to be computed 
           grp.var,       # label of grouping variable, to be looked up in dataframe, from which index is decomposed
           svobj,         # survey design object from 'svyobject', cf. Lumley's 'survey' package
           speed.up=NULL, # option to speed up calculation of sufficient statistics U0, U1: those could be passed as a vector
           na.rm=TRUE)    # what do to with missing values in the response variable and sampling weights
{
  
  # Require survey design object:
  if (is.null(svobj)) {
    stop("A survey design object must be supplied")
  }
  if (!is.element(grp.var, names(svobj$variables))) {
    stop("Variable", grp.var, "not found in survey design object")
  }
  if (!is.element(xlab, names(svobj$variables))) {
    stop("Variable", xlab, "not found in survey design object")
  }
   
  ## Switch for special limiting cases alpha tends to 0 or 1:
  if (isTRUE(all.equal(alpha, 1))) {
     if (type == "RPW") {
     	type = "L"	# MLD is limiting case when alpha tends to 1
     } else if (type == "SRPW") {
     	type = "S" 	# STI is limiting case when alpha tends to 1
     } else if (type == "REW") {
     	type = "RL" 	# "equally-weighted" MLD is limiting case when alpha tends to 1
     } else if (type == "SREW") {
     	type = "RS" 	# "equally-weighted" STI is limiting case when alpha tends to 1
     }
  } else if (isTRUE(all.equal(alpha, 0))) {
     if (type == "RPW") {
     	type = "T"	# TI is limiting case when alpha tends to 0
     } else if (type == "SRPW") {
     	type = "S" 	# STI is limiting case when alpha tends to 0
     } else if (type == "REW") {
     	type = "RT" 	# "equally-weighted" TI is limiting case when alpha tends to 0
     } else if (type == "SREW") {
     	type = "RS" 	# "equally-weighted" STI is limiting case when alpha tends to 0
     }
  }
   
  # drop observations with missing group membership info:
  svobj = subset(svobj, !is.na(svobj$variables[,grp.var]))
  grp = svobj$variables[,grp.var]
  x = svobj$variables[,xlab]
  w = 1/svobj$prob

  grp.names = levels(grp)
  if (length(grp.names) < 2) {
    warning("Expecting a factor with at least two levels. Omitting calculation of between.BJ.se.")
    return(NA)
  }
  
  if (is.factor(x) & length(levels(x)) == 2) { ## binary case
    # Coerce into a numerical variable with 1 for first level
    x = (x == levels(x)[1]) + 0
  }
  
  if (is.null(speed.up)) {
    U0 = sapply(grp.names, function(g) sum(w[grp == g], na.rm=na.rm))
    U1 = sapply(grp.names, function(g) sum(w[grp == g]*x[grp == g], na.rm=na.rm))
  } else { ## if passed internally
    U0 = speed.up$U0
    U1 = speed.up$U1
  }
    
  bj.vr = NA

  DD0 = vector("numeric", length(grp.names))
  names(DD0) = grp.names
  DD1 = vector("numeric", length(grp.names))
  names(DD1) = grp.names

  Um = ifelse(U0 > 0, U1/U0, 0)

  cond.0 = (U0 > 0)

  U0 = U0[cond.0] # trim empty groups
  U1 = U1[cond.0]
  Um = Um[cond.0]

  u0 = sum(U0)
  u1 = sum(U1)
  um = sum(Um)
  mm = length(U1)

  if (type == "T") {
    ## derivatives of between-group index:
    DD0[cond.0] = 1/u0 - (1/u1)*Um
    DD1[cond.0] = (1/u1)*log(Um) - (1/u1^2)*sum(ifelse(U1 > 0, U1*log(Um), 0))
  } else if (type == "L") {
    ## derivatives of between-group index:
    DD0[cond.0] = -(1/u0)*log(Um) + (1/u0^2)*sum(U0*log(Um))
    DD1[cond.0] = 1/u1 - (1/u0)*(1/Um)
  } else if (type == "S") {
    ## derivatives of the symmetrized between-group Theil index:
    DD0[cond.0] = ( 1/u0 - (1/u1)*Um - (1/u0)*log(Um) + (1/u0^2)*sum(U0*log(Um)) )/2
    DD1[cond.0] = ( 1/u1 - (1/u0)*(1/Um) + (1/u1)*log(Um) - (1/u1^2)*sum(ifelse(U1 > 0, U1*log(Um), 0)) )/2
  } else if (type == "RT") {
    ## derivatives of equally-weighted between-group index:
    DD0[cond.0] = (-Um/(U0*um))*(log(Um) - sum(ifelse(U1 > 0, Um*log(Um), 0))/um)
    DD1[cond.0] = (1/(U0*um))*(log(Um) - sum(ifelse(U1 > 0, Um*log(Um), 0))/um)    
  } else if (type == "RL") {
    ## derivatives of equally-weighted between-group index:
    DD0[cond.0] = (-1/U0)*(Um/um - 1/mm)
    DD1[cond.0] = (1/U1)*(Um/um - 1/mm)    
  } else if (type == "RS") {
    ## derivatives of equally-weighted between-group index:
    DD0[cond.0] = ((-Um/(U0*um))*(log(Um) - sum(ifelse(U1 > 0, Um*log(Um), 0))/um) + (-1/U0)*(Um/um - 1/mm))/2
    DD1[cond.0] = ((1/(U0*um))*(log(Um) - sum(ifelse(U1 > 0, Um*log(Um), 0))/um) + (1/U1)*(Um/um - 1/mm))/2  
  } else if (type == "RPW") {
    ## derivatives of the population-weighted between-group Renyi index:
    DD0[cond.0] = (1/u0 - Um^(1-alpha)/sum(U0*Um^(1-alpha)))/(1-alpha)
    DD1[cond.0] = (1/u1 - Um^(-alpha)/sum(U0*Um^(1-alpha)))/alpha 
  } else if (type == "SRPW") {
    ## derivatives of the population-weighted symmetrized between-group Renyi index:
    DD0[cond.0] = ((1/u0 - Um^(1-alpha)/sum(U0*Um^(1-alpha)))/(1-alpha) + (1/u0 - Um^alpha/sum(U0*Um^alpha))/alpha)/2
    DD1[cond.0] = ((1/u1 - Um^(-alpha)/sum(U0*Um^(1-alpha)))/alpha + (1/u1 - Um^(alpha-1)/sum(U0*Um^alpha))/(1-alpha))/2
  } else if (type == "REW") {
    ## derivatives of the population-weighted between-group Renyi index:
    DD0[cond.0] = (-1/U0)*(Um/um - Um^(1-alpha)/sum(Um^(1-alpha)))/alpha
    DD1[cond.0] = (1/U1)*(Um/um - Um^(1-alpha)/sum(Um^(1-alpha)))/alpha
  } else if (type == "SREW") {
    ## derivatives of the population-weighted symmetrized between-group Renyi index:
    DD0[cond.0] = ((-1/U0)*(Um/um - Um^(1-alpha)/sum(Um^(1-alpha)))/alpha + (-1/U0)*(Um/um - Um^alpha/sum(Um^alpha))/(1-alpha))/2
    DD1[cond.0] = ((1/U1)*(Um/um - Um^(1-alpha)/sum(Um^(1-alpha)))/alpha + (1/U1)*(Um/um - Um^alpha/sum(Um^alpha))/(1-alpha))/2
  }

  ## variance elements
  s = vector("numeric", length(x))
  for (g in grp.names) {
    s[grp == g] = DD0[g] + DD1[g]*x[grp == g]
  }

  if (!all(is.na(s))) { # should cover is.nan as well
    svobj = update(svobj, s = s)
    bj.vr = attr(svytotal(~s, svobj, na.rm=na.rm), "var")
  }
  return(sqrt(bj.vr))
  
} # between.BJ.se

###########################################################################################################################

between.BJ.seNA =         # Taylor Linearization technique in Biewen and Jenkins (2006) and Borrell and Talih (2011) for between-group component of NA vs non-NA grps.
  function(type = "T",    # character indicating index to work with: T=Theil, L=MLD, S=STI, etc.
	   alpha=0.5,	  # Inequality aversion parameter for Renyi family of indices 
           xlab,          # response variable label, to be looked up in the dataframe, from which index is to be computed 
           grp.var,       # label of grouping variable, to be looked up in dataframe, from which index is decomposed
           svobj,         # survey design object from 'svyobject', cf. Lumley's 'survey' package
           speed.up=NULL, # option to speed up calculation of sufficient statistics U0, U1: those could be passed as a vector
           na.rm=TRUE)    # what do to with missing values in the response variable and sampling weights
{
  
  # Require survey design object:
  if (is.null(svobj)) {
    stop("A survey design object must be supplied")
  }
  if (!is.element(grp.var, names(svobj$variables))) {
    stop("Variable", grp.var, "not found in survey design object")
  }
  if (!is.element(xlab, names(svobj$variables))) {
    stop("Variable", xlab, "not found in survey design object")
  }
   
  ## Switch for special limiting cases alpha tends to 0 or 1:
  if (isTRUE(all.equal(alpha, 1))) {
     if (type == "RPW") {
     	type = "L"	# MLD is limiting case when alpha tends to 1
     } else if (type == "SRPW") {
     	type = "S" 	# STI is limiting case when alpha tends to 1
     } else if (type == "REW") {
     	type = "RL" 	# "equally-weighted" MLD is limiting case when alpha tends to 1
     } else if (type == "SREW") {
     	type = "RS" 	# "equally-weighted" STI is limiting case when alpha tends to 1
     }
  } else if (isTRUE(all.equal(alpha, 0))) {
     if (type == "RPW") {
     	type = "T"	# TI is limiting case when alpha tends to 0
     } else if (type == "SRPW") {
     	type = "S" 	# STI is limiting case when alpha tends to 0
     } else if (type == "REW") {
     	type = "RT" 	# "equally-weighted" TI is limiting case when alpha tends to 0
     } else if (type == "SREW") {
     	type = "RS" 	# "equally-weighted" STI is limiting case when alpha tends to 0
     }
  }
   
  # collapse grouing variable into NA vs non-NA:
  grp = factor(!is.na(svobj$variables[,grp.var]), levels=c(TRUE, FALSE), labels=c("<non-NA>", "<NA>"))
  x = svobj$variables[,xlab]
  w = 1/svobj$prob

  grp.names = levels(grp)
  
  if (is.factor(x) & length(levels(x)) == 2) { ## binary case
    # Coerce into a numerical variable with 1 for first level
    x = (x == levels(x)[1]) + 0
  }
  
  if (is.null(speed.up)) {
    U0 = sapply(grp.names, function(g) sum(w[grp == g], na.rm=na.rm))
    U1 = sapply(grp.names, function(g) sum(w[grp == g]*x[grp == g], na.rm=na.rm))
  } else { ## if passed internally
    U0 = speed.up$U0
    U1 = speed.up$U1
  }
    
  bj.vr = NA

  DD0 = vector("numeric", length(grp.names))
  names(DD0) = grp.names
  DD1 = vector("numeric", length(grp.names))
  names(DD1) = grp.names

  Um = ifelse(U0 > 0, U1/U0, 0)

  cond.0 = (U0 > 0)

  U0 = U0[cond.0] # trim empty groups
  U1 = U1[cond.0]
  Um = Um[cond.0]

  u0 = sum(U0)
  u1 = sum(U1)
  um = sum(Um)
  mm = length(U1)

  if (type == "T") {
    ## derivatives of between-group index:
    DD0[cond.0] = 1/u0 - (1/u1)*Um
    DD1[cond.0] = (1/u1)*log(Um) - (1/u1^2)*sum(ifelse(U1 > 0, U1*log(Um), 0))
  } else if (type == "L") {
    ## derivatives of between-group index:
    DD0[cond.0] = -(1/u0)*log(Um) + (1/u0^2)*sum(U0*log(Um))
    DD1[cond.0] = 1/u1 - (1/u0)*(1/Um)
  } else if (type == "S") {
    ## derivatives of the symmetrized between-group Theil index:
    DD0[cond.0] = ( 1/u0 - (1/u1)*Um - (1/u0)*log(Um) + (1/u0^2)*sum(U0*log(Um)) )/2
    DD1[cond.0] = ( 1/u1 - (1/u0)*(1/Um) + (1/u1)*log(Um) - (1/u1^2)*sum(ifelse(U1 > 0, U1*log(Um), 0)) )/2
  } else if (type == "RT") {
    ## derivatives of equally-weighted between-group index:
    DD0[cond.0] = (-Um/(U0*um))*(log(Um) - sum(ifelse(U1 > 0, Um*log(Um), 0))/um)
    DD1[cond.0] = (1/(U0*um))*(log(Um) - sum(ifelse(U1 > 0, Um*log(Um), 0))/um)    
  } else if (type == "RL") {
    ## derivatives of equally-weighted between-group index:
    DD0[cond.0] = (-1/U0)*(Um/um - 1/mm)
    DD1[cond.0] = (1/U1)*(Um/um - 1/mm)    
  } else if (type == "RS") {
    ## derivatives of equally-weighted between-group index:
    DD0[cond.0] = ((-Um/(U0*um))*(log(Um) - sum(ifelse(U1 > 0, Um*log(Um), 0))/um) + (-1/U0)*(Um/um - 1/mm))/2
    DD1[cond.0] = ((1/(U0*um))*(log(Um) - sum(ifelse(U1 > 0, Um*log(Um), 0))/um) + (1/U1)*(Um/um - 1/mm))/2  
  } else if (type == "RPW") {
    ## derivatives of the population-weighted between-group Renyi index:
    DD0[cond.0] = (1/u0 - Um^(1-alpha)/sum(U0*Um^(1-alpha)))/(1-alpha)
    DD1[cond.0] = (1/u1 - Um^(-alpha)/sum(U0*Um^(1-alpha)))/alpha 
  } else if (type == "SRPW") {
    ## derivatives of the population-weighted symmetrized between-group Renyi index:
    DD0[cond.0] = ((1/u0 - Um^(1-alpha)/sum(U0*Um^(1-alpha)))/(1-alpha) + (1/u0 - Um^alpha/sum(U0*Um^alpha))/alpha)/2
    DD1[cond.0] = ((1/u1 - Um^(-alpha)/sum(U0*Um^(1-alpha)))/alpha + (1/u1 - Um^(alpha-1)/sum(U0*Um^alpha))/(1-alpha))/2
  } else if (type == "REW") {
    ## derivatives of the population-weighted between-group Renyi index:
    DD0[cond.0] = (-1/U0)*(Um/um - Um^(1-alpha)/sum(Um^(1-alpha)))/alpha
    DD1[cond.0] = (1/U1)*(Um/um - Um^(1-alpha)/sum(Um^(1-alpha)))/alpha
  } else if (type == "SREW") {
    ## derivatives of the population-weighted symmetrized between-group Renyi index:
    DD0[cond.0] = ((-1/U0)*(Um/um - Um^(1-alpha)/sum(Um^(1-alpha)))/alpha + (-1/U0)*(Um/um - Um^alpha/sum(Um^alpha))/(1-alpha))/2
    DD1[cond.0] = ((1/U1)*(Um/um - Um^(1-alpha)/sum(Um^(1-alpha)))/alpha + (1/U1)*(Um/um - Um^alpha/sum(Um^alpha))/(1-alpha))/2
  }

  ## variance elements
  s = vector("numeric", length(x))
  for (g in grp.names) {
    s[grp == g] = DD0[g] + DD1[g]*x[grp == g]
  }

  if (!all(is.na(s))) { # should cover is.nan as well
    svobj = update(svobj, s = s)
    bj.vr = attr(svytotal(~s, svobj, na.rm=na.rm), "var")
  }
  return(sqrt(bj.vr))
  
} # between.BJ.seNA


#######################################################################################################

aggregate.BJ.se =         # Taylor Linearization technique in Biewen and Jenkins (2006) and Borrell and Talih (2011) for aggregate index.
  function(type = "REW",  # character indicating index to work with: either REW or SREW
	   alpha=0.5,	  # Inequality aversion parameter for Renyi family of indices 
           xlab,          # response variable label, to be looked up in the dataframe, from which index is to be computed 
           grp.var,       # label of grouping variable, to be looked up in dataframe, from which index is decomposed
           svobj,         # survey design object from 'svyobject', cf. Lumley's 'survey' package
           speed.up=NULL, # option to speed up calculation of sufficient statistics U0, U1, T0, T1: those could be passed as a vector
           na.rm=TRUE)    # what do to with missing values in the response variable and sampling weights
{
    
  # Require survey design object:
  if (is.null(svobj)) {
    stop("A survey design object must be supplied")
  }
  if (!is.element(grp.var, names(svobj$variables))) {
    stop("Variable", grp.var, "not found in survey design object")
  }
  if (!is.element(xlab, names(svobj$variables))) {
    stop("Variable", xlab, "not found in survey design object")
  }

  ## Switch for special limiting cases alpha tends to 0 or 1:
  if (isTRUE(all.equal(alpha, 1))) {
    if (type == "REW") {
     	type = "RL" 	# "equally-weighted" MLD is limiting case when alpha tends to 1
     } else if (type == "SREW") {
     	type = "RS" 	# "equally-weighted" STI is limiting case when alpha tends to 1
     }
  } else if (isTRUE(all.equal(alpha, 0))) {
    if (type == "REW") {
     	type = "RT" 	# "equally-weighted" TI is limiting case when alpha tends to 0
     } else if (type == "SREW") {
     	type = "RS" 	# "equally-weighted" STI is limiting case when alpha tends to 0
     }
  }

  # drop observations with missing group membership info:
  svobj = subset(svobj, !is.na(svobj$variables[,grp.var]))

  grp = svobj$variables[,grp.var]
  x = svobj$variables[,xlab]
  w = 1/svobj$prob

  grp.names = levels(grp)
  if (length(grp.names) < 2) {
    warning("Expecting a factor with at least two levels. Omitting calculation of aggregate.BJ.se.")
    return(NA)
  }
  
  if (is.null(speed.up)) {
    U0 = sapply(grp.names, function(g) sum(w[grp == g], na.rm=na.rm))
    U1 = sapply(grp.names, function(g) sum(w[grp == g]*x[grp == g], na.rm=na.rm))
    Ua = sapply(grp.names, function(g) sum(w[grp == g]*x[grp == g]^alpha, na.rm=na.rm))
    Uaa = sapply(grp.names, function(g) sum(w[grp == g]*x[grp == g]^(1-alpha), na.rm=na.rm))
    T0 = sapply(grp.names, function(g) sum(w[grp == g]*log(x[grp == g]), na.rm=na.rm))
    T1 = sapply(grp.names, function(g) sum(w[grp == g]*ifelse(x[grp == g] > 0, x[grp == g]*log(x[grp == g]), 0), na.rm=na.rm))
  } else { ## if passed internally
    U0 = speed.up$U0
    U1 = speed.up$U1
    Ua = speed.up$Ua
    Uaa = speed.up$Uaa
    T0 = speed.up$T0
    T1 = speed.up$T1
  }
 
  Um = ifelse(U0 > 0, U1/U0, 0)

  cond.0 = (U0 > 0)

  U0 = U0[cond.0] # trim empty groups (needed?)
  U1 = U1[cond.0]
  Ua = Ua[cond.0]
  Uaa = Uaa[cond.0]
  T0 = T0[cond.0]
  T1 = T1[cond.0]
  Um = Um[cond.0]

  u0 = sum(U0)
  u1 = sum(U1)
  ua = sum(Ua)
  uaa = sum(Uaa)
  um = sum(Um)
  mm = length(U1)

  DD0 = vector("numeric", length(grp.names))
  names(DD0) = grp.names
  DD1 = vector("numeric", length(grp.names))
  names(DD1) = grp.names
  DDa = vector("numeric", length(grp.names))
  names(DDa) = grp.names
  DDaa = vector("numeric", length(grp.names))
  names(DDaa) = grp.names
  TT0 = vector("numeric", length(grp.names))
  names(TT0) = grp.names
  TT1 = vector("numeric", length(grp.names))
  names(TT1) = grp.names

  bj.vr = NA
  s = vector("numeric", length(x))
  
  if (type == "RT") {
    ## derivatives of the aggregate equally-weighted Theil index:
    DD0[cond.0] = (Um/U0)/um - (T1/U0^2)/um + (Um/U0)*sum(T1/U0)/um^2
    DD1[cond.0] = -(1/U0)/um - (1/U0)*sum(T1/U0)/um^2
    TT1[cond.0] = (1/U0)/um

    ## variance elements
    for (g in grp.names) {
      s[grp == g] = DD0[g] + DD1[g]*x[grp == g] + TT1[g]*ifelse(x[grp == g] > 0, x[grp == g]*log(x[grp == g]), 0)
    }
  } else if (type == "RL") {
    ## derivatives of the aggregate equally-weighted MLD index:
    DD0[cond.0] = -(Um/U0)/um + (T0/U0^2)/mm
    DD1[cond.0] = (1/U0)/um
    TT0[cond.0] = -(1/U0)/mm

    ## variance elements
    for (g in grp.names) {
      s[grp == g] = DD0[g] + DD1[g]*x[grp == g] + TT0[g]*log(x[grp == g])
    }
  } else if (type == "RS") {
    ## derivatives of the aggregate equally-weighted symmetrized Theil index:
    DD0[cond.0] = ( - (T1/U0^2)/um + (Um/U0)*sum(T1/U0)/um^2 + (T0/U0^2)/mm )/2
    DD1[cond.0] = ( - (1/U0)*sum(T1/U0)/um^2 )/2
    TT1[cond.0] = ((1/U0)/um)/2
    TT0[cond.0] = (-(1/U0)/mm)/2

    ## variance elements
    for (g in grp.names) {
      s[grp == g] = DD0[g] + DD1[g]*x[grp == g] + TT0[g]*log(x[grp == g]) + TT1[g]*ifelse(x[grp == g] > 0, x[grp == g]*log(x[grp == g]), 0)
    }
  } else if (type == "REW") {
    DD0[cond.0] = ( -(1-alpha)*(Um/U0)*(1/um) + (Uaa/U0^2)*(1/sum(Uaa/U0)) )/(alpha*(1-alpha))
    DD1[cond.0] = ((1/U0)*(1/um))/alpha
    DDaa[cond.0] = -((1/U0)*(1/sum(Uaa/U0)))/(alpha*(1-alpha))

    for (g in grp.names) {
      s[grp == g] = DD0[g] + DD1[g]*x[grp == g] + DDaa[g]*x[grp == g]^(1-alpha)
    }
  } else if (type == "SREW") {
    DD0[cond.0] = ( -(Um/U0)*(1/um) + (Uaa/U0^2)*(1/sum(Uaa/U0)) + (Ua/U0^2)*(1/sum(Ua/U0)) ) /(2*alpha*(1-alpha))
    DD1[cond.0] = ((1/U0)*(1/um))/(2*alpha*(1-alpha))
    DDaa[cond.0] = -((1/U0)*(1/sum(Uaa/U0)))/(2*alpha*(1-alpha))
    DDa[cond.0] = -((1/U0)*(1/sum(Ua/U0)))/(2*alpha*(1-alpha))

    for (g in grp.names) {
      s[grp == g] = DD0[g] + DD1[g]*x[grp == g] + DDaa[g]*x[grp == g]^(1-alpha) + DDa[g]*x[grp == g]^alpha
    }
  }
  
  if (!all(is.na(s))) { # should cover is.nan as well
     svobj = update(svobj, s = s)
     bj.vr = attr(svytotal(~s, svobj, na.rm=na.rm), "var") 
  }
  return(sqrt(bj.vr))
  
} # aggregate.BJ.se


#######################################################################################################

compute.BJ.se =           # Taylor Linearization technique in Biewen and Jenkins (2006) and Borrell and Talih (2011) for total index.
  function(type = "T",    # character indicating index to work with: T=Theil, L=MLD, S=STI, etc.
	   alpha=0.5,	  # Inequality aversion parameter for Renyi family of indices 
           xlab,          # response variable label, to be looked up in the dataframe, from which index is to be computed 
           svobj,         # survey design object from 'svyobject', cf. Lumley's 'survey' package
           speed.up=NULL, # option to speed up calculation of sufficient statistics U0, U1, Ua, Uaa: those could be passed as a vector
           na.rm=TRUE)    # what do to with missing values in the response variable and sampling weights
{

  # Require survey design object:
  if (is.null(svobj)) {
    stop("A survey design object must be supplied")
  }
  if (!is.element(xlab, names(svobj$variables))) {
    stop("Variable", xlab, "not found in survey design object")
  }

  ## Switch for special limiting cases alpha tends to 0 or 1:
  if (isTRUE(all.equal(alpha, 1))) {
     if (type == "RPW") {
     	type = "L"	# MLD is limiting case when alpha tends to 1
     } else if (type == "SRPW") {
     	type = "S" 	# STI is limiting case when alpha tends to 1
     } else if (type == "REW") {
     	type = "RL" 	# "equally-weighted" MLD is limiting case when alpha tends to 1
     } else if (type == "SREW") {
     	type = "RS" 	# "equally-weighted" STI is limiting case when alpha tends to 1
     }
  } else if (isTRUE(all.equal(alpha, 0))) {
     if (type == "RPW") {
     	type = "T"	# TI is limiting case when alpha tends to 0
     } else if (type == "SRPW") {
     	type = "S" 	# STI is limiting case when alpha tends to 0
     } else if (type == "REW") {
     	type = "RT" 	# "equally-weighted" TI is limiting case when alpha tends to 0
     } else if (type == "SREW") {
     	type = "RS" 	# "equally-weighted" STI is limiting case when alpha tends to 0
     }
  }

  x = svobj$variables[,xlab]
  w = 1/svobj$prob

  if (is.null(speed.up)) {
    U0 = sum(w, na.rm=na.rm)
    U1 = sum(w*x, na.rm=na.rm)
    Ua = sum(w*x^alpha, na.rm=na.rm)
    Uaa = sum(w*x^(1-alpha), na.rm=na.rm)
    T0 = sum(w*log(x), na.rm=na.rm)
    T1 = sum(w*ifelse(x > 0, x*log(x), 0), na.rm=na.rm)
  } else { ## if passed internally
    U0 = speed.up$U0
    U1 = speed.up$U1
    Ua = speed.up$Ua
    Uaa = speed.up$Uaa
    T0 = speed.up$T0
    T1 = speed.up$T1
  } 

  bj.vr = NA
  
  if (U1 > 0) { # variance elements for total index
    if (type == "T"  | type == "RT") { 
       s = ifelse(x > 0, x*log(x), 0)/U1 - x*(T1/U1 + 1)/U1 + 1/U0
    } else if (type == "L" | type == "RL") { # variance elements eq. 18 in Biewen & Jenkins (2006)
      s = - log(x)/U0 + x/U1 + (T0/U0 - 1)/U0
    } else if (type == "S" | type == "RS") {
      s = ( ifelse(x > 0, x*log(x), 0)/U1 - log(x)/U0 - x*(T1/U1)/U1 + (T0/U0)/U0 )/2
    } else if (type == "RPW" | type == "REW") {
      s = (-x^(1-alpha)/Uaa + (1-alpha)*x/U1 + alpha/U0)/(alpha*(1-alpha))
    } else if (type == "SRPW" | type == "SREW") {
      s = (-x^(1-alpha)/Uaa - x^alpha/Ua + x/U1 + 1/U0)/(2*alpha*(1-alpha))
    }

    if (!all(is.na(s))) { # should cover is.nan as well
      svobj = update(svobj, s = s)
      bj.vr = attr(svytotal(~s, svobj, na.rm=na.rm), "var")
    }
  } 
  return(sqrt(bj.vr))
} # compute.BJ.se

####################################################################################################

compute.MC.se =         # Implements std error calculation using CLT for empirical KDE (Martinez-Camblor, 2007)
  function(type = "T",  # character indicating index to work with: T=Theil, L=MLD, S=STI.
	   alpha=0.5,	# Inequality aversion parameter for Renyi family of indices
           x,           # response variable
           w,           # weights
           ...)         # ... includes 'from', 'kernel', 'bw' passed to density()
                        # set 'from = 0' to ensure support on the positive axis
{
  if (any(is.na(x))) {
    stop("This implementation does not support missing values.")
  }
  
  n = length(x)

  if (n < 2) {
    warning("Need at least two-points for current implementation.")
    return(NA)
  }
  
  ## Switch for special limiting cases alpha tends to 0 or 1:
  if (isTRUE(all.equal(alpha, 1))) {
     if (type == "RPW") {
     	type = "L"	# MLD is limiting case when alpha tends to 1
     } else if (type == "SRPW") {
     	type = "S" 	# STI is limiting case when alpha tends to 1
     } else if (type == "REW") {
     	type = "RL" 	# "equally-weighted" MLD is limiting case when alpha tends to 1
     } else if (type == "SREW") {
     	type = "RS" 	# "equally-weighted" STI is limiting case when alpha tends to 1
     }
  } else if (isTRUE(all.equal(alpha, 0))) {
     if (type == "RPW") {
     	type = "T"	# TI is limiting case when alpha tends to 0
     } else if (type == "SRPW") {
     	type = "S" 	# STI is limiting case when alpha tends to 0
     } else if (type == "REW") {
     	type = "RT" 	# "equally-weighted" TI is limiting case when alpha tends to 0
     } else if (type == "SREW") {
     	type = "RS" 	# "equally-weighted" STI is limiting case when alpha tends to 0
     }
  }

  U0 = sum(w)
  U1 = sum(w*x)
  mu = U1/U0
  
  # Kernel density estimate (KDE) of the probability density function
  tmp.den = density(x=x, weights=w/U0, ...) 
  n.trp = length(tmp.den$y)
      
  # Approximate smoothed cumulative distribution function (SCDF) estimate via trapezoid rule:
  tmp.dis = list(x = tmp.den$x[-1], 
  y = cumsum((tmp.den$y[-1] + tmp.den$y[-n.trp])/2*diff(tmp.den$x)))             

  # Theil/MLD index must be re-computed from the SCDF. Integral approximations done via trapezoid rule.
  if (type == "T") { 
    tmp.index.r = ifelse(tmp.den$x[-1] > 0, tmp.den$x[-1]/mu * log(tmp.den$x[-1]/mu), 0) * tmp.den$y[-1]
    tmp.index.l = ifelse(tmp.den$x[-n.trp] > 0, tmp.den$x[-n.trp]/mu * log(tmp.den$x[-n.trp]/mu), 0) * tmp.den$y[-n.trp]
    tmp.index = sum((tmp.index.l + tmp.index.r)/2*diff(tmp.den$x))
    
    tmp2 = (tmp.index - log(tmp.dis$x/mu))/mu  # corrected typo in martinez-camblor 2007
  } else if (type == "L") {
    tmp2 = 1/tmp.dis$x - 1/mu   # cf. own derivation
  } else if (type == "S") {
    tmp.index.r = ifelse(tmp.den$x[-1] > 0, tmp.den$x[-1]/mu * log(tmp.den$x[-1]/mu), 0) * tmp.den$y[-1]
    tmp.index.l = ifelse(tmp.den$x[-n.trp] > 0, tmp.den$x[-n.trp]/mu * log(tmp.den$x[-n.trp]/mu), 0) * tmp.den$y[-n.trp]
    tmp.index = sum((tmp.index.l + tmp.index.r)/2*diff(tmp.den$x))
    
    tmp2 = ((tmp.index - log(tmp.dis$x/mu))/mu + 1/tmp.dis$x - 1/mu)/2 # check??
  }
      
  n.dis = length(tmp.dis$y)
  
  tmp.r = (tmp.dis$y[-1]*tmp2[-1]) %o% ((1-tmp.dis$y[-1])*tmp2[-1]) 
  tmp.r[lower.tri(tmp.r)] = t(tmp.r)[lower.tri(t(tmp.r))] # symmetrize
  
  tmp.l = (tmp.dis$y[-n.dis]*tmp2[-n.dis]) %o% ((1-tmp.dis$y[-n.dis])*tmp2[-n.dis]) 
  tmp.l[lower.tri(tmp.l)] = t(tmp.l)[lower.tri(t(tmp.l))] # symmetrize	        
  tmp.d = diff(tmp.dis$x)%o%diff(tmp.dis$x)               # dxdy elements
  
  mc.vr = sum((tmp.r + tmp.l)/2 *tmp.d)/n     # integral approximation via trapezoid rule

  return(sqrt(mc.vr))

} # compute.MC.se

####################################################################################################

theil.index =               # Returns a list with the computed index, normalized index, welfare function,
                            # MC-based SE and 95% CI, BJ-based SE and 95% CI,
                            # mean for x, SRS-based SE for the mean,
                            # sample size (n), overall sum of the weights (U0), overall aggregate share of x (U1), and other total stats. 
  function(type = "T",      # string indicating index to work with: T=Theil, L=MLD, S=STI
                            # REW=Renyi-equally-weighted, RPW = Renyi-population-weighted, SREW or SRPW = Symmetrized Renyi.
           alpha = 0.5,     # inequality aversion parameter for the Renyi family of indices
           x=NULL,          # response variable for which index is to be computed
           w=NULL,          # weights
           xlab="",         # variable label for response variable to look up in survey.design object
           svobj=NULL,      # Either a survey design object or the pair 'x,w' must be specified. If both are passed, svobj overrides.
           na.rm=TRUE,      # what to do with missing values in x or w.
           compute.MC=TRUE, # whether to compute M-C std errors
           compute.BJ=TRUE, # whether to compute BJ std errors
           ...)             # additional parameters to pass to compute.MC.se() if needed

{
  if (is.null(svobj)) { # for a non-design based calculation
    if (is.null(x) | is.null(w)) {
      stop("Either a survey design object or the pair 'x,w' must be specified.")
    }
    if (length(w) != length(x)) {
      stop("Parameters x and w must have the same length")
    }
  } else {
    if (!is.element(xlab, names(svobj$variables))) {
      stop(paste("Variable", xlab, "not found in survey.design object"))
    }
    if (!is.null(x)) {
      warning("Supplied value 'x' over-ridden from 'svobj'")
    }
    x = svobj$variables[,xlab]
    if (is.null(w)) { # only over-ride w if NULL to allow for bootstrap/BRR weights
      w = 1/svobj$prob
    } else {
      if (length(w) != length(x)) {
        stop("Parameters x and w must have the same length")
      }
    }
  } 
  if (na.rm) {
    w = w[!is.na(x)]
    x = x[!is.na(x)]
  }    
  if (any(is.na(w))) {
    stop("Sampling weights cannot be NA")		
  }
  if (any(w < 0)) {
    stop("Sampling weights cannot be negative")			
  }	
  if (any(w > 0 & w < 1)) {
    stop("Positive sampling weights must be at least one")			
  }

  ## Switch for special limiting cases alpha tends to 0 or 1:
  if (isTRUE(all.equal(alpha, 1))) {
     if (type == "RPW") {
     	type = "L"	# MLD is limiting case when alpha tends to 1
     } else if (type == "SRPW") {
     	type = "S" 	# STI is limiting case when alpha tends to 1
     } else if (type == "REW") {
      	type = "RL" 	# "equally-weighted" MLD is limiting case when alpha tends to 1
     } else if (type == "SREW") {
     	type = "RS" 	# "equally-weighted" STI is limiting case when alpha tends to 1
     }
  } else if (isTRUE(all.equal(alpha, 0))) {
     if (type == "RPW") {
     	type = "T"	# TI is limiting case when alpha tends to 0
     } else if (type == "SRPW") {
     	type = "S" 	# STI is limiting case when alpha tends to 0
     } else if (type == "REW") {
      	type = "RT" 	# "equally-weighted" TI is limiting case when alpha tends to 0
     } else if (type == "SREW") {
     	type = "RS" 	# "equally-weighted" STI is limiting case when alpha tends to 0
     }
  }

  ## set-up default values:
  index = 0
  welfare = 0
  norm.index = 0
  mc.se = NA
  mc.CI.lo = NA
  mc.CI.hi = NA
  bj.se = NA
  bj.CI.lo = NA
  bj.CI.hi = NA
  
  n = length(x)				 # sample size  
  U0 = sum(w)    			 # corresponding estimated population size
  U1 = sum(w*x) 			 # Note: U1 will be 0 if x is empty or all zeroes
  Ua = sum(w*x^alpha)
  Uaa = sum(w*x^(1-alpha))       
  T0 = sum(w*log(x))                     # Note: T0 might be -Inf if some x is 0
  T1 = sum(w*ifelse(x > 0, x*log(x), 0)) # x*log(x) set to zero if x is 0

  ## Sample Mean:
  mu = U1/U0

  ## SRS Sample Variance, ignoring stratification/clustering
  U2 = sum(w*x^2)
  V2 = sum(w^2)
  vr = (U0*U2 - U1^2)/(U0^2 - V2)

  if (U1 > 0) {  ## if U1 = 0, or type is other than specified, return default values as defined above
    
    if (type == "T" | type == "RT") {  ## Theil index
      index = T1/U1 - log(mu)
      if (U0 > 1) {  ## max(Theil index) is ln(U0)
        norm.index = U0/(U0-1) * (1 - exp(-index))
        welfare = exp(index)*mu
      }
    } else if (type == "L" | type == "RL") {  ## MLD index
      index = -T0/U0 + log(mu)
      norm.index = 1 - exp(-index)  ## MLD index is unbounded
      welfare = exp(index)*mu
    } else if (type == "S" | type == "RS") {  ## symmetrized Theil index
      index = (T1/U1 - T0/U0)/2
      norm.index = 1 - exp(-index)  ## index is unbounded
      welfare = exp(index)*mu  # so welfare(S) = sqrt(welfare(T))*sqrt(welfare(L))
    } else if (type == "RPW" | type == "REW") {
      index = (alpha*log(U0) + (1-alpha)*log(U1) - log(Uaa))/(alpha*(1-alpha))
      norm.index = 1 - exp(-index)
      welfare = exp(index)*mu
    } else if (type == "SRPW" | type == "SREW") {
      index = (log(U0) + log(U1) - log(Ua) - log(Uaa))/(2*alpha*(1-alpha))
      norm.index = 1 - exp(-index)
      welfare = exp(index)*mu
    }
    
    if (compute.MC) {
      ## Central Limit theorem of Martinez-Camblor 2007:
      mc.se = compute.MC.se(type, x, w, ...)
      mc.CI.lo = index - mc.se*1.959964
      mc.CI.hi = index + mc.se*1.959964
    }
    if (compute.BJ & !is.null(svobj)) {
      ## Taylor Linearization, following Borrell and Talih (2011) and Biewen and Jenkins (2006): 
      bj.se = compute.BJ.se(type, alpha, xlab, svobj, list(U0=U0, U1=U1, Ua=Ua, Uaa=Uaa, T0=T0, T1=T1), na.rm)
      bj.CI.lo = index - bj.se*1.959964
      bj.CI.hi = index + bj.se*1.959964
    }
  }

  return(list(index=index,
              norm.index=norm.index,
              welfare=welfare,
              mc.se=mc.se, mc.CI.lo=mc.CI.lo, mc.CI.hi=mc.CI.hi,
              bj.se=bj.se, bj.CI.lo=bj.CI.lo, bj.CI.hi=bj.CI.hi,
              mean=mu, srs.se=sqrt(vr*V2)/U0,
              n=n, N=U0, share=U1, Ua=Ua, Uaa=Uaa, T0=T0, T1=T1))

} # theil.index

#######################################################################################################

theil.decomposition =        # Recursive function to implement recursive decomposition of the Theil/MLD index
  function(type="T",         # character indicating index to work with: T=Theil, L=MLD, S=STI, etc.
           alpha = 0.5,      # inequality aversion parameter for the Renyi family of indices
           data=NULL,        # Either a survey design object or a data frame must be specified
           response="",      # variable label for response variable to look up in data frame
           w.var="",         # variable label for sampling weight variable to look up in data frame
           svobj=NULL,       # Either a survey design object or a data frame must be specified
           grouping=NULL,    # vector of labels for grouping variables to use for recursive index decomposition
           na.rm=TRUE,       # what to do with missing values in the response or weighting variables.
           compute.MC=TRUE,  # whether to compute M-C std errors
           compute.BJ=TRUE,  # whether to compute BJ std errors
           w.alt=NULL,       # alternative vector of BRR/rescaled bootstrap weights to use
           ...)              # additional parameters to pass to compute.MC.se()

{

  ### Preliminary set-up:
  
  if (is.null(svobj)) { # for a non-design based calculation
    if (is.null(data)) {
      stop("Either a survey design object or a data frame must be specified.")
    }
    if (!is.element(response, names(data))) {
      stop(paste("Variable", response, "not found in data frame"))
    }
    x = data[, response]
    
    if (is.null(w.alt)) {
      if (!is.element(w.var, names(data))) {
        stop(paste("Variable", w.var, "not found in data frame"))
      }
      w = data[, w.var]
    } else { # vector of BRR/rescaled bootstrap weights
      if (length(w.alt) != nrow(data)) {
        stop("Specified weight vector should have same number of observations as in the data")
      }
      w = w.alt
    }
  } else { # for a design-based calculation
    w = w.alt # will over-ride weights in svobj if non NULL
  }
 
  ## Switch for special limiting cases alpha tends to 0 or 1:
  if (isTRUE(all.equal(alpha, 1))) {
     if (type == "RPW") {
     	type = "L"	# MLD is limiting case when alpha tends to 1
     } else if (type == "SRPW") {
     	type = "S" 	# STI is limiting case when alpha tends to 1
     } else if (type == "REW") {
     	type = "RL" 	# "equally-weighted" MLD is limiting case when alpha tends to 1
     } else if (type == "SREW") {
        type = "RS" 	# "equally-weighted" STI is limiting case when alpha tends to 1
     }
  } else if (isTRUE(all.equal(alpha, 0))) {
     if (type == "RPW") {
     	type = "T"	# TI is limiting case when alpha tends to 0
     } else if (type == "SRPW") {
     	type = "S" 	# STI is limiting case when alpha tends to 0
     } else if (type == "REW") {
     	type = "RT" 	# "equally-weighted" TI is limiting case when alpha tends to 0
     } else if (type == "SREW") {
     	type = "RS" 	# "equally-weighted" STI is limiting case when alpha tends to 0
     }
  }

  ### Compute level 1 index:
  
  if (is.null(svobj)) {
    ALL = unlist(theil.index(type=type, alpha=alpha, x=x, w=w, na.rm=na.rm, compute.MC=compute.MC, compute.BJ=compute.BJ, ...))
  } else {
    ALL = unlist(theil.index(type=type, alpha=alpha, xlab=response, w=w, svobj=svobj, na.rm=na.rm, compute.MC=compute.MC, compute.BJ=compute.BJ, ...))    
  }
      
  ALL = t(t(c(ALL[1:2], ALL[-c(1:2)])))
  colnames(ALL) = c("ALL")
  rownames(ALL)[c(1:2)] = c("index", "norm.index")
  
  ### Base case: If no grouping variable return level 1 index.
      
  if (is.null(grouping)) {
    return(list(TTs = ALL, 
                rel = matrix(1, 2, 1, dimnames=list(c("rel.n", "rel.share"), c("ALL"))),
                relG = NULL,
                overall.index = list(index = ALL["index",1],
                                     mc.se = ALL["mc.se", 1],
                                     mc.CI = c(ALL["mc.CI.lo",1], ALL["mc.CI.hi",1]),
                                     bj.se = ALL["bj.se", 1],
                                     bj.CI = c(ALL["bj.CI.lo",1], ALL["bj.CI.hi",1])),
                between.grp.index = list(index = 0, mc.se = 0, mc.CI = c(0, 0), bj.se = 0, bj.CI = c(0,0)),
                wtd.wthn.grp.index = list(index = ALL["index",1],
                                     mc.se = ALL["mc.se", 1],
                                     mc.CI = c(ALL["mc.CI.lo",1], ALL["mc.CI.hi",1]),
                                     bj.se = ALL["bj.se", 1],
                                     bj.CI = c(ALL["bj.CI.lo",1], ALL["bj.CI.hi",1])), 
                prop.ineq.from.between = 0))
  }
    
  ### Recursion main: For one grouping variable.

  if (length(grouping) == 1) {
    
    if (is.null(svobj)) {

      ## deal explicitely with NA category
      anyNA = any(is.na(data[,grouping[1]]))

      if (anyNA) {
        NAs = unlist(theil.index(type=type, alpha=alpha,
                                 x=x[is.na(data[,grouping[1]])],
                                 w=w[is.na(data[,grouping[1]])],
                                 na.rm=na.rm, compute.MC=compute.MC, compute.BJ=compute.BJ,...))
      }
      
      grp.levels = levels(data[,grouping[1]])

      if (length(grp.levels) < 2) {
        stop("\n Expecting a factor with at least two levels in theil.decomposition.\n")
      }
            
      TTs = sapply(grp.levels, 
        function(y) unlist(theil.index(type=type, alpha=alpha,
                                       x = x[!is.na(data[,grouping[1]]) & data[,grouping[1]] == y],  
                                       w = w[!is.na(data[,grouping[1]]) & data[,grouping[1]] == y],           
                                       na.rm = na.rm, compute.MC=compute.MC, compute.BJ = compute.BJ, ...))
        )


    } else {
      
      ## repeat for svobj...

      anyNA = any(is.na(svobj$variables[,grouping[1]]))

      if (anyNA) {
        NAs = unlist(theil.index(type = type, alpha=alpha,
                                 xlab = response,
                                 w = w[is.na(svobj$variables[,grouping[1]])],
                                 svobj = subset(svobj, is.na(svobj$variables[,grouping[1]])),
                                 na.rm = na.rm, compute.MC=compute.MC, compute.BJ = compute.BJ,...))
      }
      
      grp.levels = levels(svobj$variables[,grouping[1]])
      
      if (length(grp.levels) < 2) {
        stop("\n Expecting a factor with at least two levels in theil.decomposition.\n")
      }
            
      TTs = sapply(grp.levels, 
        function(y) unlist(theil.index(type = type, alpha=alpha,
                                       xlab = response,
                                       w = w[!is.na(svobj$variables[,grouping[1]]) & (svobj$variables[,grouping[1]] == y)],
                                       svobj = subset(svobj, !is.na(svobj$variables[,grouping[1]]) & (svobj$variables[,grouping[1]] == y)),
                                       na.rm = na.rm, compute.MC=compute.MC, compute.BJ = compute.BJ, ...))
        )


    }

    ## add <NA> category to table:

    TTsG = TTs

    if (anyNA) {
      NAs = t(t(c(NAs[1:2], NAs[-c(1:2)])))
      colnames(NAs) = c("<NA>")
      rownames(NAs)[c(1:2)] = c("index", "norm.index")
      TTs = cbind(TTs, NAs)
    }
    
    ### relative frequencies and shares including NAs:
    
    rel.n = TTs["N",] / ALL["N", 1]
    rel.share = TTs["share",] / ALL["share", 1]

    ### relative frequencies and shares not including NAs:

    rel.nG = NULL
    rel.shareG = NULL
    if (anyNA) {
      rel.nG = TTsG["N",] / sum(TTsG["N",])
      rel.shareG = TTsG["share",] / sum(TTsG["share",])
    }

    ### Overall index from previous hierachical level:
    
    overall.index = ALL["index", 1]
    overall.mc.se = ALL["mc.se", 1]
    overall.mc.CI.lo = ALL["mc.CI.lo", 1] 
    overall.mc.CI.hi = ALL["mc.CI.hi", 1]
    overall.bj.se = ALL["bj.se", 1]
    overall.bj.CI.lo = ALL["bj.CI.lo", 1] 
    overall.bj.CI.hi = ALL["bj.CI.hi", 1]

    ### initialize:
    
    wtd.wthn.grp.mc.se = NA
    wtd.wthn.grp.mc.CI.lo = NA
    wtd.wthn.grp.mc.CI.hi = NA
    between.grp.mc.se = NA
    between.grp.mc.CI.lo = NA
    between.grp.mc.CI.hi = NA
    wtd.wthn.grp.bj.se = NA
    wtd.wthn.grp.bj.CI.lo = NA
    wtd.wthn.grp.bj.CI.hi = NA
    between.grp.bj.se = NA
    between.grp.bj.CI.lo = NA
    between.grp.bj.CI.hi = NA

    ### Aggregate index (if different from overall, e.g., for the REW)
    
    aggregate.index = NA
    aggregate.mc.se = NA
    aggregate.mc.CI.lo = NA
    aggregate.mc.CI.hi = NA
    aggregate.bj.se = NA
    aggregate.bj.CI.lo = NA
    aggregate.bj.CI.hi = NA

    ### Compute weighted-within group index and between-group index for non NA categories only

    U0 = TTsG["N", ]
    U1 = TTsG["share", ]
    Ua = TTsG["Ua", ]
    Uaa = TTsG["Uaa", ]
    T0 = TTsG["T0", ]
    T1 = TTsG["T1", ]
    u0 = sum(U0)
    u1 = sum(U1)
    ua = sum(Ua)
    uaa = sum(Uaa)   

    if (type == "T") {
      contrib.to.wthn = (U1/u1)*TTsG["index", ]
      contrib.to.btwn = (U1/u1)*log(U1/U0)

      if (compute.MC) {
        contrib.to.wthn.mc.se = (U1/u1)^2*TTsG["mc.se", ]^2
      }

      wtd.wthn.grp.index = sum(contrib.to.wthn)
      between.grp.index = sum(contrib.to.btwn) - log(u1/u0)
    } else if (type == "L") {
      contrib.to.wthn = (U0/u0)*TTsG["index",]
      contrib.to.btwn = - (U0/u0)*log(U1/U0)

      if (compute.MC) {
        contrib.to.wthn.mc.se = (U0/u0)^2*TTsG["mc.se", ]^2
      }

      wtd.wthn.grp.index = sum(contrib.to.wthn)
      between.grp.index = sum(contrib.to.btwn) + log(u1/u0)
    } else if (type == "S") { ## use total stats...
      contrib.to.wthn = ((U0/u0)*log(U1/U0) - (1/u0)*T0 - (U1/u1)*log(U1/U0) + (1/u1)*T1)/2
      contrib.to.btwn = ((U1/u1 - U0/u0)*log(U1/U0))/2

      wtd.wthn.grp.index = sum(contrib.to.wthn)
      between.grp.index = sum(contrib.to.btwn)
    } else if (type == "RT") {
      contrib.to.wthn = ((U1/U0)/sum(U1/U0))*TTsG["index", ]       
      contrib.to.btwn = ((U1/U0)/sum(U1/U0))*log(U1/U0)

      wtd.wthn.grp.index = sum(contrib.to.wthn)
      between.grp.index = sum(contrib.to.btwn) - log(sum(U1/U0)/length(U0))

      aggregate.index = log(length(U0)) - log(sum(U1/U0)) + sum(T1/U0)/sum(U1/U0)
    } else if (type == "RL") {
      contrib.to.wthn = (1/length(U0))*TTsG["index", ]
      contrib.to.btwn = -(1/length(U0))*log(U1/U0)

      wtd.wthn.grp.index = sum(contrib.to.wthn)
      between.grp.index = sum(contrib.to.btwn) + log(sum(U1/U0)/length(U0))

      aggregate.index = - log(length(U0)) + log(sum(U1/U0)) - sum(T0/U0)/length(U0)
    } else if (type == "RS") { ## Use total stats...
      contrib.to.wthn = ( (T1/U0 - (U1/U0)*log(U1/U0))/sum(U1/U0) + (log(U1/U0) - T0/U0)/length(U0) )/2
      contrib.to.btwn = ( ((U1/U0)/sum(U1/U0))*log(U1/U0) - (1/length(U0))*log(U1/U0) )/2

      wtd.wthn.grp.index = sum(contrib.to.wthn)
      between.grp.index = sum(contrib.to.btwn)

      aggregate.index = (sum(T1/U0)/sum(U1/U0) - sum(T0/U0)/length(U0))/2
    } else if (type == "RPW") {
      contrib.to.wthn = (Uaa/uaa)*exp(alpha*(1-alpha)*TTsG["index",])
      contrib.to.btwn = U0^alpha*U1^(1-alpha)

      wtd.wthn.grp.index = log(sum(contrib.to.wthn))/(alpha*(1-alpha))
      between.grp.index = (alpha*log(u0) + (1-alpha)*log(u1) - log(sum(contrib.to.btwn)))/(alpha*(1-alpha))
    } else if (type == "SRPW") {

      ## need RPW indices with alpha and 1-alpha as well for within-group contributions...
      TTs.a.tmp = sapply(grp.levels, 
        function(y) unlist(theil.index(type = "RPW", alpha=alpha,
                                       xlab = response,
                                       w = w[!is.na(svobj$variables[,grouping[1]]) & (svobj$variables[,grouping[1]] == y)],
                                       svobj = subset(svobj, !is.na(svobj$variables[,grouping[1]]) & (svobj$variables[,grouping[1]] == y)),
                                       na.rm = na.rm, compute.MC=compute.MC, compute.BJ = compute.BJ, ...))
        )

      TTs.aa.tmp = sapply(grp.levels, 
        function(y) unlist(theil.index(type = "RPW", alpha=1-alpha,
                                       xlab = response,
                                       w = w[!is.na(svobj$variables[,grouping[1]]) & (svobj$variables[,grouping[1]] == y)],
                                       svobj = subset(svobj, !is.na(svobj$variables[,grouping[1]]) & (svobj$variables[,grouping[1]] == y)),
                                       na.rm = na.rm, compute.MC=compute.MC, compute.BJ = compute.BJ, ...))
        )


      contrib.to.wthn.a = (Uaa/uaa)*exp(alpha*(1-alpha)*TTs.a.tmp["index",])
      contrib.to.wthn.aa = (Ua/ua)*exp(alpha*(1-alpha)*TTs.aa.tmp["index",])
      contrib.to.btwn.a = U0^alpha*U1^(1-alpha)
      contrib.to.btwn.aa = U0^(1-alpha)*U1^alpha

      rm(TTs.a.tmp, TTs.aa.tmp) # clean-up

      wtd.wthn.grp.index = (log(sum(contrib.to.wthn.a)) + log(sum(contrib.to.wthn.aa)))/(2*alpha*(1-alpha))
      between.grp.index = (log(u0) + log(u1) - log(sum(contrib.to.btwn.a)) - log(sum(contrib.to.btwn.aa)))/(2*alpha*(1-alpha))
    } else if (type == "REW") {
      contrib.to.wthn = ((Uaa/U0)/sum(Uaa/U0))*exp(alpha*(1-alpha)*TTsG["index",])
      contrib.to.btwn = (U1/U0)^(1-alpha)

      wtd.wthn.grp.index = log(sum(contrib.to.wthn))/(alpha*(1-alpha))
      between.grp.index = (alpha*log(length(U0)) + (1-alpha)*log(sum(U1/U0)) - log(sum(contrib.to.btwn)))/(alpha*(1-alpha))

      aggregate.index = (alpha*log(length(U0)) + (1-alpha)*log(sum(U1/U0)) - log(sum(Uaa/U0)))/(alpha*(1-alpha))
    } else if (type == "SREW") {

      ## need REW indices with alpha and 1-alpha as well for within-group contributions...
      TTs.a.tmp = sapply(grp.levels, 
        function(y) unlist(theil.index(type = "REW", alpha=alpha,
                                       xlab = response,
                                       w = w[!is.na(svobj$variables[,grouping[1]]) & (svobj$variables[,grouping[1]] == y)],
                                       svobj = subset(svobj, !is.na(svobj$variables[,grouping[1]]) & (svobj$variables[,grouping[1]] == y)),
                                       na.rm = na.rm, compute.MC=compute.MC, compute.BJ = compute.BJ, ...))
        )

      TTs.aa.tmp = sapply(grp.levels, 
        function(y) unlist(theil.index(type = "REW", alpha=1-alpha,
                                       xlab = response,
                                       w = w[!is.na(svobj$variables[,grouping[1]]) & (svobj$variables[,grouping[1]] == y)],
                                       svobj = subset(svobj, !is.na(svobj$variables[,grouping[1]]) & (svobj$variables[,grouping[1]] == y)),
                                       na.rm = na.rm, compute.MC=compute.MC, compute.BJ = compute.BJ, ...))
        )


      contrib.to.wthn.a = ((Uaa/U0)/sum(Uaa/U0))*exp(alpha*(1-alpha)*TTs.a.tmp["index",])
      contrib.to.wthn.aa = ((Ua/U0)/sum(Ua/U0))*exp(alpha*(1-alpha)*TTs.aa.tmp["index",])
      contrib.to.btwn.a = (U1/U0)^(1-alpha)
      contrib.to.btwn.aa = (U1/U0)^alpha

      rm(TTs.a.tmp, TTs.aa.tmp) # clean-up

      wtd.wthn.grp.index = (log(sum(contrib.to.wthn.a)) + log(sum(contrib.to.wthn.aa)))/(2*alpha*(1-alpha))
      between.grp.index = (log(length(U0)) + log(sum(U1/U0)) - log(sum(contrib.to.btwn.a)) - log(sum(contrib.to.btwn.aa)))/(2*alpha*(1-alpha))

      aggregate.index = (log(length(U0)) + log(sum(U1/U0)) - log(sum(Uaa/U0)) - log(sum(Ua/U0)))/(2*alpha*(1-alpha))
    }
    
    if (compute.MC) {

      ## Asymptotic SEs and 95% Confidence Intervals:
      
      wtd.wthn.grp.mc.se = sqrt(sum(contrib.to.wthn.mc.se))
      wtd.wthn.grp.mc.CI.lo = wtd.wthn.grp.index - wtd.wthn.grp.mc.se*1.959964
      wtd.wthn.grp.mc.CI.hi = wtd.wthn.grp.index + wtd.wthn.grp.mc.se*1.959964

      between.grp.mc.CI.hi = overall.mc.CI.hi - wtd.wthn.grp.mc.CI.lo
      between.grp.mc.CI.lo = overall.mc.CI.lo - wtd.wthn.grp.mc.CI.hi      

      if (!is.na(between.grp.mc.CI.lo) & between.grp.mc.CI.lo <= 0) { # no lower bound (yet!)
        between.grp.mc.CI.lo = NA
      }
    }
    if (compute.BJ) {

      ## Taylor Linearization SEs and 95% Confidence Intervals:
      
      wtd.wthn.grp.bj.se = within.BJ.se(type=type, alpha=alpha, xlab=response, grp.var=grouping[1], svobj=svobj, speed.up=list(U0=U0, U1=U1, Ua=Ua, Uaa=Uaa, T0=T0, T1=T1), na.rm=na.rm)
      wtd.wthn.grp.bj.CI.lo = wtd.wthn.grp.index - wtd.wthn.grp.bj.se*1.959964
      wtd.wthn.grp.bj.CI.hi = wtd.wthn.grp.index + wtd.wthn.grp.bj.se*1.959964

      between.grp.bj.se = between.BJ.se(type=type, alpha=alpha, xlab=response, grp.var=grouping[1], svobj=svobj, speed.up=list(U0=U0, U1=U1), na.rm=na.rm)
      between.grp.bj.CI.lo = between.grp.index - between.grp.bj.se*1.959964
      between.grp.bj.CI.hi = between.grp.index + between.grp.bj.se*1.959964

      if (type == "REW" | type == "SREW") {
        aggregate.bj.se = aggregate.BJ.se(type=type, alpha=alpha, xlab=response, grp.var=grouping[1], svobj=svobj, speed.up=list(U0=U0, U1=U1, Ua=Ua, Uaa=Uaa, T0=T0, T1=T1), na.rm=na.rm)
        aggregate.bj.CI.lo = aggregate.index - aggregate.bj.se*1.959964
        aggregate.bj.CI.hi = aggregate.index + aggregate.bj.se*1.959964 
      }      
    }
    
    TTs = cbind(ALL, TTs)
    
    if (type == "REW" | type == "SREW") {
      overall.index.list = list(index = aggregate.index,
                                mc.se = aggregate.mc.se,
                                mc.CI = c(aggregate.mc.CI.lo, aggregate.mc.CI.hi),
                                bj.se = aggregate.bj.se,
                                bj.CI = c(aggregate.bj.CI.lo, aggregate.bj.CI.hi))
    } else {
      overall.index.list = list(index = overall.index,
                                mc.se = overall.mc.se,
                                mc.CI = c(overall.mc.CI.lo, overall.mc.CI.hi),
                                bj.se = overall.bj.se,
                                bj.CI = c(overall.bj.CI.lo, overall.bj.CI.hi))
    }

    return(list(TTs=TTs,  
                rel = rbind(rel.n, rel.share),
                relG = rbind(rel.nG, rel.shareG),
                overall.index = overall.index.list,
                between.grp.index = list(index = between.grp.index,
                                         mc.se = between.grp.mc.se,
                                         mc.CI = c(between.grp.mc.CI.lo, between.grp.mc.CI.hi),
                                         bj.se = between.grp.bj.se,
                                         bj.CI = c(between.grp.bj.CI.lo, between.grp.bj.CI.hi)),
                wtd.wthn.grp.index = list(index = wtd.wthn.grp.index,
                                          mc.se = wtd.wthn.grp.mc.se,
                                          mc.CI = c(wtd.wthn.grp.mc.CI.lo, wtd.wthn.grp.mc.CI.hi),
                                          bj.se = wtd.wthn.grp.bj.se,           
                                          bj.CI = c(wtd.wthn.grp.bj.CI.lo, wtd.wthn.grp.bj.CI.hi)),
                prop.ineq.from.between = between.grp.index/overall.index))

  }
  
  ## Recursion step:

  if (is.null(svobj)) {

    ## deal explicitely with NA category...
    anyNA = any(is.na(data[,grouping[1]]))
    if (anyNA) {
      NAss = theil.decomposition(type = type, alpha=alpha,
                                 data = data[is.na(data[,grouping[1]]), ], 
                                 response = response,
                                 w.var = w.var,
                                 grouping = grouping[-1],
                                 na.rm = na.rm,
                                 compute.MC=compute.MC, compute.BJ=compute.BJ, 
                                 w.alt = w.alt[is.na(data[,grouping[1]])], ...)
    }

    ## non-NA categories...
    grp.levels = levels(data[,grouping[1]])
    TTlist = lapply(grp.levels, 
      function(y) theil.decomposition(type = type, alpha=alpha,
                                      data = data[!is.na(data[,grouping[1]]) & data[,grouping[1]] == y, ], 
                                      response = response,
                                      w.var = w.var,
                                      grouping = grouping[-1],
                                      na.rm = na.rm,
                                      compute.MC=compute.MC, compute.BJ=compute.BJ, 
                                      w.alt = w.alt[!is.na(data[,grouping[1]]) & data[, grouping[1]] == y], ...))


  } else { # repeat for svobj...
    
    anyNA = any(is.na(svobj$variables[,grouping[1]]))
    if (anyNA) {
      NAss = theil.decomposition(type = type, alpha=alpha,
                                response = response,
                                w.var = w.var,
                                svobj = subset(svobj, is.na(svobj$variables[,grouping[1]])),
                                grouping = grouping[-1],
                                na.rm = na.rm,
                                compute.MC=compute.MC, compute.BJ=compute.BJ, 
                                w.alt = w.alt[is.na(svobj$variables[,grouping[1]])], ...)
    }

    grp.levels = levels(svobj$variables[,grouping[1]])
    TTlist = lapply(grp.levels, 
      function(y) theil.decomposition(type = type, alpha=alpha,  
                                      response = response,
                                      w.var = w.var,
                                      svobj = subset(svobj, !is.na(svobj$variables[,grouping[1]]) & (svobj$variables[,grouping[1]] == y)),
                                      grouping = grouping[-1],
                                      na.rm = na.rm,
                                      compute.MC=compute.MC, compute.BJ=compute.BJ, 
                                      w.alt = w.alt[!is.na(svobj$variables[,grouping[1]]) & (svobj$variables[, grouping[1]] == y)], ...))


  }

  names(TTlist) = grp.levels

  ## append NA category if present:
  
  if (anyNA) {
    TTlist[[length(grp.levels) + 1]] = NAss
    names(TTlist)[length(grp.levels) + 1] = "<NA>"
  }
  
  return(TTlist)

} # theil.decomposition

#################################################################################################################

format.print.svymeans = function(a, b, n.n, a.na, d, dd=NULL, ddd=NULL, dddd=NULL, ddddd=NULL, level.names, digits, print.diff=TRUE, n.suppress=30) {

  n.n = n.n[n.n[,"n"] > 0,]

  cat("\n - Group means, (design-based) std errors, and sample (n) and weighted (N) sizes:\n\n")
  names(a) = c("mean", "se")
  row.names(b) = c("ALL")
  if (!is.null(a.na)) {
    row.names(a.na) = c("<NA>")
    out.tab = t(cbind(t(b), t(a), t(a.na)))
  } else {
    out.tab = t(cbind(t(b), t(a)))
  }

  if (n.suppress) { ## suppress groups with sample size < n.suppress from printout:
    excl.names = NULL
    if (any(n.n[, "n"] < n.suppress)) {
      out.tab[n.n[,"n"] < n.suppress,] = NA
      cat("\t Means/SEs for groups with sample size n<", n.suppress, "are suppressed from this printout.\n\n")
      excl.names = row.names(n.n)[n.n[,"n"] < n.suppress]  ## used below.
    }
  }

  print(cbind(round(out.tab, digits), n.n))

  if (print.diff) {
    cat("\n - Differences in group means.\n\n")
    if (n.suppress & any(is.element(level.names, excl.names))) {
      cat("\n\t Print method for differences in means not yet implemented for non-NA categories that are supressed due to n<", n.suppress,".\n")
      return()
    }
    if (length(level.names) == 2) {
      d = cbind(t(round(d[-1,-4], digits)), d[-1,4])
      colnames(d)[4] = "Pr(>|t|)"
      rownames(d) =  paste(level.names[2], level.names[1], sep=" - ")
      print(d)
    } else if (length(level.names) == 3) {
      d = cbind(apply(d[-1,-4], 2, function(x) round(x, digits)), d[-1,4])
      dd = cbind(apply(dd[-1,-4], 2, function(x) round(x, digits)), dd[-1,4])
      dd = rbind(d, dd[-1,])
      colnames(dd)[4] = "Pr(>|t|)"
      rownames(dd) = paste(level.names[c(2, 3, 3)], level.names[c(1, 1, 2)], sep=" - ")
      print(dd)
    } else if (length(level.names) == 4) {
      d = cbind(apply(d[-1,-4], 2, function(x) round(x, digits)), d[-1,4])
      dd = cbind(apply(dd[-1,-4], 2, function(x) round(x, digits)), dd[-1,4])
      ddd = cbind(apply(ddd[-1,-4], 2, function(x) round(x, digits)), ddd[-1,4])
      ddd = rbind(d, dd[-1,], ddd[-(1:2),])
      colnames(ddd)[4] = "Pr(>|t|)"
      rownames(ddd) = paste(level.names[c(2, 3, 4, 3, 4, 4)], level.names[c(1, 1, 1, 2, 2, 3)], sep=" - ")
      print(ddd)
    } else if (length(level.names) == 5) {
      d = cbind(apply(d[-1,-4], 2, function(x) round(x, digits)), d[-1,4])
      dd = cbind(apply(dd[-1,-4], 2, function(x) round(x, digits)), dd[-1,4])
      ddd = cbind(apply(ddd[-1,-4], 2, function(x) round(x, digits)), ddd[-1,4])
      dddd = cbind(apply(dddd[-1,-4], 2, function(x) round(x, digits)), dddd[-1,4])    
      dddd = rbind(d, dd[-1,], ddd[-(1:2),], dddd[-(1:3),])
      colnames(dddd)[4] = "Pr(>|t|)"
      rownames(dddd) = paste(level.names[c(2, 3, 4, 5, 3, 4, 5, 4, 5, 5)], level.names[c(1, 1, 1, 1, 2, 2, 2, 3, 3, 4)], sep=" - ")
      print(dddd)
    } else if (length(level.names) == 6) {
      d = cbind(apply(d[-1,-4], 2, function(x) round(x, digits)), d[-1,4])
      dd = cbind(apply(dd[-1,-4], 2, function(x) round(x, digits)), dd[-1,4])
      ddd = cbind(apply(ddd[-1,-4], 2, function(x) round(x, digits)), ddd[-1,4])
      dddd = cbind(apply(dddd[-1,-4], 2, function(x) round(x, digits)), dddd[-1,4])
      ddddd = cbind(apply(ddddd[-1,-4], 2, function (x) round (x, digits)), ddddd[-1,4])    
      ddddd = rbind(d, dd[-1,], ddd[-(1:2),], dddd[-(1:3),], ddddd[-(1:4),])
      colnames(ddddd)[4] = "Pr(>|t|)"
      rownames(ddddd) = paste(level.names[c(2, 3, 4, 5, 6, 3, 4, 5, 6, 4, 5, 6, 5, 6, 6)], level.names[c(1, 1, 1, 1, 1, 2, 2, 2, 2, 3, 3, 3, 4, 4, 5)], sep=" - ")
      print(dddd)
    } else {
      cat("\n\t Not yet implemented for categories with more than 6 levels.\n") 
    }
  }
  return()
} # format.print.svymeans

#####################################################################################################################

format.print.index.tbl = function(tbl, digits, n.suppress=30) {   ## print formatting for report
  tbl = tbl[tbl[,"n"] >= n.suppress,]              ## suppress comparisons to DSU group
  tbl = tbl[, -1]                                  ## remove 'n' column
  tbl = tbl[!is.element(dimnames(tbl)[[1]], c("ALL", "<NA>")), ] 
  
  rtbl = NULL

  if (!is.null(nrow(tbl))) {
    if (nrow(tbl) > 1) {
      diffnames = function(x, y) {
                    paste(x, y, sep=" - ")
                  }
      ntbl = outer(dimnames(tbl)[[1]], dimnames(tbl)[[1]], diffnames)
      otbl = outer(tbl[,1], tbl[,1], '-')
      stbl = sqrt(outer(tbl[,2]^2, tbl[,2]^2, '+'))
      ttbl = otbl/stbl
      ptbl = 2*pnorm(abs(ttbl), lower.tail=FALSE) # two-sided p-value
      rtbl = round(cbind(otbl[lower.tri(otbl)], stbl[lower.tri(stbl)], ttbl[lower.tri(ttbl)]), digits)
      rtbl = cbind(rtbl, ptbl[lower.tri(ptbl)])
      dimnames(rtbl)[[2]] = c("Estimate", "Std. Error", "t value", "Pr(>|t|)")
      dimnames(rtbl)[[1]] = ntbl[lower.tri(ntbl)]
    }
  }
    
  return(rtbl)

} # format.print.index.tbl

##################################
# Last modified: 11 December 2012. MT.
##################################

