########################################################################
# Talih, M. (2012).
# A reference-invariant health disparity index based on Renyi divergence.
# Annals of Applied Statistics. Accepted for publication.
########################################################################

########################################################################
# Code is backward compatible with that used in:
#
# Borrell, L.N., M. Talih (2012). 
# Examining periodontal disease disparities among U.S. adults 20 years of age and older: 
# NHANES III (1988-1994) and NHANES 1999-2004. 
# Public Health Reports, 127, pp.497-506.
#
# Borrell, L.N., M. Talih (2011). 
# A Symmetrized Theil index measure of health disparities: 
# An example using dental caries in U.S. children and adolescents. 
# Statistics in Medicine, 30 (3), pp. 277-90. 
# First published online 5 November 2010. 
# DOI: 10.1002/sim.4114.
#
#########################################################################

###############################
# Last modified 11 December 2012
###############################


###############################################################
## Age group subset - See data setup file for specific grouping

if (agegrp.var == "adult") {
  age.subset = adult.subset
} else { 
  if (agegrp.var == "senior") {
    age.subset = senior.subset
  } else {
    if (agegrp.var == "alladults") {
      age.subset = (adult.subset | senior.subset)
    } else {
      if (agegrp.var == "hp2020adults") {
        age.subset = hp2020adults.subset
      } else {
        if (agegrp.var == "adults20andOver") {
          age.subset = adults20andOver.subset
        } else {
          if (agegrp.var == "adults25andOver") {
            age.subset = adults25andOver.subset
          } else {
            stop("Please expand code implementation to desired age group subset")
          }
        }
      }
    }
  }    
}

############################
## Bootstrap and BRR weights:

w.boot.sub = w.boot[age.subset & positive.weights & complete,]
w.BRR.sub = w.BRR[age.subset & positive.weights & complete,]

###########################################################
## working data subset (NH3 is default name for data.frame):

NH3.sub = NH3[age.subset & positive.weights & complete,]

####################################################################################
## Recode factors with dropped categories -- see data setup file for category labels

if (agegrp.var == "adult") {
  warning("Age group variable in data frame assumed to be labeled AGEGRP1")
  NH3.sub$AGEGRP1 = factor(as.vector(NH3.sub$AGEGRP1), levels=c("20-34", "35-49", "50-64"))
} else {
  if (agegrp.var == "senior") {
    warning("Age group variable in data frame assumed to be labeled AGEGRP1")
    NH3.sub$AGEGRP1 = factor(as.vector(NH3.sub$AGEGRP1), levels=c("65-74", "75+"))
  } else {
    if (agegrp.var == "hp2020adults") {
      warning("Age group variable in data frame assumed to be labeled AGEGRPN")
      NH3.sub$AGEGRPN = factor(as.vector(NH3.sub$AGEGRPN), levels=c("45-54", "55-64", "65-74"))
    } else {
      warning("No recoding of factors with dropped age group categories.")
    }
  }
}

################################################################
## Prevalence subsets -- see data setup file for variable labels

if (bin.var == "DYEREPORT") {
  zero.DT = zero.DYEREPORT
  positive.DT = positive.DYEREPORT
} else {
  if (bin.var == "PD") {
    zero.DT = zero.PKDMEAN
    positive.DT = positive.PKDMEAN
  } else {
    if (bin.var == "CAL") {
      zero.DT = zero.CALMEAN
      positive.DT = positive.CALMEAN
    } else {
      if (bin.var == "HP2020mod") {
        if (resp.var != "") {
          zero.DT = zero.HP2020mod & !is.na(as.vector(NH3[,resp.var])) & as.vector(NH3[,resp.var]) == 0
          positive.DT = positive.HP2020mod  & !is.na(as.vector(NH3[,resp.var])) & as.vector(NH3[,resp.var]) > 0
        } else {
          zero.DT = zero.HP2020mod
          positive.DT = positive.HP2020mod
        }
      } else {
        if (bin.var == "HP2020sev") {
          if (resp.var != "") {
            zero.DT = zero.HP2020sev & !is.na(as.vector(NH3[,resp.var])) & as.vector(NH3[,resp.var]) == 0
            positive.DT = positive.HP2020sev & !is.na(as.vector(NH3[,resp.var])) & as.vector(NH3[,resp.var]) > 0
          } else {
            zero.DT = zero.HP2020sev
            positive.DT = positive.HP2020sev
          }
        } else {
          if (bin.var == "HP2020perio") {
            if (resp.var != "") {
              zero.DT = zero.HP2020perio & !is.na(as.vector(NH3[,resp.var])) & as.vector(NH3[,resp.var]) == 0
              positive.DT = positive.HP2020perio & !is.na(as.vector(NH3[,resp.var])) & as.vector(NH3[,resp.var]) > 0
            } else {
              zero.DT = zero.HP2020perio
              positive.DT = positive.HP2020perio
            }
          } else {
            if (bin.var == "DMARETHN2020null") {
              if (resp.var != "") {
                zero.DT = zero.DMARETHN2020null & !is.na(as.vector(NH3[,resp.var])) & as.vector(NH3[,resp.var]) == 0
                positive.DT = positive.DMARETHN2020null & !is.na(as.vector(NH3[,resp.var])) & as.vector(NH3[,resp.var]) > 0
              } else {
                zero.DT = zero.DMARETHN2020null
                positive.DT = positive.DMARETHN2020null
              }
            } else {
              if (bin.var == "TC") {
                if (resp.var != "") {
                  zero.DT = zero.LBXTC
                  positive.DT = positive.LBXTC
                } else {
                  stop("Response variable and disease indicator variable do not match.")
                }
              } else {
                stop("Please expand code implementation to desired age group subset")
              }
            }
          }
        }
      }
    }
  }
}

healthy.sub = zero.DT[age.subset & positive.weights & complete]
unhealthy.sub = positive.DT[age.subset & positive.weights & complete]

################################################
## Survey design object needs a different set up:

NH3.tmp = NH3

##########################################
## Recode factors with dropped categories:

if (agegrp.var == "adult") {
  warning("Age group variable in data frame assumed to be labeled AGEGRP1")
  NH3.tmp$AGEGRP1 = factor(as.vector(NH3.tmp$AGEGRP1), levels=c("20-34", "35-49", "50-64"))
} else {
  if (agegrp.var == "senior") {
    warning("Age group variable in data frame assumed to be labeled AGEGRP1")
    NH3.tmp$AGEGRP1 = factor(as.vector(NH3.tmp$AGEGRP1), levels=c("65-74", "75+"))
  } else { 
    if (agegrp.var == "hp2020adults") {
      warning("Age group variable in data frame assumed to be labeled AGEGRPN")
      NH3.tmp$AGEGRPN = factor(as.vector(NH3.tmp$AGEGRPN), levels=c("45-54", "55-64", "65-74"))
    } else {
      warning("No recoding of factors with dropped age group categories.")
    }
  }
}

#############################
## Prevalence design objects:

NH3.tmp.str = svydesign(id=as.formula(paste("~", psu.var)), strata=as.formula(paste("~", str.var)), weights=as.formula(paste("~", wgt.var)), data=NH3.tmp, nest=TRUE)

NH3.sub.str = subset(NH3.tmp.str, subset= (age.subset & positive.weights & complete))
rm(NH3.tmp, NH3.tmp.str)

healthy.str = subset(NH3.sub.str, subset= healthy.sub)
unhealthy.str = subset(NH3.sub.str, subset= unhealthy.sub)

#################################
# Last modified 11 December 2012. MT.
#################################
