########################################################################
# Talih, M. (2012).
# A reference-invariant health disparity index based on Renyi divergence.
# Annals of Applied Statistics. Accepted for publication
########################################################################

###########################################
# Setup and code for Figure 1 and Table 1 #
###########################################

###############################
# Last modified 11 December 2012
###############################

setwd("C:/***/pubperio")  # set to desired working directory

##################################################################################################
## Alpha divergence (Generalized entropy class with distributions not necessarily summing to 1) ##
##################################################################################################

## Usual representation:

DA.q = function(p, q, a){
    if (missing(a) || length(a) == 0) {
        stop("Desired aversion parameter 'a' must be specified")
    }
    if (length(a) > 1) {
        warning("Multiple values for the aversion parameter 'a' are specified, all but the first one will be ignored")
    }
    if(!is.vector(p) || !is.vector(q)) {
        stop("First two arguments should be specified as vectors")
    }
    if(length(p) != length(q)) {
        stop("Vectors should have the same length")
    }
    if(any(p < 0) || any(q < 0)) {
        stop("Neither vector can have negative values")
    }
    r = q/p
    if (a == 1) { # MLD
        return(sum(ifelse(p>0, p*(r-1-log(r)), 0)))
    }
    if (a == 0) { # TI
        return(sum(ifelse(q>0, p*(1-r+r*log(r)), 0)))
    }
    return(1/(a*(1-a))*sum(ifelse(p>0, p*(a+(1-a)*r-r^(1-a)), 0)))
} #DA.q


## Alternative representation using ratio r=q/p:

DA.r = function(p, r, a){ 
    if (missing(a) || length(a) == 0) {
        stop("Desired aversion parameter 'a' must be specified")
    }
    if (length(a) > 1) {
        warning("Multiple values for the aversion parameter 'a' are specified, all but the first one will be ignored")
    }
    if(!is.vector(p) || !is.vector(r)) {
        stop("First two arguments should be specified as vectors")
    }
    if(length(p) != length(r)) {
        stop("Vectors should have the same length")
    }
    if(any(p < 0) || any(r < 0)) {
        stop("Neither vector can have negative values")
    }
    if (a == 1) { # MLD
        return(sum(ifelse(p>0, p*(r-1-log(r)), 0)))
    }
    if (a == 0) { # TI
        return(sum(ifelse(p>0, p*(1-r+r*log(r)), 0)))
    }
    return(1/(a*(1-a))*sum(ifelse(p>0, p*(a+(1-a)*r-r^(1-a)), 0)))
} #DA.r

############################################################################################################
## Scale-Invariant Alpha divergence (Generalized entropy class with distributions normalized to sum to 1) ##
############################################################################################################

## Usual representation:

IDA.q = function(p, q, a){
    if (missing(a) || length(a) == 0) {
        stop("Desired aversion parameter 'a' must be specified")
    }
    if (length(a) > 1) {
        warning("Multiple values for the aversion parameter 'a' are specified, all but the first one will be ignored")
    }
    if(!is.vector(p) || !is.vector(q)) {
        stop("First two arguments should be specified as vectors")
    }
    if(length(p) != length(q)) {
        stop("Vectors should have the same length")
    }
    if(any(p < 0) || any(q < 0)) {
        stop("Neither vector can have negative values")
    }
    p = p/sum(p)
    q = q/sum(q)
    r = q/p
    if (a == 1) { # MLD
        return(sum(ifelse(p>0, p*(r-1-log(r)), 0)))
    }
    if (a == 0) { # TI
        return(sum(ifelse(p>0, p*(1-r+r*log(r)), 0)))
    }
    return(1/(a*(1-a))*sum(ifelse(p>0, p*(a+(1-a)*r-r^(1-a)), 0)))
} #IDA.q


## Alternative representation using ratio r=q/p:

IDA.r = function(p, r, a){ 
    if (missing(a) || length(a) == 0) {
        stop("Desired aversion parameter 'a' must be specified")
    }
    if (length(a) > 1) {
        warning("Multiple values for the aversion parameter 'a' are specified, all but the first one will be ignored")
    }
    if(!is.vector(p) || !is.vector(r)) {
        stop("First two arguments should be specified as vectors")
    }
    if(length(p) != length(r)) {
        stop("Vectors should have the same length")
    }
    if(any(p < 0) || any(r < 0)) {
        stop("Neither vector can have negative values")
    }
    q = p*r
    p = p/sum(p)
    q = q/sum(q)
    r = q/p
    if (a == 1) { # MLD
        return(sum(ifelse(p>0, p*(r-1-log(r)), 0)))
    }
    if (a == 0) { # TI
        return(sum(ifelse(p>0, p*(1-r+r*log(r)), 0)))
    }
    return(1/(a*(1-a))*sum(ifelse(p>0, p*(a+(1-a)*r-r^(1-a)), 0)))
} #IDA.r

###########################################################
## Alpha-Gamma divergence (Generalized Renyi divergence) ##
###########################################################

## Usual representation:

AG.q = function(p, q, a){
    if (missing(a) || length(a) == 0) {
        stop("Desired aversion parameter 'a' must be specified")
    }
    if (length(a) > 1) {
        warning("Multiple values for the aversion parameter 'a' are specified, all but the first one will be ignored")
    }
    if(!is.vector(p) || !is.vector(q)) {
        stop("First two arguments should be specified as vectors")
    }
    if(length(p) != length(q)) {
        stop("Vectors should have the same length")
    }
    if(any(p < 0) || any(q < 0)) {
        stop("Neither vectors can have negative values")
    }
    if (a == 1) { # MLD (need to normalize first!)
        p = p/sum(p)
        q = q/sum(q)
        r = q/p
        return(sum(ifelse(p>0, p*(r-1-log(r)), 0)))
    }
    if (a == 0) { # TI (need to normalize first!)
        p = p/sum(p)
        q = q/sum(q)
        r = q/p
        return(sum(ifelse(p>0, p*(1-r+r*log(r)), 0)))
    }
    r = q/p
    return(1/(a*(1-a))*(a*log(sum(p)) + (1-a)*log(sum(p*r)) - log(sum(ifelse(p>0, p*r^(1-a), 0)))))
} #AG.q


## Alternative representation using ratio r=q/p:

AG.r = function(p, r, a){
    if (missing(a) || length(a) == 0) {
        stop("Desired aversion parameter 'a' must be specified")
    }
    if (length(a) > 1) {
        warning("Multiple values for the aversion parameter 'a' are specified, all but the first one will be ignored")
    }
    if(!is.vector(p) || !is.vector(r)) {
        stop("First two arguments should be specified as vectors")
    }
    if(length(p) != length(r)) {
        stop("Vectors should have the same length")
    }
    if(any(p < 0) || any(r < 0)) {
        stop("Neither vectors can have negative values")
    }
    if (a == 1) { # MLD (NB: need to normalize here)!
        q = p*r
        p = p/sum(p)
        q = q/sum(q)
        r = q/p
        return(sum(ifelse(p>0, p*(r-1-log(r)), 0)))
    }
    if (a == 0) { # TI (NB: need to normalize here)!
        q = p*r
        p = p/sum(p)
        q = q/sum(q)
        r = q/p
        return(sum(ifelse(p>0, p*(1-r+r*log(r)), 0)))
    }
    return(1/(a*(1-a))*(a*log(sum(p)) + (1-a)*log(sum(p*r)) - log(sum(ifelse(p>0, p*r^(1-a), 0)))))
} #AG.r

std.a = function(a) {
    if (a > 0) {
        return(a)
    }
    if (a <= 0) {
        return(1-a)
    }
}

std2.a = function(a) {
    if (a >= 0.5) {
        return(a)
    }
    if (a < 0.5) {
        return(1-a)
    }
}

############################
### Symmetrized indices ####
############################

SDA.q = function(p, q, a){
  DA.q(p,q,a) + DA.q(p,q,1-a)
} #SDA.q

SDA.r = function(p, r, a){
  DA.r(p,r,a) + DA.r(p,r,1-a)
} #SDA.r

SIDA.q = function(p, q, a){
  IDA.q(p,q,a) + IDA.q(p,q,1-a)
} #SDA.q

SIDA.r = function(p, r, a){
  IDA.r(p,r,a) + IDA.r(p,r,1-a)
} #SDA.r

SAG.q = function(p, q, a){
  AG.q(p,q,a) + AG.q(p,q,1-a)
} #SAG.q

SAG.r = function(p, r, a){
  AG.r(p,r,a) + AG.r(p,r,1-a)
} #SAG.r


###################################################################################
## Case Studies 3 & 5 in Harper et al. (2010; Milbank Quaterly) -- Figures 2 & 5 ##
###################################################################################

p.ew = c(0.25, 0.25, 0.25, 0.25)

# Average as reference
r1.0 = c(0.5, 0.4, 0.3, 0.1)/sum(p.ew*c(0.5, 0.4, 0.3, 0.1))
r1.1 = c(0.5, 0.3, 0.3, 0.1)/sum(p.ew*c(0.5, 0.3, 0.3, 0.1))
r1.2 = c(0.4, 0.4, 0.3, 0.1)/sum(p.ew*c(0.4, 0.4, 0.3, 0.1))
r1.3 = c(0.5, 0.4, 0.4, 0.1)/sum(p.ew*c(0.5, 0.4, 0.4, 0.1))

# Best (least adverse) rate as reference
r2.0 = c(0.5, 0.4, 0.3, 0.1)/min(c(0.5, 0.4, 0.3, 0.1))
r2.1 = c(0.5, 0.3, 0.3, 0.1)/min(c(0.5, 0.3, 0.3, 0.1))
r2.2 = c(0.4, 0.4, 0.3, 0.1)/min(c(0.4, 0.4, 0.3, 0.1))
r2.3 = c(0.5, 0.4, 0.4, 0.1)/min(c(0.5, 0.4, 0.4, 0.1))

postscript(file = "case-studies.eps", width=8.5, height=11, horizontal=F, family="Palatino", paper="letter")
par(mfrow=c(4,3))

####################################
## alpha divergence || ref = average

#AMIN = 0.01
#AMAX = 4.5
#alist = seq(AMIN, AMAX, 0.05)
#
#idisp.0 = unlist(lapply(alist, function(a) 1-exp(-std.a(a)*DA.r(p.ew, r1.0, a))))
#idisp.1 = unlist(lapply(alist, function(a) 1-exp(-std.a(a)*DA.r(p.ew, r1.1, a))))
#idisp.2 = unlist(lapply(alist, function(a) 1-exp(-std.a(a)*DA.r(p.ew, r1.2, a))))
#idisp.3 = unlist(lapply(alist, function(a) 1-exp(-std.a(a)*DA.r(p.ew, r1.3, a))))
#
#plot(alist, idisp.0, xlab="Parameter value", ylab="Standardized between-group GE", ylim=c(0, 1), type="l", col=1)
#lines(alist, idisp.1, type="l", col=2)
#lines(alist, idisp.2, type="l", col=4)
#lines(alist, idisp.3, type="l", col=6)
#abline(v=c(0,1), col="gray", lty=3)
#
#legend(3, 0.4, legend=c("Baseline", "Scenario 1", "Scenario 2", "Scenario 3"), col=c(1, 2, 4, 6), lty=c(1,1,1,1), bty="n")
#title("GE class || Reference = Pop. Average")

###########################################
## alpha divergence || ref = average (zoom) 

#AMIN = 0.01
#AMAX = 1.25
#alist = seq(AMIN, AMAX, 0.05)
#
#idisp.0 = unlist(lapply(alist, function(a) 1-exp(-std.a(a)*DA.r(p.ew, r1.0, a))))
#idisp.1 = unlist(lapply(alist, function(a) 1-exp(-std.a(a)*DA.r(p.ew, r1.1, a))))
#idisp.2 = unlist(lapply(alist, function(a) 1-exp(-std.a(a)*DA.r(p.ew, r1.2, a))))
#idisp.3 = unlist(lapply(alist, function(a) 1-exp(-std.a(a)*DA.r(p.ew, r1.3, a))))
#
#plot(alist, idisp.0, xlab="Parameter value", ylab="Standardized between-group GE", ylim=c(0, 0.2), type="l", col=1)
#lines(alist, idisp.1, type="l", col=2)
#lines(alist, idisp.2, type="l", col=4)
#lines(alist, idisp.3, type="l", col=6)
#abline(v=c(0,1), col="gray", lty=3)
#
#legend(0.6, 0.05, legend=c("Baseline", "Scenario 1", "Scenario 2", "Scenario 3"), col=c(1, 2, 4, 6), lty=c(1,1,1,1), bty="n")
#title("GE class || Reference = Pop. Average")

###########################################
## alpha divergence || ref = best-off group

#AMIN = 0.01
#AMAX = 4.5
#alist = seq(AMIN, AMAX, 0.05)
#
#idisp.0 = unlist(lapply(alist, function(a) 1-exp(-std.a(a)*DA.r(p.ew, r2.0, a))))
#idisp.1 = unlist(lapply(alist, function(a) 1-exp(-std.a(a)*DA.r(p.ew, r2.1, a))))
#idisp.2 = unlist(lapply(alist, function(a) 1-exp(-std.a(a)*DA.r(p.ew, r2.2, a))))
#idisp.3 = unlist(lapply(alist, function(a) 1-exp(-std.a(a)*DA.r(p.ew, r2.3, a))))
#
#plot(alist, idisp.0, xlab="Parameter value", ylab="Standardized between-group GE", ylim=c(0, 1), type="l", col=1)
#lines(alist, idisp.1, type="l", col=2)
#lines(alist, idisp.2, type="l", col=4)
#lines(alist, idisp.3, type="l", col=6)
#abline(v=c(0,1), col="gray", lty=3)
#
#legend(3, 0.4, legend=c("Baseline", "Scenario 1", "Scenario 2", "Scenario 3"), col=c(1, 2, 4, 6), lty=c(1,1,1,1), bty="n")
#title("GE class || Reference = Best-off group")

###########################################
## alpha divergence || ref = best-off group (zoom)

#AMIN = 0.01
#AMAX = 1.25
#alist = seq(AMIN, AMAX, 0.05)
#
#idisp.0 = unlist(lapply(alist, function(a) 1-exp(-std.a(a)*DA.r(p.ew, r2.0, a))))
#idisp.1 = unlist(lapply(alist, function(a) 1-exp(-std.a(a)*DA.r(p.ew, r2.1, a))))
#idisp.2 = unlist(lapply(alist, function(a) 1-exp(-std.a(a)*DA.r(p.ew, r2.2, a))))
#idisp.3 = unlist(lapply(alist, function(a) 1-exp(-std.a(a)*DA.r(p.ew, r2.3, a))))
#
#plot(alist, idisp.0, xlab="Parameter value", ylab="Standardized between-group GE", ylim=c(0, 0.8), type="l", col=1)
#lines(alist, idisp.1, type="l", col=2)
#lines(alist, idisp.2, type="l", col=4)
#lines(alist, idisp.3, type="l", col=6)
#abline(v=c(0,1), col="gray", lty=3)
#
#legend(0.6, 0.2, legend=c("Baseline", "Scenario 1", "Scenario 2", "Scenario 3"), col=c(1, 2, 4, 6), lty=c(1,1,1,1), bty="n")
#title("GE class || Reference = Best-off group")


###################################
## scale-invariant alpha divergence
#
#AMIN = 0.5
#AMAX = 5.5
#alist = seq(AMIN, AMAX, 0.05)
#
#idisp.0 = unlist(lapply(alist, function(a) 1-exp(-std.a(a)*IDA.r(p.ew, r2.0, a))))
#idisp.1 = unlist(lapply(alist, function(a) 1-exp(-std.a(a)*IDA.r(p.ew, r2.1, a))))
#idisp.2 = unlist(lapply(alist, function(a) 1-exp(-std.a(a)*IDA.r(p.ew, r2.2, a))))
#idisp.3 = unlist(lapply(alist, function(a) 1-exp(-std.a(a)*IDA.r(p.ew, r2.3, a))))
#
#plot(alist, idisp.0, xlab="Parameter value", ylab="Standardized between-group GE", ylim=c(0, 1), type="l", col=1)
#lines(alist, idisp.1, type="l", col=2)
#lines(alist, idisp.2, type="l", col=4)
#lines(alist, idisp.3, type="l", col=6)
#abline(v=c(0,1), col="gray", lty=3)
#
#legend(1, 1, legend=c("Baseline", "Scenario 1", "Scenario 2", "Scenario 3"), col=c(1, 2, 4, 6), lty=c(1,1,1,1), bty="n")
#title("GE class || Reference-invariant")

###################################
## Symmetrized scale-invariant alpha divergence

AMIN = 0.5
AMAX = 5.5
alist = seq(AMIN, AMAX, 0.05)

idisp.0 = unlist(lapply(alist, function(a) 1-exp(-std2.a(a)*SIDA.r(p.ew, r2.0, a))))
idisp.1 = unlist(lapply(alist, function(a) 1-exp(-std2.a(a)*SIDA.r(p.ew, r2.1, a))))
idisp.2 = unlist(lapply(alist, function(a) 1-exp(-std2.a(a)*SIDA.r(p.ew, r2.2, a))))
idisp.3 = unlist(lapply(alist, function(a) 1-exp(-std2.a(a)*SIDA.r(p.ew, r2.3, a))))

plot(alist, 100*idisp.0, xlab="Parameter value", ylab="Standardized between-group index (percent)", ylim=c(0, 100), type="l", col=1, axes=F, bty="n")
lines(alist, 100*idisp.1, type="l", col=2)
lines(alist, 100*idisp.2, type="l", col=4)
lines(alist, 100*idisp.3, type="l", col=6)
#abline(v=c(1), col="gray", lty=3)
axis(1, at=c(0.5, 1, 2, 3, 4, 5), labels=c(0.5, 1, 2, 3, 4, 5))
axis(2, at=seq(0,100,20), labels=seq(0, 100, 20))
legend(x="topleft", legend=c("Baseline", "Scenario 1", "Scenario 2", "Scenario 3"), col=c(1, 2, 4, 6), lty=c(1,1,1,1), bty="n")
title("Symmetrized reference-invariant GE\n Equally weighted")

########################################################
## Absolute change in index relative to baseline scenario

plot(alist, 100*(idisp.1 - idisp.0), xlab="Parameter value", ylab="Absolute change in index (p.p.)", ylim=c(-10,10), type="l", col=2, axes=F, bty="n")
lines(alist, 100*(idisp.2 - idisp.0), type="l", col=4)
lines(alist, 100*(idisp.3 - idisp.0), type="l", col=6)
abline(h=c(0), col="gray", lty=3)
axis(1, at=c(0.5, 1, 2, 3, 4, 5), labels=c(0.5, 1, 2, 3, 4, 5))
axis(2, at=seq(-10,10,5), labels=seq(-10, 10, 5))
legend(x="topleft", legend=c("Scenario 1", "Scenario 2", "Scenario 3"), col=c(2, 4, 6), lty=c(1,1,1), bty="n")
title("Symmetrized reference-invariant GE\n Equally weighted")

########################################################
## Percent change in index relative to baseline scenario

plot(alist, 100*(idisp.1 - idisp.0)/idisp.0, xlab="Parameter value", ylab="Percent change in index", ylim=c(-20,20), type="l", col=2, axes=F, bty="n")
lines(alist, 100*(idisp.2 - idisp.0)/idisp.0, type="l", col=4)
lines(alist, 100*(idisp.3 - idisp.0)/idisp.0, type="l", col=6)
abline(h=c(0), col="gray", lty=3)
axis(1, at=c(0.5, 1, 2, 3, 4, 5), labels=c(0.5, 1, 2, 3, 4, 5))
axis(2, at=seq(-20,20,10), labels=seq(-20, 20, 10))
legend(x="topleft", legend=c("Scenario 1", "Scenario 2", "Scenario 3"), col=c(2, 4, 6), lty=c(1,1,1), bty="n")
title("Symmetrized reference-invariant GE\n Equally weighted")

###################################
## Generalized Renyi divergence

#AMIN = 0.5
#AMAX = 4.5
#alist = seq(AMIN, AMAX, 0.05)
#
#idisp.0 = unlist(lapply(alist, function(a) 1-exp(-std.a(a)*AG.r(p.ew, r2.0, a))))
#idisp.1 = unlist(lapply(alist, function(a) 1-exp(-std.a(a)*AG.r(p.ew, r2.1, a))))
#idisp.2 = unlist(lapply(alist, function(a) 1-exp(-std.a(a)*AG.r(p.ew, r2.2, a))))
#idisp.3 = unlist(lapply(alist, function(a) 1-exp(-std.a(a)*AG.r(p.ew, r2.3, a))))
#
#plot(alist, idisp.0, xlab="Parameter value", ylab="Standardized between-group RI", ylim=c(0, 1), type="l", col=1)
#lines(alist, idisp.1, type="l", col=2)
#lines(alist, idisp.2, type="l", col=4)
#lines(alist, idisp.3, type="l", col=6)
#abline(v=c(0,1), col="gray", lty=3)
#
#legend(1, 1, legend=c("Baseline", "Scenario 1", "Scenario 2", "Scenario 3"), col=c(1, 2, 4, 6), lty=c(1,1,1,1), bty="n")
#title("Renyi index")

###################################
## Symmetrized Generalized Renyi divergence

AMIN = 0.5
AMAX = 5.5
alist = seq(AMIN, AMAX, 0.05)

idisp.0 = unlist(lapply(alist, function(a) 1-exp(-std2.a(a)*SAG.r(p.ew, r2.0, a))))
idisp.1 = unlist(lapply(alist, function(a) 1-exp(-std2.a(a)*SAG.r(p.ew, r2.1, a))))
idisp.2 = unlist(lapply(alist, function(a) 1-exp(-std2.a(a)*SAG.r(p.ew, r2.2, a))))
idisp.3 = unlist(lapply(alist, function(a) 1-exp(-std2.a(a)*SAG.r(p.ew, r2.3, a))))

plot(alist, 100*idisp.0, xlab="Parameter value", ylab="Standardized between-group index (percent)", ylim=c(0, 100), type="l", col=1, axes=F, bty="n")
lines(alist, 100*idisp.1, type="l", col=2)
lines(alist, 100*idisp.2, type="l", col=4)
lines(alist, 100*idisp.3, type="l", col=6)
#abline(v=c(1), col="gray", lty=3)
axis(1, at=c(0.5, 1, 2, 3, 4, 5), labels=c(0.5, 1, 2, 3, 4, 5))
axis(2, at=seq(0,100,20), labels=seq(0, 100, 20))
legend(x="topleft", legend=c("Baseline", "Scenario 1", "Scenario 2", "Scenario 3"), col=c(1, 2, 4, 6), lty=c(1,1,1,1), bty="n")
title("Symmetrized Renyi index\n Equally weighted")


########################################################
## Absolute change in index relative to baseline scenario

plot(alist, 100*(idisp.1 - idisp.0), xlab="Parameter value", ylab="Absolute change in index (p.p.)", ylim=c(-10,10), type="l", col=2, axes=F, bty="n")
lines(alist, 100*(idisp.2 - idisp.0), type="l", col=4)
lines(alist, 100*(idisp.3 - idisp.0), type="l", col=6)
abline(h=c(0), col="gray", lty=3)
axis(1, at=c(0.5, 1, 2, 3, 4, 5), labels=c(0.5, 1, 2, 3, 4, 5))
axis(2, at=seq(-10,10,5), labels=seq(-10, 10, 5))
legend(x="topleft", legend=c("Scenario 1", "Scenario 2", "Scenario 3"), col=c(2, 4, 6), lty=c(1,1,1), bty="n")
title("Symmetrized Renyi index\n Equally weighted")

########################################################
## Percent change in index relative to baseline scenario

plot(alist, 100*(idisp.1 - idisp.0)/idisp.0, xlab="Parameter value", ylab="Percent change in index", ylim=c(-20,20), type="l", col=2, axes=F, bty="n")
lines(alist, 100*(idisp.2 - idisp.0)/idisp.0, type="l", col=4)
lines(alist, 100*(idisp.3 - idisp.0)/idisp.0, type="l", col=6)
abline(h=c(0), col="gray", lty=3)
axis(1, at=c(0.5, 1, 2, 3, 4, 5), labels=c(0.5, 1, 2, 3, 4, 5))
axis(2, at=seq(-20,20,10), labels=seq(-20, 20, 10))
legend(x="topleft", legend=c("Scenario 1", "Scenario 2", "Scenario 3"), col=c(2, 4, 6), lty=c(1,1,1), bty="n")
title("Symmetrized Renyi index\n Equally weighted")

###################################################################################
## Modification of example in Harper et al (2010) using unequally weighted groups##
###################################################################################

p.ew = c(0.25, 0.25, 0.25, 0.25)
p.pw = c(0.05, 0.15, 0.15, 0.65)

# Average as reference (assuming true p = p.pw)
r1.0 = c(0.5, 0.4, 0.3, 0.1)/sum(p.pw*c(0.5, 0.4, 0.3, 0.1))
r1.1 = c(0.5, 0.3, 0.3, 0.1)/sum(p.pw*c(0.5, 0.3, 0.3, 0.1))
r1.2 = c(0.4, 0.4, 0.3, 0.1)/sum(p.pw*c(0.4, 0.4, 0.3, 0.1))
r1.3 = c(0.5, 0.4, 0.4, 0.1)/sum(p.pw*c(0.5, 0.4, 0.4, 0.1))

# Best (least adverse) rate as reference
r2.0 = c(0.5, 0.4, 0.3, 0.1)/min(c(0.5, 0.4, 0.3, 0.1))
r2.1 = c(0.5, 0.3, 0.3, 0.1)/min(c(0.5, 0.3, 0.3, 0.1))
r2.2 = c(0.4, 0.4, 0.3, 0.1)/min(c(0.4, 0.4, 0.3, 0.1))
r2.3 = c(0.5, 0.4, 0.4, 0.1)/min(c(0.5, 0.4, 0.4, 0.1))


####################################
## alpha divergence || ref = average

#AMIN = 0.01
#AMAX = 4.5
#alist = seq(AMIN, AMAX, 0.05)
#
#idisp.0 = unlist(lapply(alist, function(a) 1-exp(-std.a(a)*DA.r(p.ew, r1.0, a))))
#idisp.1 = unlist(lapply(alist, function(a) 1-exp(-std.a(a)*DA.r(p.ew, r1.1, a))))
#idisp.2 = unlist(lapply(alist, function(a) 1-exp(-std.a(a)*DA.r(p.ew, r1.2, a))))
#idisp.3 = unlist(lapply(alist, function(a) 1-exp(-std.a(a)*DA.r(p.ew, r1.3, a))))
#
#plot(alist, idisp.0, xlab="Parameter value", ylab="Standardized between-group GE", ylim=c(0, 1), type="l", col=1)
#lines(alist, idisp.1, type="l", col=2)
#lines(alist, idisp.2, type="l", col=4)
#lines(alist, idisp.3, type="l", col=6)
#abline(v=c(0,1), col="gray", lty=3)
#
#legend(3, 0.4, legend=c("Baseline", "Scenario 1", "Scenario 2", "Scenario 3"), col=c(1, 2, 4, 6), lty=c(1,1,1,1), bty="n")
#title("GE class || Reference = Pop. Average")

###########################################
## alpha divergence || ref = average (zoom) 

#AMIN = 0.01
#AMAX = 1.25
#alist = seq(AMIN, AMAX, 0.05)

#idisp.0 = unlist(lapply(alist, function(a) 1-exp(-std.a(a)*DA.r(p.ew, r1.0, a))))
#idisp.1 = unlist(lapply(alist, function(a) 1-exp(-std.a(a)*DA.r(p.ew, r1.1, a))))
#idisp.2 = unlist(lapply(alist, function(a) 1-exp(-std.a(a)*DA.r(p.ew, r1.2, a))))
#idisp.3 = unlist(lapply(alist, function(a) 1-exp(-std.a(a)*DA.r(p.ew, r1.3, a))))

#plot(alist, idisp.0, xlab="Parameter value", ylab="Standardized between-group GE", ylim=c(0, 0.2), type="l", col=1)
#lines(alist, idisp.1, type="l", col=2)
#lines(alist, idisp.2, type="l", col=4)
#lines(alist, idisp.3, type="l", col=6)
#abline(v=c(0,1), col="gray", lty=3)

#legend(0.6, 0.05, legend=c("Baseline", "Scenario 1", "Scenario 2", "Scenario 3"), col=c(1, 2, 4, 6), lty=c(1,1,1,1), bty="n")
#title("GE class || Reference = Pop. Average")

###########################################
## alpha divergence || ref = best-off group

#AMIN = 0.01
#AMAX = 4.5
#alist = seq(AMIN, AMAX, 0.05)

#idisp.0 = unlist(lapply(alist, function(a) 1-exp(-std.a(a)*DA.r(p.ew, r2.0, a))))
#idisp.1 = unlist(lapply(alist, function(a) 1-exp(-std.a(a)*DA.r(p.ew, r2.1, a))))
#idisp.2 = unlist(lapply(alist, function(a) 1-exp(-std.a(a)*DA.r(p.ew, r2.2, a))))
#idisp.3 = unlist(lapply(alist, function(a) 1-exp(-std.a(a)*DA.r(p.ew, r2.3, a))))

#plot(alist, idisp.0, xlab="Parameter value", ylab="Standardized between-group GE", ylim=c(0, 1), type="l", col=1)
#lines(alist, idisp.1, type="l", col=2)
#lines(alist, idisp.2, type="l", col=4)
#lines(alist, idisp.3, type="l", col=6)
#abline(v=c(0,1), col="gray", lty=3)

#legend(3, 0.4, legend=c("Baseline", "Scenario 1", "Scenario 2", "Scenario 3"), col=c(1, 2, 4, 6), lty=c(1,1,1,1), bty="n")
#title("GE class || Reference = Best-off group")

###########################################
## alpha divergence || ref = best-off group (zoom)

#AMIN = 0.01
#AMAX = 1.25
#alist = seq(AMIN, AMAX, 0.05)

#idisp.0 = unlist(lapply(alist, function(a) 1-exp(-std.a(a)*DA.r(p.ew, r2.0, a))))
#idisp.1 = unlist(lapply(alist, function(a) 1-exp(-std.a(a)*DA.r(p.ew, r2.1, a))))
#idisp.2 = unlist(lapply(alist, function(a) 1-exp(-std.a(a)*DA.r(p.ew, r2.2, a))))
#idisp.3 = unlist(lapply(alist, function(a) 1-exp(-std.a(a)*DA.r(p.ew, r2.3, a))))

#plot(alist, idisp.0, xlab="Parameter value", ylab="Standardized between-group GE", ylim=c(0, 0.8), type="l", col=1)
#lines(alist, idisp.1, type="l", col=2)
#lines(alist, idisp.2, type="l", col=4)
#lines(alist, idisp.3, type="l", col=6)
#abline(v=c(0,1), col="gray", lty=3)

#legend(0.6, 0.2, legend=c("Baseline", "Scenario 1", "Scenario 2", "Scenario 3"), col=c(1, 2, 4, 6), lty=c(1,1,1,1), bty="n")
#title("GE class || Reference = Best-off group")


###################################
## scale-invariant alpha divergence

#AMIN = 0.5
#AMAX = 4.5
#alist = seq(AMIN, AMAX, 0.05)
#
#idisp.0 = unlist(lapply(alist, function(a) 1-exp(-std.a(a)*IDA.r(p.ew, r2.0, a))))
#idisp.1 = unlist(lapply(alist, function(a) 1-exp(-std.a(a)*IDA.r(p.ew, r2.1, a))))
#idisp.2 = unlist(lapply(alist, function(a) 1-exp(-std.a(a)*IDA.r(p.ew, r2.2, a))))
#idisp.3 = unlist(lapply(alist, function(a) 1-exp(-std.a(a)*IDA.r(p.ew, r2.3, a))))
#
#plot(alist, idisp.0, xlab="Parameter value", ylab="Standardized between-group GE", ylim=c(0, 1), type="l", col=1)
#lines(alist, idisp.1, type="l", col=2)
#lines(alist, idisp.2, type="l", col=4)
#lines(alist, idisp.3, type="l", col=6)
#abline(v=c(0,1), col="gray", lty=3)
#
#legend(1, 1, legend=c("Baseline", "Scenario 1", "Scenario 2", "Scenario 3"), col=c(1, 2, 4, 6), lty=c(1,1,1,1), bty="n")
#title("GE class || Reference-invariant")

###################################
## Symmetrized scale-invariant alpha divergence

AMIN = 0.5
AMAX = 5.5
alist = seq(AMIN, AMAX, 0.05)

idisp.0 = unlist(lapply(alist, function(a) 1-exp(-std2.a(a)*SIDA.r(p.pw, r2.0, a))))
idisp.1 = unlist(lapply(alist, function(a) 1-exp(-std2.a(a)*SIDA.r(p.pw, r2.1, a))))
idisp.2 = unlist(lapply(alist, function(a) 1-exp(-std2.a(a)*SIDA.r(p.pw, r2.2, a))))
idisp.3 = unlist(lapply(alist, function(a) 1-exp(-std2.a(a)*SIDA.r(p.pw, r2.3, a))))

plot(alist, idisp.0, xlab="Parameter value", ylab="Standardized between-group index", ylim=c(0, 1), type="l", col=1, axes=F, bty="n")
lines(alist, idisp.1, type="l", col=2)
lines(alist, idisp.2, type="l", col=4)
lines(alist, idisp.3, type="l", col=6)
#abline(v=c(1), col="gray", lty=3)
axis(1, at=c(0.5, 1, 2, 3, 4, 5), labels=c(0.5, 1, 2, 3, 4, 5))
axis(2, at=seq(0,1,0.2), labels=seq(0, 100, 20))
legend(x="topleft", legend=c("Baseline", "Scenario 1", "Scenario 2", "Scenario 3"), col=c(1, 2, 4, 6), lty=c(1,1,1,1), bty="n")
title("Symmetrized reference-invariant GE\n Population weighted")

########################################################
## Absolute change in index relative to baseline scenario

plot(alist, 100*(idisp.1 - idisp.0), xlab="Parameter value", ylab="Absolute change in index (p.p.)", ylim=c(-10,10), type="l", col=2, axes=F, bty="n")
lines(alist, 100*(idisp.2 - idisp.0), type="l", col=4)
lines(alist, 100*(idisp.3 - idisp.0), type="l", col=6)
abline(h=c(0), col="gray", lty=3)
axis(1, at=c(0.5, 1, 2, 3, 4, 5), labels=c(0.5, 1, 2, 3, 4, 5))
axis(2, at=seq(-10,10,5), labels=seq(-10, 10, 5))
legend(x="topleft", legend=c("Scenario 1", "Scenario 2", "Scenario 3"), col=c(2, 4, 6), lty=c(1,1,1), bty="n")
title("Symmetrized reference-invariant GE\n Population weighted")

########################################################
## Percent change in index relative to baseline scenario

plot(alist, 100*(idisp.1 - idisp.0)/idisp.0, xlab="Parameter value", ylab="Percent change in index", ylim=c(-20,20), type="l", col=2, axes=F, bty="n")
lines(alist, 100*(idisp.2 - idisp.0)/idisp.0, type="l", col=4)
lines(alist, 100*(idisp.3 - idisp.0)/idisp.0, type="l", col=6)
abline(h=c(0), col="gray", lty=3)
axis(1, at=c(0.5, 1, 2, 3, 4, 5), labels=c(0.5, 1, 2, 3, 4, 5))
axis(2, at=seq(-20,20,10), labels=seq(-20, 20, 10))
legend(x="topleft", legend=c("Scenario 1", "Scenario 2", "Scenario 3"), col=c(2, 4, 6), lty=c(1,1,1), bty="n")
title("Symmetrized reference-invariant GE\n Population weighted")

###################################
## Generalized Renyi divergence

#AMIN = 0.5
#AMAX = 4.5
#alist = seq(AMIN, AMAX, 0.05)

#idisp.0 = unlist(lapply(alist, function(a) 1-exp(-std.a(a)*AG.r(p.ew, r2.0, a))))
#idisp.1 = unlist(lapply(alist, function(a) 1-exp(-std.a(a)*AG.r(p.ew, r2.1, a))))
#idisp.2 = unlist(lapply(alist, function(a) 1-exp(-std.a(a)*AG.r(p.ew, r2.2, a))))
#idisp.3 = unlist(lapply(alist, function(a) 1-exp(-std.a(a)*AG.r(p.ew, r2.3, a))))
#
#plot(alist, idisp.0, xlab="Parameter value", ylab="Standardized between-group RI", ylim=c(0, 1), type="l", col=1)
#lines(alist, idisp.1, type="l", col=2)
#lines(alist, idisp.2, type="l", col=4)
#lines(alist, idisp.3, type="l", col=6)
#abline(v=c(0,1), col="gray", lty=3)
#
#legend(1, 1, legend=c("Baseline", "Scenario 1", "Scenario 2", "Scenario 3"), col=c(1, 2, 4, 6), lty=c(1,1,1,1), bty="n")
#title("Renyi index")

###################################
## Symmetrized Generalized Renyi divergence

AMIN = 0.5
AMAX = 5.5
alist = seq(AMIN, AMAX, 0.05)

idisp.0 = unlist(lapply(alist, function(a) 1-exp(-std2.a(a)*SAG.r(p.pw, r2.0, a))))
idisp.1 = unlist(lapply(alist, function(a) 1-exp(-std2.a(a)*SAG.r(p.pw, r2.1, a))))
idisp.2 = unlist(lapply(alist, function(a) 1-exp(-std2.a(a)*SAG.r(p.pw, r2.2, a))))
idisp.3 = unlist(lapply(alist, function(a) 1-exp(-std2.a(a)*SAG.r(p.pw, r2.3, a))))

plot(alist, idisp.0, xlab="Parameter value", ylab="Standardized between-group index", ylim=c(0, 1), type="l", col=1, axes=F, bty="n")
lines(alist, idisp.1, type="l", col=2)
lines(alist, idisp.2, type="l", col=4)
lines(alist, idisp.3, type="l", col=6)
#abline(v=c(1), col="gray", lty=3)

axis(1, at=c(0.5, 1, 2, 3, 4, 5), labels=c(0.5, 1, 2, 3, 4, 5))
axis(2, at=seq(0,1, 0.2), labels=seq(0, 100, 20))
legend(x="topleft", legend=c("Baseline", "Scenario 1", "Scenario 2", "Scenario 3"), col=c(1, 2, 4, 6), lty=c(1,1,1,1), bty="n")
title("Symmetrized Renyi index\n Population weighted")

########################################################
## Absolute change in index relative to baseline scenario

plot(alist, 100*(idisp.1 - idisp.0), xlab="Parameter value", ylab="Absolute change in index (p.p.)", ylim=c(-10,10), type="l", col=2, axes=F, bty="n")
lines(alist, 100*(idisp.2 - idisp.0), type="l", col=4)
lines(alist, 100*(idisp.3 - idisp.0), type="l", col=6)
abline(h=c(0), col="gray", lty=3)
axis(1, at=c(0.5, 1, 2, 3, 4, 5), labels=c(0.5, 1, 2, 3, 4, 5))
axis(2, at=seq(-10,10,5), labels=seq(-10, 10, 5))
legend(x="topleft", legend=c("Scenario 1", "Scenario 2", "Scenario 3"), col=c(2, 4, 6), lty=c(1,1,1), bty="n")
title("Symmetrized Renyi index\n Population weighted")

########################################################
## Percent change in index relative to baseline scenario

plot(alist, 100*(idisp.1 - idisp.0)/idisp.0, xlab="Parameter value", ylab="Percent change in index", ylim=c(-20,20), type="l", col=2, axes=F, bty="n")
lines(alist, 100*(idisp.2 - idisp.0)/idisp.0, type="l", col=4)
lines(alist, 100*(idisp.3 - idisp.0)/idisp.0, type="l", col=6)
abline(h=c(0), col="gray", lty=3)
axis(1, at=c(0.5, 1, 2, 3, 4, 5), labels=c(0.5, 1, 2, 3, 4, 5))
axis(2, at=seq(-20,20,10), labels=seq(-20, 20, 10))
legend(x="topleft", legend=c("Scenario 1", "Scenario 2", "Scenario 3"), col=c(2, 4, 6), lty=c(1,1,1), bty="n")
title("Symmetrized Renyi index\n Population weighted")


dev.off()

