########################################################################
# Talih, M. (2012).
# A reference-invariant health disparity index based on Renyi divergence.
# Annals of Applied Statistics. Accepted for publication.
########################################################################

########################################################################
# Code is backward compatible with that used in:
#
# Borrell, L.N., M. Talih (2012). 
# Examining periodontal disease disparities among U.S. adults 20 years of age and older: 
# NHANES III (1988-1994) and NHANES 1999-2004. 
# Public Health Reports, 127, pp.497-506.
#
# Borrell, L.N., M. Talih (2011). 
# A Symmetrized Theil index measure of health disparities: 
# An example using dental caries in U.S. children and adolescents. 
# Statistics in Medicine, 30 (3), pp. 277-90. 
# First published online 5 November 2010. 
# DOI: 10.1002/sim.4114.
#
#########################################################################

###############################
# Last modified 11 December 2012
###############################

############################################################################
## Read in exported SAS file, tab-delimited, use SEQN as (unique) row names:

NH3 = read.table(paste(r03.PATH, "Exported SAS data/NH0104_SUB_rev.txt", sep=""), sep="\t", header=T)
row.names(NH3) = NH3$SEQN

## Make smaller dataset for easier manipulation:

NH3 = NH3[, c("DMARETHN", "DMARETHN2020", "HSSEX", "HSAGEIR", "SDPPSU6", "SDPSTRA6", "WTPFEX6", "NEDUC", "NPIR", "US_BORN", "AGEGRPN", "HP2020perio")]

###########################################################################
## Refer back to SOCIODEMOGRAPHIC NH0104 ***.sas file for category labels:

NH3$DMARETHN[as.vector(NH3$DMARETHN) == 5] = 4
NH3$DMARETHN = factor(as.vector(NH3$DMARETHN), levels=1:4, labels=c("Non-Hispanic White", "Non-Hispanic Black", "Mexican American", "Other"))
NH3$DMARETHN2020 = factor(as.vector(NH3$DMARETHN2020), levels=1:3, labels=c("Non-Hispanic White", "Non-Hispanic Black", "Mexican American"))
NH3$HSSEX = factor(as.vector(NH3$HSSEX), levels=1:2, labels=c("Male", "Female"))
NH3$AGEGRPN = factor(as.vector(NH3$AGEGRPN), levels=1:6, labels=c("20-34", "35-44", "45-54", "55-64", "65-74", "75+"))
NH3$NEDUC = factor(as.vector(NH3$NEDUC), levels=1:4, labels=c("< High school", "High school graduate", "Some College or AA degree", "College graduate or above"))
NH3$NPIR = factor(as.vector(NH3$NPIR), levels=1:5, labels=c("<100", "100-199", "200-399", "400-499", "500+"))
NH3$US_BORN = factor(as.vector(NH3$US_BORN), levels=1:2, labels=c("U.S.", "Outside U.S."))
NH3$HP2020perio = factor(as.vector(NH3$HP2020perio), levels=c(1,0), labels=c("Yes", "No"))

###########################
## Sample design parameters

wgt.var = "WTPFEX6"
str.var = "SDPSTRA6"
psu.var = "SDPPSU6"

#################################################################################
## Subsets definitions (Please see and modify "common/theil44-setup.R" as needed)

positive.weights = !is.na(NH3[,wgt.var]) & NH3[,wgt.var] > 0
complete = (vector("logical", nrow(NH3)) == 0)                 # set all entries to TRUE (not used)

zero.HP2020perio =  !is.na(as.vector(NH3$HP2020perio)) & as.vector(NH3$HP2020perio) == "No"
positive.HP2020perio = !is.na(as.vector(NH3$HP2020perio)) & as.vector(NH3$HP2020perio) == "Yes"

## following HP2020 objective OH-5 target population:

hp2020adults.subset = !is.na(NH3$HSAGEIR) & NH3$HSAGEIR >= 45 & NH3$HSAGEIR <= 74

#########################################################################
## Survey design object with native stratification:

NH3.str = svydesign(id=as.formula(paste("~", psu.var)), strata=as.formula(paste("~", str.var)), weights=as.formula(paste("~", wgt.var)), data=NH3, nest=TRUE)

##########################################################################

###################################################################################
## Create dummy variable for sampling distribution under the null of `no disparities'

NH3$DMARETHN2020null = as.vector(NH3$HP2020perio)
zero.DMARETHN2020null =  !is.na(NH3$DMARETHN2020null) & NH3$DMARETHN2020null == "No"
positive.DMARETHN2020null = !is.na(NH3$DMARETHN2020null) & NH3$DMARETHN2020null == "Yes"

denom = sum(NH3[positive.weights & hp2020adults.subset & !is.na(as.vector(NH3$HP2020perio)) & !is.na(NH3$DMARETHN2020), wgt.var])
sum(NH3[positive.weights & hp2020adults.subset & !is.na(as.vector(NH3$HP2020perio)) & !is.na(NH3$DMARETHN2020) & (NH3$DMARETHN2020 == "Non-Hispanic White"), wgt.var])/denom
# [1] 0.8490989
sum(NH3[positive.weights & hp2020adults.subset & !is.na(as.vector(NH3$HP2020perio)) & !is.na(NH3$DMARETHN2020) & (NH3$DMARETHN2020 == "Non-Hispanic Black"), wgt.var])/denom
# [1] 0.09867985
sum(NH3[positive.weights & hp2020adults.subset & !is.na(as.vector(NH3$HP2020perio)) & !is.na(NH3$DMARETHN2020) & (NH3$DMARETHN2020 == "Mexican American"), wgt.var])/denom
# [1] 0.05222121


## match Mexican American rate....
denom = sum(NH3[positive.weights & hp2020adults.subset & positive.HP2020perio & !is.na(NH3$DMARETHN2020), wgt.var])
cumsum(NH3[positive.weights & hp2020adults.subset & positive.HP2020perio & !is.na(NH3$DMARETHN2020) & (NH3$DMARETHN2020 == "Mexican American"), wgt.var])/denom

NH3$DMARETHN2020null[positive.weights & hp2020adults.subset & positive.HP2020perio & !is.na(NH3$DMARETHN2020) & (NH3$DMARETHN2020 == "Mexican American")][-c(1:77)] = "No"
zero.DMARETHN2020null =  !is.na(NH3$DMARETHN2020null) & NH3$DMARETHN2020null == "No"
positive.DMARETHN2020null = !is.na(NH3$DMARETHN2020null) & NH3$DMARETHN2020null == "Yes"

denom = sum(NH3[positive.weights & hp2020adults.subset & positive.DMARETHN2020null & !is.na(NH3$DMARETHN2020), wgt.var])
cumsum(NH3[positive.weights & hp2020adults.subset & positive.DMARETHN2020null & !is.na(NH3$DMARETHN2020) & (NH3$DMARETHN2020 == "Mexican American"), wgt.var])/denom
cumsum(NH3[positive.weights & hp2020adults.subset & positive.DMARETHN2020null & !is.na(NH3$DMARETHN2020) & (NH3$DMARETHN2020 == "Non-Hispanic Black"), wgt.var])/denom
cumsum(NH3[positive.weights & hp2020adults.subset & positive.DMARETHN2020null & !is.na(NH3$DMARETHN2020) & (NH3$DMARETHN2020 == "Non-Hispanic White"), wgt.var])/denom


## match non-Hispanic black rate....
denom = sum(NH3[positive.weights & hp2020adults.subset & positive.DMARETHN2020null & !is.na(NH3$DMARETHN2020), wgt.var])
cumsum(NH3[positive.weights & hp2020adults.subset & positive.DMARETHN2020null & !is.na(NH3$DMARETHN2020) & (NH3$DMARETHN2020 == "Non-Hispanic Black"), wgt.var])/denom

NH3$DMARETHN2020null[positive.weights & hp2020adults.subset & positive.DMARETHN2020null & !is.na(NH3$DMARETHN2020) & (NH3$DMARETHN2020 == "Non-Hispanic Black")][-c(1:64)] = "No"
zero.DMARETHN2020null =  !is.na(NH3$DMARETHN2020null) & NH3$DMARETHN2020null == "No"
positive.DMARETHN2020null = !is.na(NH3$DMARETHN2020null) & NH3$DMARETHN2020null == "Yes"

denom = sum(NH3[positive.weights & hp2020adults.subset & positive.DMARETHN2020null & !is.na(NH3$DMARETHN2020), wgt.var])
cumsum(NH3[positive.weights & hp2020adults.subset & positive.DMARETHN2020null & !is.na(NH3$DMARETHN2020) & (NH3$DMARETHN2020 == "Mexican American"), wgt.var])/denom
cumsum(NH3[positive.weights & hp2020adults.subset & positive.DMARETHN2020null & !is.na(NH3$DMARETHN2020) & (NH3$DMARETHN2020 == "Non-Hispanic Black"), wgt.var])/denom
cumsum(NH3[positive.weights & hp2020adults.subset & positive.DMARETHN2020null & !is.na(NH3$DMARETHN2020) & (NH3$DMARETHN2020 == "Non-Hispanic White"), wgt.var])/denom

NH3$DMARETHN2020null = factor(NH3$DMARETHN2020null, levels=c("Yes", "No"))

### OK. Work with this. Decent approximation to q = p.
############################################################


########################
## BRR implementation ##
########################

## Traditional BRR with Fay's adjustment.
## Uses Hadamard matrix to give orthogonal components instead of (failing in) sampling 2^49 possible sequences!

NH3.repstr = as.svrepdesign(NH3.str, type="BRR", large="merge", fay.rho=FAY.RHO, compress=FALSE)
w.BRR = NH3.repstr$pweight * NH3.repstr$repweights
rm(NH3.repstr)

# Fay, R.E. (1989). Theoretical Application of Weighting for Variance Calculation.
# Proceedings of the Section on Survey Research Methods of the American Statistical Association
# pp. 212-217.

###########################################
#### Rescaled bootstrap implementation ####
###########################################

### Generating bootstrap weights cf. appendix in Cheng, Han & Gansky (2008):

# Reference: Rao, J.N.K., Wu, C.F.J., and Yue, K. (1992). Some Recent Work in Resampling Methods.
# Survey Methodology, 18, 209-217.

w.boot = NULL

if (B > 1) {
  set.seed(SEED)
  w.boot = matrix(0, nrow(NH3), B, dimnames=list(row.names(NH3), 1:B))
  str.levels = unique(NH3[, str.var])
  for (b in 1:B) {
    w.boot[, b] = NH3[, wgt.var] 	# initialize to existing sampling weights
    for (str in str.levels) {
      cls.levels = unique(NH3[NH3[, str.var] == str, psu.var])
      cls.len = length(cls.levels)
      if (cls.len > 1) {
        str.cls = unique(sample(cls.levels, cls.len - 1, replace=T))
        for (cls in cls.levels) {
          if (is.element(cls, str.cls)) { # cluster is selected. Multiply bootstrap weight by cls.len/(cls.len-1).
            w.boot[NH3[, str.var] == str & NH3[, psu.var] == cls, b] = cls.len/(cls.len-1) * NH3[NH3[, str.var] == str & NH3[, psu.var] == cls, wgt.var]
          }
          else { # cluster not selected. Set bootstrap weight to zero.
            w.boot[NH3[, str.var] == str & NH3[, psu.var] == cls, b] = 0
          }
        }
      }
    }
  }
  
  # clean up:
  rm(b, str, cls, str.levels, cls.levels, str.cls, cls.len)
}

##################################
# Last modified 11 December 2012. MT
##################################


