########################################################################
# Talih, M. (2012).
# A reference-invariant health disparity index based on Renyi divergence.
# Annals of Applied Statistics. Accepted for publication.
########################################################################

########################################################################
# Code is backward compatible with that used in:
#
# Borrell, L.N., M. Talih (2012). 
# Examining periodontal disease disparities among U.S. adults 20 years of age and older: 
# NHANES III (1988-1994) and NHANES 1999-2004. 
# Public Health Reports, 127, pp.497-506.
#
# Borrell, L.N., M. Talih (2011). 
# A Symmetrized Theil index measure of health disparities: 
# An example using dental caries in U.S. children and adolescents. 
# Statistics in Medicine, 30 (3), pp. 277-90. 
# First published online 5 November 2010. 
# DOI: 10.1002/sim.4114.
#
#########################################################################

#################################
# Last modified 11 December 2012
#################################

############################################################################
## Read in exported SAS file, tab-delimited, use SEQN as (unique) row names:

NH3 = read.table(paste(r03.PATH, "Exported SAS data/NHALLPERIO.txt", sep=""), sep="\t", header=T)
NH3 = NH3[NH3$YEAR == 2,] # subset for NHANES 1999-2004 data only (NHALLPERIO also has NHANES III data)
row.names(NH3) = NH3$SEQN

## Make smaller dataset for easier manipulation:

NH3 = NH3[, c("DMARETHN", "HSSEX", "HSAGEIR", "SDPPSU6", "SDPSTRA6", "WTPFEX6", "DEPEXFLG", "DEPLAFLG", "EDUC", "INC3", "PIR", "US_BORN", "AGEGRP1", "PKDMEAN", "CALMEAN", "DYEREPORT")]

###############################################################################################
## Refer back to SOCIODEMOGRAPHIC NH9904 ***.sas file for category labels:

NH3$DMARETHN = factor(as.vector(NH3$DMARETHN), levels=1:5, labels=c("NH-White", "NH-Black", "MexAm", "OtherM", "OtherH"))
NH3$HSSEX = factor(as.vector(NH3$HSSEX), levels=1:2, labels=c("Male", "Female"))
NH3$AGEGRP1 = factor(as.vector(NH3$AGEGRP1), levels=1:5, labels=c("20-34", "35-49", "50-64", "65-74", "75+"))
NH3$EDUC = factor(as.vector(NH3$EDUC), levels=1:3, labels=c("< 12", "12", "> 12"))
NH3$INC3 = factor(as.vector(NH3$INC3), levels=1:3, labels=c("< 15K", "[15K, 25K)", "25K+"))
NH3$PIR = factor(as.vector(NH3$PIR), levels=1:3, labels=c("Lo", "Med", "Hi"))
NH3$US_BORN = factor(as.vector(NH3$US_BORN), levels=1:2, labels=c("US-Born", "Foreign"))
NH3$DEPEXFLG = factor(as.vector(NH3$DEPEXFLG), levels=1:2, labels=c("Yes", "No"))
NH3$DEPLAFLG = factor(as.vector(NH3$DEPLAFLG), levels=1:2, labels=c("Yes", "No"))
NH3$DYEREPORT = factor(as.vector(NH3$DYEREPORT), levels=c(1,0), labels=c("Yes", "No"))

#### additional indicator variables (needed for compatibility with R code in Borrell & Talih (2010))
NH3$PD = factor((NH3$PKDMEAN > 0) + 0, levels=c(1,0), labels=c("Yes", "No"))
NH3$CAL = factor((NH3$CALMEAN > 0) + 0, levels=c(1,0), labels=c("Yes", "No"))

###########################
## Sample design parameters

wgt.var = "WTPFEX6"
str.var = "SDPSTRA6"
psu.var = "SDPPSU6"

#################################################################################
## Subsets definitions (Please see and modify "common/theil44-setup.R" as needed)

positive.weights = !is.na(NH3[,wgt.var]) & NH3[,wgt.var] > 0

complete.dental = !is.na(NH3$DEPEXFLG) & NH3$DEPEXFLG == "Yes"
complete.perio = !is.na(NH3$DEPLAFLG) & NH3$DEPLAFLG == "Yes"
complete = complete.dental & complete.perio

zero.CALMEAN =  !is.na(NH3$CALMEAN) & NH3$CALMEAN == 0
positive.CALMEAN = !is.na(NH3$CALMEAN) & NH3$CALMEAN > 0
zero.PKDMEAN =  !is.na(NH3$PKDMEAN) & NH3$PKDMEAN == 0
positive.PKDMEAN = !is.na(NH3$PKDMEAN) & NH3$PKDMEAN > 0
zero.DYEREPORT =  !is.na(as.vector(NH3$DYEREPORT)) & as.vector(NH3$DYEREPORT) == 0
positive.DYEREPORT = !is.na(as.vector(NH3$DYEREPORT)) & as.vector(NH3$DYEREPORT) > 0

# following Dye, Tan and Smith (2007):

adult.subset = !is.na(NH3$HSAGEIR) & NH3$HSAGEIR >= 20 & NH3$HSAGEIR <= 64
senior.subset = !is.na(NH3$HSAGEIR) & NH3$HSAGEIR >= 65

#########################################################################
## Survey design object with native stratification:

NH3.str = svydesign(id=as.formula(paste("~", psu.var)), strata=as.formula(paste("~", str.var)), weights=as.formula(paste("~", wgt.var)), data=NH3, nest=TRUE)

##########################################################################

########################
## BRR implementation ##
########################

## Traditional BRR with Fay's adjustment.
## Uses Hadamard matrix to give orthogonal components instead of (failing in) sampling 2^49 possible sequences!

NH3.repstr = as.svrepdesign(NH3.str, type="BRR", large="merge", fay.rho=FAY.RHO, compress=FALSE)
w.BRR = NH3.repstr$pweight * NH3.repstr$repweights
rm(NH3.repstr)

# Fay, R.E. (1989). Theoretical Application of Weighting for Variance Calculation.
# Proceedings of the Section on Survey Research Methods of the American Statistical Association
# pp. 212-217.

###########################################
#### Rescaled bootstrap implementation ####
###########################################

### Generating bootstrap weights cf. appendix in Cheng, Han & Gansky (2008):

# Reference: Rao, J.N.K., Wu, C.F.J., and Yue, K. (1992). Some Recent Work in Resampling Methods.
# Survey Methodology, 18, 209-217.

w.boot = NULL

if (B > 1) {
  set.seed(SEED)
  w.boot = matrix(0, nrow(NH3), B, dimnames=list(row.names(NH3), 1:B))
  str.levels = unique(NH3[, str.var])
  for (b in 1:B) {
    w.boot[, b] = NH3[, wgt.var] 	# initialize to existing sampling weights
    for (str in str.levels) {
      cls.levels = unique(NH3[NH3[, str.var] == str, psu.var])
      cls.len = length(cls.levels)
      if (cls.len > 1) {
        str.cls = unique(sample(cls.levels, cls.len - 1, replace=T))
        for (cls in cls.levels) {
          if (is.element(cls, str.cls)) { # cluster is selected. Multiply bootstrap weight by cls.len/(cls.len-1).
            w.boot[NH3[, str.var] == str & NH3[, psu.var] == cls, b] = cls.len/(cls.len-1) * NH3[NH3[, str.var] == str & NH3[, psu.var] == cls, wgt.var]
          }
          else { # cluster not selected. Set bootstrap weight to zero.
            w.boot[NH3[, str.var] == str & NH3[, psu.var] == cls, b] = 0
          }
        }
      }
    }
  }
  
  # clean up:
  rm(b, str, cls, str.levels, cls.levels, str.cls, cls.len)
}

#################################
# Last modified 11 December 2012. MT.
#################################


